#include "ace/Log_Msg.h"
#include "ace/Local_Tokens.h"
#include "ace/Thread.h"
#include "ace/Token_Manager.h"

#if !defined (__INLINE__)
#include "ace/Local_Tokens.i"
#endif /* __INLINE__ */

ACE_Tokens::ACE_Tokens (void)
: reference_count_ (0)
{
  ACE_TRACE ("ACE_Tokens::ACE_Tokens");
}

ACE_TPQ_Entry::ACE_TPQ_Entry (const ACE_Token_Proxy *new_proxy, 
			      void *type,
			      const char *client_id)
     // this typecast is safe.  just do it
: proxy_ ((ACE_Token_Proxy *) new_proxy),
  nesting_level_ (0),
  arg_ (0),
  sleep_hook_ (0),
  type_ (type),
  next_ (0)
{
  ACE_TRACE ("ACE_TPQ_Entry::ACE_TPQ_Entry");

  if (client_id != 0)
    {
      ACE_OS::strcpy (client_id_, client_id);
    }
  else
    {
      char node_name[MAXHOSTNAMELEN];
      ACE_OS::hostname (node_name, sizeof node_name);

      // Make sure name isn't too long...  we're adding nodename, \0, 16
      // for the pid and thr_id, and 3 for the slashes (/).
      if ((ACE_OS::strlen (node_name) + 1 + 16 + 3) > ACE_TOKEN_CONST::MAXCLIENTIDLEN)
	{
	  // truncate the string
	  node_name[ACE_TOKEN_CONST::MAXCLIENTIDLEN - (1 + 16 + 3)] = '/0';
	  errno = ENAMETOOLONG;
	  ACE_ERROR ((LM_ERROR, "name too long\n"));
	}

      ACE_OS::sprintf (client_id_, "/%s/%d/%d", node_name,
		       ACE_OS::getpid (), ACE_Thread::self ());
    }
}

ACE_TPQ_Entry::ACE_TPQ_Entry (void)
: proxy_ (0),
  nesting_level_ (0),
  arg_ (0),
  sleep_hook_ (0)
{
  ACE_TRACE ("ACE_TPQ_Entry::ACE_TPQ_Entry null const.");
}

ACE_TPQ_Entry::ACE_TPQ_Entry (const ACE_TPQ_Entry &rhs)
{
  ACE_TRACE ("ACE_TPQ_Entry::ACE_TPQ_Entry copy const.");
  *this = rhs;
}

ACE_TPQ_Entry::~ACE_TPQ_Entry (void)
{
  ACE_TRACE ("ACE_TPQ_Entry::~ACE_TPQ_Entry");
}

void
ACE_TPQ_Entry::set (void *arg,
		    void (*sleep_hook) (void *))
{
  ACE_TRACE ("ACE_TPQ_Entry::set");
  arg_ = arg;
  sleep_hook_ = sleep_hook;
}

void
ACE_TPQ_Entry::operator= (const ACE_TPQ_Entry& rhs)
{
  ACE_TRACE ("ACE_TPQ_Entry::operator=");
  if (&rhs == this)
    return;
  proxy_ = rhs.proxy ();
  nesting_level_ = rhs.nesting_level ();
  arg_ = rhs.arg ();
  this->client_id (rhs.client_id ());
  sleep_hook_ = rhs.sleep_hook ();
}

void
ACE_TPQ_Entry::client_id (const char *id)
{
  ACE_TRACE ("ACE_TPQ_Entry::client_id");

  int n = ACE_OS::strlen (id) + 1;

  if (n > ACE_TOKEN_CONST::MAXCLIENTIDLEN)
    n = ACE_TOKEN_CONST::MAXCLIENTIDLEN;

  ACE_OS::strncpy (client_id_, id, n);
}

// ************************************************************

ACE_TSS_TPQ_Entry::ACE_TSS_TPQ_Entry (const ACE_Token_Proxy *proxy,
				      void *type,
				      const char *client_id)
: proxy_ (proxy),
  type_ (type),
  client_id_ (client_id)
{
  ACE_TRACE ("ACE_TSS_TPQ_Entry::ACE_TSS_TPQ_Entry");
}

ACE_TPQ_Entry *
ACE_TSS_TPQ_Entry::make_TSS_TYPE (void) const
{
  ACE_TRACE ("ACE_TSS_TPQ_Entry::make_TSS_TYPE");
  return new ACE_TPQ_Entry (proxy_, type_, client_id_);
}

ACE_TSS_TPQ_Entry::operator ACE_TPQ_Entry * (void)
{
  ACE_TRACE ("ACE_TSS_TPQ_Entry::operator");
  return ACE_TSS<ACE_TPQ_Entry>::operator ACE_TPQ_Entry *();
}

// ************************************************************

ACE_Token_Proxy_Queue::ACE_Token_Proxy_Queue (void)
: head_ (0),
  tail_ (0),
  size_ (0)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::ACE_Token_Proxy_Queue");
}

void
ACE_Token_Proxy_Queue::enqueue (ACE_TPQ_Entry *tpq,
				int position)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::enqueue");
  tpq->next_ = 0;

  ++size_;

  // stick at front of list
  if (head_ == 0)
    {
      head_ = tail_ = tpq;
      return;
    }
  // stick at back of list
  if (position == -1)
    {
      tail_->next_ = tpq;
      tail_ = tpq;
      return;
    }

  // walk through list to insertion point
  ACE_TPQ_Entry *temp = head_;

  for (int x = position; x >= 0; --x)
    {
      // end of queue?
      if (temp->next_ == 0)
	break;
      // advance pointer
      else
	temp = temp->next_;
    }

  // insert new tpq after temp
  tpq->next_ = temp->next_;
  temp->next_ = tpq;
}

void
ACE_Token_Proxy_Queue::dequeue (void)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::dequeue");

  if (head_ == 0)
    return;
  
  head_ = head_->next_;

  --size_;

  if (head_ == 0)
    assert (size_ == 0);
}

int
ACE_Token_Proxy_Queue::member (const char *id)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::member");

  for (ACE_TPQ_Entry *temp = head_; temp != 0; temp = temp->next_)
    if (ACE_OS::strcmp (temp->client_id (), id) == 0)
      // we found it!
      return 1;

  // we didn't find it :-(
  return 0;
}

int
ACE_Token_Proxy_Queue::is_owning_reader (const ACE_Token_Proxy *caller)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::is_owning_reader");

  for (ACE_TPQ_Entry *temp = head_; temp != 0; temp = temp->next_)
    {
      if (temp->type () != (void *) caller->type ())
	// not the same type, return false.
	return 0;
      if (ACE_OS::strcmp (temp->client_id (), caller->client_id ()) == 0)
	// we found it!
	return 1;
    }

  // we didn't find it, return false.
  return 0;
}

void
ACE_Token_Proxy_Queue::remove (const ACE_TPQ_Entry *remove_me)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::remove");
  // sanity
  if ((remove_me == 0) || (head_ == 0))
    return;

  // is it the head?
  if (head_ == remove_me) // pointer comparison.
    {
      head_ = head_->next_;
      if (head_ == 0)
	tail_ = 0;
      --size_;
      return;
    }

  ACE_TPQ_Entry *temp = head_;
  ACE_TPQ_Entry *previous = 0;

  // is it in the middle or tail?
  while (temp != 0)
    {
      if (temp == remove_me)
	{
	  // previous should never be null since the first if
	  // conditional should always be false
	  previous->next_ = temp->next_;
	  // is it the tail?
	  if (tail_ == temp)
	    tail_ = previous;

	  --size_;
	  return;
	}

      previous = temp;
      temp = temp->next_;
    }

  // it wasn't in the list.
  return;
}

// ************************************************************
// ************************************************************
// ************************************************************

ACE_Mutex_Token::ACE_Mutex_Token (const char *name)
{
  ACE_TRACE ("ACE_Mutex_Token::ACE_Mutex_Token");

  int n = ACE_OS::strlen (name) + 1;  // + 1 for \0

  if (n > ACE_TOKEN_CONST::MAXTOKENNAMELEN)
    n = ACE_TOKEN_CONST::MAXTOKENNAMELEN;

  ACE_OS::strncpy (token_name_, name, n);
}

ACE_Mutex_Token::~ACE_Mutex_Token (void)
{
  ACE_TRACE ("ACE_Mutex_Token::~ACE_Mutex_Token");
}

int 
ACE_Mutex_Token::acquire (ACE_TPQ_Entry *caller,
			  int ignore_deadlock,
			  int notify)
{
  ACE_TRACE ("ACE_Mutex_Token::acquire");
  // We need to acquire two locks. This one to ensure that only one
  // thread uses this token at a time.
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  // This one to ensure an atomic transaction across all tokens.  Note
  // that this order is crucial too.  It's resource coloring for other
  // threads which may be calling this same token.
  ACE_TOKEN_CONST::GUARD m2 (ACE_Token_Manager::instance ()->mutex ());

  // Does _anyone_ own the token?
  if (this->owner () == 0)
    {
      // there are no waiters, so queue as the first waiter (the owner.)
      waiters_.enqueue (caller, -1);
      return 0;  // success
    }

  // Does the caller already own it?
  if (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0)
    {
      // recursive acquisition
      this->owner ()->nesting_level (1);
      return 0;  // success
    }

  // check for deadlock
  if ((!ignore_deadlock) && 
      (ACE_Token_Manager::instance ()->check_deadlock (caller->proxy ()) == 1))
    {
      errno = EDEADLK;
      return -1;
    }

  // Someone owns it. Sorry, you're getting queued up at the end of
  // the waiter queue.
  waiters_.enqueue (caller, -1);

  if (notify)
    this->owner ()->call_sleep_hook ();

  errno = EWOULDBLOCK;
  return -1;
}

int 
ACE_Mutex_Token::tryacquire (ACE_TPQ_Entry *caller)
{
  ACE_TRACE ("ACE_Mutex_Token::tryacquire");
  // We need to acquire two locks. This one to ensure that only one
  // thread uses this token at a time.
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  // This one to ensure an atomic transaction across all tokens.  Note
  // that this order is crucial too.  It's resource coloring for other
  // threads which may be calling this same token.
  ACE_TOKEN_CONST::GUARD m2 (ACE_Token_Manager::instance ()->mutex ());
  
  // Does _anyone_ own the token?
  if (this->owner () == 0)
    {
      waiters_.enqueue (caller, -1);
      return 0;  // success
    }
  // Does the caller already own it?
  else if (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0)
    {
      // recursive acquisition
      this->owner ()->nesting_level (1);
      return 0;  // success
    }
  else
    // Someone owns it.  Fail.
    {
      errno = EWOULDBLOCK;
      return -1;
    }  
}

int
ACE_Mutex_Token::renew (ACE_TPQ_Entry *caller,
			int requeue_position)
{
  ACE_TRACE ("ACE_Mutex_Token::renew");
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  
  // verify that the caller is the owner
  if ((this->owner () == 0) || 
      (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) != 0))
    {
      errno = EACCES;
      return -1;
    }

  // The caller is the owner, so check to see if there are any
  // waiters.  If not, we just keep the token.  == 1 means that there
  // is only the owner.
  if (waiters_.size () == 1)
    return 0;

  // requeue the caller
  waiters_.dequeue ();
  waiters_.enqueue (caller, requeue_position);

  // notify new owner
  if (this->owner () != 0)
    this->owner ()->proxy ()->token_acquired (this->owner ()->arg ());

  // tell the caller that the operation would block.
  errno = EWOULDBLOCK;
  return -1;
}

// Release the current holder of the token (which had
// better be the caller's thread!).

int
ACE_Mutex_Token::release (ACE_TPQ_Entry *caller)
{
  ACE_TRACE ("ACE_Mutex_Token::release");
  ACE_TOKEN_CONST::GUARD m (this->lock_);

  // does anyone own the token?
  if (this->owner () == 0)
    {
      errno = EACCES;
      return -1;
    }

  // is the caller the owner
  if (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0)
    {
      // check the nesting level
      if (this->owner ()->nesting_level () > 0)
	this->owner ()->nesting_level (-1);
      else
	{
	  waiters_.dequeue ();
	  // notify new owner
	  if (this->owner () != 0)
	    this->owner ()->proxy ()->token_acquired (this->owner ()->arg ());
	}
    }
  else
    waiters_.remove (caller);

  return 0;
}

// ************************************************************
// ************************************************************
// ************************************************************

ACE_RW_Token::ACE_RW_Token (const char *name)
{
  ACE_TRACE ("ACE_RW_Token::ACE_RW_Token");

  int n = ACE_OS::strlen (name) + 1;  // + 1 for \0

  if (n > ACE_TOKEN_CONST::MAXTOKENNAMELEN)
    n = ACE_TOKEN_CONST::MAXTOKENNAMELEN;

  ACE_OS::strncpy (token_name_, name, n);
}

ACE_RW_Token::~ACE_RW_Token (void)
{
  ACE_TRACE ("ACE_RW_Token::~ACE_RW_Token");
}

// The logic in this method is insane.  Someone figure a better way
// and send it too me!  harrison@cs.wustl.edu
int 
ACE_RW_Token::acquire (ACE_TPQ_Entry *caller,
		       int ignore_deadlock,
		       int notify)
{
  ACE_TRACE ("ACE_RW_Token::acquire");
  // We need to acquire two locks. This one to ensure that only one
  // thread uses this token at a time.
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  // This one to ensure an atomic transaction across all tokens.  Note
  // that this order is crucial too.  It's resource coloring for other
  // threads which may be calling this same token.
  ACE_TOKEN_CONST::GUARD m2 (ACE_Token_Manager::instance ()->mutex ());

  int error_number = 0;
  int return_value = 0;

  // reader

  if (caller->type () == (void *) ACE_Token_Proxy::RLOCK)
    {
      // check for recursive acquisition
      if (waiters_.is_owning_reader (caller->proxy ()) == 1)
	{
	  caller->nesting_level (1);
	  return 0;  // success
	}

      // are there any writers?
      if (num_writers_ == 0)
	goto SUCCESS;
      else
	goto FAILURE;
    }

  // writer

  num_writers_++;

  // Does _anyone_ own the token?
  if (this->owner () == 0)
    // there are no waiters, so queue as the first waiter (the owner.)
    goto SUCCESS;

  // check for recursive acquisition
  if (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0)
    {
      this->owner ()->nesting_level (1);
      return 0;  // success
    }
  // fallthrough to FAILURE

 FAILURE:
  // check for deadlock
  if ((!ignore_deadlock) && 
      (ACE_Token_Manager::instance ()->check_deadlock (caller->proxy ()) == 1))
    {
      num_writers_--;
      errno = EDEADLK;
      return -1;
    }

  // queue the caller at the end of the queue.
  waiters_.enqueue (caller, -1);

  if (notify)
    {
      // if it's a writer, just notify it.
      if (this->owner ()->type () == (void *) ACE_Token_Proxy::WLOCK)
	this->owner ()->call_sleep_hook ();
      else
	{
	  // call back all reader owners.
	  ACE_TPQ_Entry *temp = this->owner ();
	  do
	    {
	      temp->call_sleep_hook ();
	      temp = temp->next_;
	    }
	  while ((temp != 0) && 
		 (temp->type () == (void *) ACE_Token_Proxy::RLOCK));
	}
    }

  errno = EWOULDBLOCK;
  return -1;

 SUCCESS:

  // queue the caller at the end of the queue.
  waiters_.enqueue (caller, -1);

  return 0;
}


int 
ACE_RW_Token::tryacquire (ACE_TPQ_Entry *caller)
{
  ACE_TRACE ("ACE_RW_Token::tryacquire");
  // We need to acquire two locks. This one to ensure that only one
  // thread uses this token at a time.
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  // This one to ensure an atomic transaction across all tokens.  Note
  // that this order is crucial too.  It's resource coloring for other
  // threads which may be calling this same token.
  ACE_TOKEN_CONST::GUARD m2 (ACE_Token_Manager::instance ()->mutex ());
  
  // reader reader reader reader reader reader reader

  if (caller->type () == (void*) ACE_Token_Proxy::RLOCK)
    {
      // check for recursive acquisition
      if (waiters_.is_owning_reader (caller->proxy ()) == 1)
	{
	  caller->nesting_level (1);
	  return 0;  // success
	}

      // are there any writers?
      if (num_writers_ == 0)
	goto SUCCESS;
      else
	{
	  errno = EWOULDBLOCK;
	  return -1;
	}
    }

  // writer writer writer writer writer writer writer

  // Does _anyone_ own the token?
  if (this->owner () == 0)
    {
      // there are no waiters, so queue as the first waiter (the owner.)
      num_writers_ = 1;
      goto SUCCESS;
    }

  // check for recursive acquisition
  if (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0)
    {
      caller->nesting_level (1);
      return 0;  // success
    }
  else
    {
      errno = EWOULDBLOCK;
      return -1;
    }

 SUCCESS:
  // queue the caller at the end of the queue.
  waiters_.enqueue (caller, -1);

  return 0;
}

int
ACE_RW_Token::renew (ACE_TPQ_Entry *caller,
		     int requeue_position)
{
  ACE_TRACE ("ACE_RW_Token::renew");
  ACE_TOKEN_CONST::GUARD m (this->lock_);
  
  // verify that the caller is the owner
  if (caller->type () == (void *) ACE_Token_Proxy::RLOCK)
    {
      if (waiters_.is_owning_reader (caller->proxy ()) == 1)
	{
	  errno = EACCES;
	  return -1;
	}
    }
  else
    {
      // is this writer the owner
      if ((this->owner () == 0) ||
	  (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) != 0))
	{
	  errno = EACCES;
	  return -1;
	}
    }

  // The caller is the owner, so check to see if there are any
  // waiters.  If not, we just keep the token.
  if (waiters_.size () == 1)
    return 0;

  // remove the caller
  waiters_.remove (caller);

  // requeue the caller.
  waiters_.enqueue (caller, requeue_position);

  if (caller->type () == (void *) ACE_Token_Proxy::RLOCK)
    {
      // if the requeued reader is now an owner, return success.
      if (waiters_.is_owning_reader (caller->proxy ()) == 1)
	return 0; // success
      // else fallthrough and return would block.
    }
  else // writer
    {
      // requeue the caller. The caller now exists in the waiter list twice.
      waiters_.enqueue (caller, requeue_position);

      // get a new owner
      this->notify_new_owner (caller);
    }

  // tell the caller that the operation would block.
  errno = EWOULDBLOCK;
  return -1;
}

int
ACE_RW_Token::release (ACE_TPQ_Entry *caller)
{
  ACE_TRACE ("ACE_RW_Token::release");
  ACE_TOKEN_CONST::GUARD m (this->lock_);

  int is_owner = 0;

  // verify that the caller is the owner
  if (caller->type () == (void *) ACE_Token_Proxy::RLOCK)
    {
      if (waiters_.is_owning_reader (caller->proxy ()) == 1)
	is_owner = 1;
      else
	ACE_DEBUG ((LM_DEBUG, "(%t) caller is not owner\n"));
    }
  else
    {
      if ((this->owner () != 0) && 
	  (ACE_OS::strcmp (caller->client_id (), this->owner ()->client_id ()) == 0))
	is_owner = 1;
    }

  if (is_owner)
    // The caller is an owner.
    {
      // check the nesting level
      if (caller->nesting_level () > 0)
	caller->nesting_level (-1);
      else
	{
	  // remove the caller and notify the new owner(s).
	  waiters_.remove (caller);
	  if (caller->type () == (void *) ACE_Token_Proxy::WLOCK)
	    num_writers_--;

	  this->notify_new_owner (caller);
	}
    }
  else
    waiters_.remove (caller);

  return 0;
}

void
ACE_RW_Token::notify_new_owner (ACE_TPQ_Entry *old_owner)
{
  ACE_TRACE ("ACE_RW_Token::new_owner");

  if (this->owner () == 0)
    return;

  if (this->owner ()->type () == (void *) ACE_Token_Proxy::RLOCK)
    {
      if (old_owner->type () == (void *) ACE_Token_Proxy::RLOCK)
	// the owners already know that they're owners
	return;

      // The current owner is a reader and the previous owner was a
      // writer, so notify all waiting readers up to the first writer.
      // call back all reader owners.
      ACE_TPQ_Entry *temp = this->owner ();
      do
	{
	  temp->proxy ()->token_acquired (temp->arg ());
	  temp = temp->next_;
	}
      while ((temp != 0) && 
	     (temp->type () == (void *) ACE_Token_Proxy::RLOCK));
    }

  else // writer
    this->owner ()->proxy ()->token_acquired (this->owner ()->arg ());
}


// ************************************************************
// ************************************************************
// ************************************************************
// 7..

ACE_Token_Proxy::ACE_Token_Proxy (ACE_Token_Proxy::PROXY_TYPE proxy_type,
				  ACE_Tokens::TOKEN_TYPE token_type)
: proxy_type_ (proxy_type),
  mutex_ (0),
  token_name_ (0, token_type),
  waiter_ (this, (void *) proxy_type, 0)
{
  ACE_TRACE ("ACE_Token_Proxy::ACE_Token_Proxy");
}

ACE_Token_Proxy::ACE_Token_Proxy (const char *token_name,
				  int ignore_deadlock,
				  int debug,
				  ACE_Token_Proxy::PROXY_TYPE proxy_type,
				  ACE_Tokens::TOKEN_TYPE token_type)
: proxy_type_ (proxy_type),
  mutex_ (0),
  token_name_ (0, token_type),
  waiter_ (this, (void *) proxy_type, 0)
{
  ACE_TRACE ("ACE_Token_Proxy::ACE_Token_Proxy2");
  this->open (token_name, ignore_deadlock, debug);
}

ACE_Token_Proxy::ACE_Token_Proxy (const ACE_Token_Proxy &m)
: proxy_type_ (m.proxy_type_),
  mutex_ (0),
  token_name_ (m.token_name_),
  waiter_ (this, (void *) m.proxy_type_, m.client_id ())
{
  ACE_TRACE ("ACE_Local_Mutex::ACE_Local_Mutex");
  this->open (m.name (),
	      m.ignore_deadlock_, m.debug_);
}

ACE_Token_Proxy *
ACE_Token_Proxy::clone (void)
{
  ACE_TRACE ("ACE_Token_Proxy::clone");
  return new ACE_Token_Proxy (*this);
}

// @@ should I do a mutex_->release ()?
ACE_Token_Proxy::~ACE_Token_Proxy (void)
{
  ACE_TRACE ("ACE_Local_Mutex::~ACE_Local_Mutex");

  if (mutex_ != 0)
    // notify token manager that we are done with it so it can
    // free it if necessary
    ACE_Token_Manager::instance ()->release_mutex (mutex_);
}

ACE_Token_Proxy::PROXY_TYPE
ACE_Token_Proxy::type (void) const
{
  ACE_TRACE ("ACE_Local_Mutex::type");
  return proxy_type_;
}

int 
ACE_Token_Proxy::open (const char *token_name,
		       int ignore_deadlock,
		       int debug)
{
  ACE_TRACE ("ACE_Token_Proxy::open");

  // store some parameters
  ignore_deadlock_ = ignore_deadlock;
  debug_ = debug;

  // used in case a name was not specified.
  char name[BUFSIZ];

  // we must have a name
  if (token_name == 0)
    {
      ACE_OS::sprintf (name, "token %d", this);
      token_name = name;
    }

  token_name_.name (token_name);

  // get or create the underlying mutex
  ACE_Token_Manager::instance ()->get_mutex (token_name_, mutex_);

  // check for failed get or failed new.
  if (mutex_ == 0)
    {
      errno = ENOMEM;
      ACE_ERROR_RETURN ((LM_ERROR, "Can't allocate mutex"), -1);
    }

  return 0;
}

int 
ACE_Token_Proxy::acquire (int notify,
			  void (*sleep_hook)(void *),
			  ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Proxy::acquire");
  if (mutex_ == 0)
    {
      errno = ENOENT;
      ACE_ERROR_RETURN ((LM_ERROR, "Not open.\n"), -1);
    }

  // this is used in handle_options, if needed.
  ACE_TOKEN_CONST::MUTEX lock;
  ACE_TOKEN_CONST::COND_VAR cond_var (lock);

  // make sure no one calls our token_acquired until we have a chance
  // to sleep first!  If after we call an EWOULDBLOCK
  // mutex_->acquire() below, but before we enter handle_options to
  // wait on the cond_var, a thread tries to give take us off the
  // waiter queue and signal us, IT WILL FIRST HAVE TO ACQUIRE THIS
  // cond_var.mutex ().  _This_ is why we acquire it.
  cond_var.mutex ().acquire ();

  waiter_->set ((void *) &cond_var, sleep_hook);

  if (mutex_->acquire (waiter_, ignore_deadlock_, notify) == -1)
    // acquire failed
    {
      switch (errno)
	{
	case EWOULDBLOCK :
	  if (debug_)
	    ACE_DEBUG ((LM_DEBUG, "(%t) waiting for %s, owner is %s\n", 
			this->name (),
			mutex_->owner_id ()));

	  // no error, but would block,
	  // if error, return error (-1), otherwise, return whether we
	  // called the holder or not.
	  return this->handle_options (options, cond_var) == -1 
	    ? -1 : notify == 1;
	case EDEADLK :
	  if (ignore_deadlock_)
	    return this->handle_options (options, cond_var) == -1
	      ? -1 : notify == 1;
	  else
	    {
	      cond_var.mutex ().release ();
	      ACE_RETURN(-1);
	    }
	default :
	  cond_var.mutex ().release ();
	  ACE_ERROR_RETURN ((LM_ERROR, 
			     "%p\n","Local_Mutex acquire."), -1);
	  break;
	}
    }
  else
    // we have the token
    {
      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) acquired %s\n",
		    this->name ()));
      cond_var.mutex ().release ();
      return 0;
    }
}

int 
ACE_Token_Proxy::tryacquire (void (*sleep_hook)(void *))
{
  ACE_TRACE ("ACE_Token_Proxy::tryacquire");
  if (mutex_ == 0)
    {
      errno = ENOENT;
      ACE_ERROR_RETURN ((LM_ERROR, "Not open.\n"), -1);
    }

  waiter_->sleep_hook (sleep_hook);

  return mutex_->tryacquire (waiter_);
}

int 
ACE_Token_Proxy::renew (int requeue_position,
			ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Proxy::renew");
  if (mutex_ == 0)
    {
      errno = ENOENT;
      ACE_ERROR_RETURN ((LM_ERROR, "Not open.\n"), -1);
    }

  // this is used in handle_options, if needed.
  ACE_TOKEN_CONST::MUTEX lock;
  ACE_TOKEN_CONST::COND_VAR cond_var (lock);

  // make sure no one calls our token_acquired until we have a chance
  // to sleep first!
  cond_var.mutex ().acquire ();

  waiter_->arg ((void *) &cond_var);

  if (mutex_->renew (waiter_, requeue_position) == -1)
    {
      // check for error
      if (errno != EWOULDBLOCK)
	ACE_ERROR_RETURN ((LM_ERROR, 
			   "Local_Mutex renew failed."), -1);
      
      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) renew blocking for %s, owner is %s\n", 
		    this->name (),
		    mutex_->owner_id ()));

      // no error, but would block, so block or return
      return this->handle_options (options, cond_var);
    }
  else
    // we have the token
    {
      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) renewed %s\n", 
		    this->name ()));
      cond_var.mutex ().release ();
      return 0;
    }
}

int
ACE_Token_Proxy::handle_options (ACE_Synch_Options &options,
				 ACE_TOKEN_CONST::COND_VAR &cv)
{
  ACE_TRACE ("ACE_Token_Proxy::handle_options");
  // some operation failed

  if (options[ACE_Synch_Options::USE_REACTOR] == 1)
    // asynchronous
    {
      // if (options[ACE_Synch_Options::USE_TIMEOUT] == 1)
      // ACE_ERROR_RETURN ((LM_ERROR, "Timeouts not yet supported" "
      // with asynchronous operations."), -1);
      cv.mutex ().release ();
      ACE_RETURN(-1);
    }
  else
    // synchronous
    {
      // block on condition variable
      while (cv.wait ((ACE_Time_Value*) options.time_value ()) == -1)
	{
	  // Note, this should obey whatever thread-specific
	  // interrupt policy is currently in place...
	  if (errno == EINTR)
	    continue;
	  // We come here if a timeout occurs or some serious
	  // ACE_Condition object error.
	  // @@ do we have the mutex here?
	  cv.mutex ().release ();
	  ACE_ERROR_RETURN ((LM_ERROR, "condition variable wait"
			     " bombed."), -1);
	}

      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) unblocking.\n", 
		    this->client_id ()));
      return 0;       // operation succeeded
    }
}

int 
ACE_Token_Proxy::release (ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Proxy::release");
  if (mutex_ == 0)
    {
      errno = ENOENT;
      ACE_ERROR_RETURN ((LM_ERROR,
			 "Must open before releasing ACE_Token_Proxy.\n"), -1);
    }

  if (mutex_->release (waiter_) != 0)
    // release failed
    ACE_ERROR_RETURN ((LM_ERROR, "(%t) release failed.\n"), -1);
  else
    {
      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) released %s, owner is %s\n", 
		    this->name (),
		    mutex_->owner_id ()));

      return 0;
    }
}

int 
ACE_Token_Proxy::remove (ACE_Synch_Options &)
{
  ACE_TRACE ("ACE_Token_Proxy::remove");
  return 0;
}

void 
ACE_Token_Proxy::sleep_hook (void)
{
  ACE_TRACE ("ACE_Token_Proxy::sleep_hook");
  // Somebody wants our token!  (Let'em wait...)
  return;
}

void 
ACE_Token_Proxy::token_acquired (void *arg)
{
  ACE_TRACE ("ACE_Token_Proxy::token_acquired");
  ACE_TOKEN_CONST::COND_VAR *cv = (ACE_TOKEN_CONST::COND_VAR *) arg;
  cv->mutex ().acquire ();
  cv->mutex ().release ();
  // We've been taken off the waiters list and given the token!
  // This implementation signals the internal condition
  // variable. Thus, if asynchronous acquires are used, this must be
  // overriden to do something more useful!
  cv->signal ();

  return;
}

// ************************************************************
// ************************************************************
// ************************************************************

#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
#if !defined (ACE_MT_SAFE) && 
 template class ACE_Null_Condition <ACE_Null_Mutex>;
 template class ACE_Guard <ACE_Null_Mutex>;
#endif /* ACE_MT_SAFE */
 template class ACE_TSS <ACE_TPQ_Entry>;
 template class ACE_Token_Name <ACE_Tokens::TOKEN_TYPE>;
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */
