/* -*- C++ -*- */

INLINE int
ACE_Tokens::dec_reference (void)
{
  ACE_TRACE ("ACE_Tokens::dec_reference");
  if (reference_count_ == 0)
    {
      ACE_DEBUG ((LM_DEBUG, "dec_reference already zero"));
      return 0;
    }
    
  return --reference_count_;
}

INLINE void
ACE_Tokens::inc_reference (void)
{
  ACE_TRACE ("ACE_Tokens::inc_reference");
  ++reference_count_;
}

INLINE ACE_Token_Proxy_Queue *
ACE_Tokens::waiters ()
{
  ACE_TRACE ("ACE_Tokens::waiters");
  return &waiters_;
}

INLINE int
ACE_Tokens::no_of_waiters ()
{
  ACE_TRACE ("ACE_Tokens::no_of_waiters");
  return waiters_.size ();
}

INLINE const char*
ACE_Tokens::owner_id ()
{
  ACE_TRACE ("ACE_Tokens::owner_id");
  if (this->owner () == 0)
    return "no owner";
  else
    return this->owner ()->client_id ();
}

INLINE const char*
ACE_Tokens::name ()
{
  ACE_TRACE ("ACE_Tokens::name");
  return token_name_;
}

INLINE ACE_Token_Proxy *
ACE_Tokens::current_owner (void)
{
  ACE_TRACE ("ACE_Tokens::current_owner");
//  ACE_GUARD m (this->lock_);

  if (this->owner () == 0)
    return 0;
  else
    return this->owner ()->proxy ();
}

INLINE ACE_TPQ_Entry *
ACE_Tokens::owner (void)
{
  ACE_TRACE ("ACE_Tokens::owner");
  return (ACE_TPQ_Entry *) waiters_.head ();
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE ACE_Tokens::TOKEN_TYPE
ACE_Mutex_Token::type (void) const
{
  ACE_TRACE ("ACE_Mutex_Token::type");
  return ACE_Tokens::MUTEX;
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE ACE_Tokens::TOKEN_TYPE
ACE_RW_Token::type (void) const
{
  ACE_TRACE ("ACE_RW_Token::type");
  return ACE_Tokens::RWLOCK;
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE const ACE_TPQ_Entry *
ACE_Token_Proxy_Queue::head (void)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::head");
  if (head_ == 0)
    return 0;
  else
    return head_;
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE ACE_Token_Proxy *
ACE_TPQ_Entry::proxy (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::proxy");
  return proxy_;
}

INLINE void
ACE_TPQ_Entry::proxy (ACE_Token_Proxy *proxy)
{
  ACE_TRACE ("ACE_TPQ_Entry::proxy");
  proxy_ = proxy;
}

INLINE int
ACE_TPQ_Entry::nesting_level (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::nesting_level");
  return nesting_level_;
}

INLINE void
ACE_TPQ_Entry::nesting_level (int delta)
{
  ACE_TRACE ("ACE_TPQ_Entry::nesting_level");
  nesting_level_ += delta;
}

INLINE void *
ACE_TPQ_Entry::arg (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::arg");
  return arg_;
}

INLINE void
ACE_TPQ_Entry::arg (void *arg)
{
  ACE_TRACE ("ACE_TPQ_Entry::arg");
  arg_ = arg;
}

INLINE const char *
ACE_TPQ_Entry::client_id (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::client_id");
  return client_id_;
}

INLINE ACE_TPQ_Entry::PTVF
ACE_TPQ_Entry::sleep_hook (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::sleep_hook");
  return sleep_hook_;
}

INLINE void
ACE_TPQ_Entry::sleep_hook (void (*sh)(void *))
{
  ACE_TRACE ("ACE_TPQ_Entry::sleep_hook");
  sleep_hook_ = sh;
}

INLINE void
ACE_TPQ_Entry::call_sleep_hook (void)
{
  ACE_TRACE ("ACE_TPQ_Entry::call_sleep_hook");

  // if a function has been registered, call it.
  if (this->sleep_hook () != 0)
    this->sleep_hook () (this->arg ());
  else
    // otherwise, call back the sleep_hook method
    this->proxy ()->sleep_hook ();
}

INLINE void
ACE_TPQ_Entry::type (void *t)
{
  ACE_TRACE ("ACE_TPQ_Entry::type");
  type_ = t;
}

INLINE void *
ACE_TPQ_Entry::type (void) const
{
  ACE_TRACE ("ACE_TPQ_Entry::type");
  return type_;
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE int
ACE_Token_Proxy_Queue::size (void)
{
  ACE_TRACE ("ACE_Token_Proxy_Queue::size");
  return size_;
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE const char*
ACE_Token_Proxy::client_id (void) const
{
  ACE_TRACE ("ACE_Token_Proxy::client_id");
  // thread-specific
  return waiter_->client_id ();
}

INLINE void
ACE_Token_Proxy::client_id (const char *client_id)
{
  ACE_TRACE ("ACE_Token_Proxy::client_id");
  waiter_->client_id (client_id);
}

INLINE const char*
ACE_Token_Proxy::owner_id ()
{
  ACE_TRACE ("ACE_Token_Proxy::owner_id");
  return mutex_->owner_id ();
}

INLINE const char*
ACE_Token_Proxy::name () const
{
  ACE_TRACE ("ACE_Token_Proxy::owner_id");
  return mutex_->name ();
}

// ************************************************************
// ************************************************************
// ************************************************************

INLINE
ACE_Local_Mutex::ACE_Local_Mutex (void)
: ACE_Token_Proxy (ACE_Token_Proxy::MUTEX,
		   ACE_Tokens::MUTEX)
{
  ACE_TRACE ("ACE_Local_Mutex::ACE_Local_Mutex");
}

INLINE
ACE_Local_Mutex::ACE_Local_Mutex (const char *token_name,
				  int ignore_deadlock,
				  int debug)
: ACE_Token_Proxy (token_name, ignore_deadlock,
		   debug, ACE_Token_Proxy::MUTEX,
		   ACE_Tokens::MUTEX)
{
  ACE_TRACE ("ACE_Local_Mutex::ACE_Local_Mutex");
}

// ************************************************************
// ************************************************************
// ************************************************************


INLINE
ACE_Local_RLock::ACE_Local_RLock (void)
: ACE_Token_Proxy (ACE_Token_Proxy::RLOCK,
		   ACE_Tokens::RWLOCK)
{
  ACE_TRACE ("ACE_Local_RLock::ACE_Local_RLock");
}

INLINE
ACE_Local_RLock::ACE_Local_RLock (const char *token_name,
				  int ignore_deadlock,
				  int debug)
: ACE_Token_Proxy (token_name, ignore_deadlock,
		   debug, ACE_Token_Proxy::RLOCK,
		   ACE_Tokens::RWLOCK)
{
  ACE_TRACE ("ACE_Local_RLock::ACE_Local_RLock");
}

// ************************************************************
// ************************************************************
// ************************************************************


INLINE
ACE_Local_WLock::ACE_Local_WLock (void)
: ACE_Token_Proxy (ACE_Token_Proxy::WLOCK,
		   ACE_Tokens::RWLOCK)
{
  ACE_TRACE ("ACE_Local_WLock::ACE_Local_WLock");
}

INLINE
ACE_Local_WLock::ACE_Local_WLock (const char *token_name,
				  int ignore_deadlock,
				  int debug)
: ACE_Token_Proxy (token_name, ignore_deadlock,
		   debug, ACE_Token_Proxy::WLOCK,
		   ACE_Tokens::RWLOCK)
{
  ACE_TRACE ("ACE_Local_WLock::ACE_Local_WLock");
}
