/* Provides a useful variable-length argument error handling
   abstraction. */

#include "ace/Log_Record.h"

ACE_ALLOC_HOOK_DEFINE(ACE_Log_Record)

void
ACE_Log_Record::dump (void) const
{
  ACE_TRACE ("ACE_Log_Record::dump");
}

void 
ACE_Log_Record::msg_data (char *data)
{
  ACE_TRACE ("ACE_Log_Record::msg_data");
  ACE_OS::strncpy (this->msg_data_, data, ACE_Log_Record::MAXLOGMSGLEN);
  this->round_up ();
}

ACE_Log_Record::ACE_Log_Record (ACE_Log_Priority lp, 
				long ts, 
				long p)
  : type_ (long (lp)), 
    time_stamp_ (ts), 
    pid_ (p) 
{
  ACE_TRACE ("ACE_Log_Record::ACE_Log_Record");
}

void
ACE_Log_Record::round_up (void)
{
  ACE_TRACE ("ACE_Log_Record::round_up");
  // Determine the length of the payload.
  int len = (sizeof *this - MAXLOGMSGLEN) + ACE_OS::strlen (this->msg_data_);

  // Round up to the alignment.
  this->length_ = (len + ACE_Log_Record::ALIGN_WORDB - 1) 
    & ~(ACE_Log_Record::ALIGN_WORDB - 1);
}

ACE_Log_Record::ACE_Log_Record (void)
{
  ACE_TRACE ("ACE_Log_Record::ACE_Log_Record");
}

/* Print out the record on the stderr stream with the appropriate
   format. */

int
ACE_Log_Record::print (const char host_name[], 
		       int verbose, 
		       FILE *fp)
{
  ACE_TRACE ("ACE_Log_Record::print");
  if (verbose)
    {
      time_t now  = this->time_stamp_;
      char *ctp = ACE_OS::ctime (&now);

      /* 01234567890123456789012345 */
      /* Wed Oct 18 14:25:36 1989n0 */

      ctp[24] = '\0';

      return ACE_OS::fprintf (fp, "%s@%s@%d@%d@%s", 
			ctp + 4, host_name, this->pid_,
			this->type_, this->msg_data_);
    }
  else
    return ACE_OS::fprintf (fp, "%s", this->msg_data_);
}

ostream &
operator << (ostream &s, ACE_Log_Record &r)
{
  ACE_TRACE ("operator <<");
#if 0
  if (verbose)
    {
      time_t now  = r.time_stamp_;
      char *ctp = ACE_OS::ctime (&now);

      /* 01234567890123456789012345 */
      /* Wed Oct 18 14:25:36 1989n0 */

      ctp[24] = '\0';

      s << ctp + 4 
	<< "@<localhost>@"
	<< r.pid_
	<< '@'
	<< r.type_
	<< '@'
	<< r.msg_data_;
    }
  else
#endif
  s << r.msg_data_;
  return s;
}

