// Defines the member functions for the memory mapping facility.

#include "ace/Mem_Map.h"

#if !defined (__INLINE__)
#include "ace/Mem_Map.i"
#endif /* __INLINE__ */

ACE_ALLOC_HOOK_DEFINE(ACE_Mem_Map)

void
ACE_Mem_Map::dump (void) const
{
  ACE_TRACE ("ACE_Mem_Map::dump");
}

// This function does the dirty work of actually calling ACE_OS::mmap
// to map the file into memory.

int 
ACE_Mem_Map::map_it (ACE_HANDLE handle, 
		     int len_request, 
		     int prot, 
		     int share, 
		     void *addr, 
		     off_t pos) 
{
  ACE_TRACE ("ACE_Mem_Map::map_it");
  this->base_addr_ = addr;
  this->handle_	   = handle;

  long l = ACE_OS::filesize (this->handle_);

  if (l == -1)
    return -1;

  this->length_ = l;

  if (len_request == -1)
    len_request = 0;

  if ((this->length_ == 0 && len_request > 0)
      || this->length_ < size_t (len_request))
    {
      this->length_ = len_request;

      if (ACE_OS::lseek (this->handle_, 
		   len_request > 0 ? len_request - 1 : 0, 
		   SEEK_SET) == -1
	  || ACE_OS::write (this->handle_, "", 1) == -1
	  || ACE_OS::lseek (this->handle_, 0, SEEK_SET) == -1)
	return -1;
    }

  this->base_addr_ = ACE_OS::mmap (this->base_addr_, 
				   this->length_,
				   prot, share, this->handle_, 
				   off_t (ACE::round_to_pagesize (pos)));
  return this->base_addr_ == MAP_FAILED ? -1 : 0; 
}

int
ACE_Mem_Map::map (const char file_name[], 
		  int len, 
		  int flags, 
		  int mode, 
		  int prot, 
		  int share, 
		  void *addr, 
		  off_t pos)
{
  ACE_TRACE ("ACE_Mem_Map::map");
  ACE_HANDLE handle = ACE_OS::open (file_name, flags, mode);

  this->filename_ = file_name;

  if (handle == ACE_INVALID_HANDLE)
    return -1;
  else 
    return this->map_it (handle, len, prot, share, addr, pos);
}

ACE_Mem_Map::ACE_Mem_Map (void)
  : filename_ (0), 
    length_ (0),
    base_addr_ (0), 
    handle_ (ACE_INVALID_HANDLE)
{
  ACE_TRACE ("ACE_Mem_Map::ACE_Mem_Map");
}

// Map a file specified by FILE_NAME. 

ACE_Mem_Map::ACE_Mem_Map (const char file_name[], 
			  int len, 
			  int flags, 
			  int mode, 
			  int prot, 
			  int share, 
			  void *addr, 
			  off_t pos)
{
  ACE_TRACE ("ACE_Mem_Map::ACE_Mem_Map");
  if (this->map (file_name, len, flags, mode, prot, share, addr, pos) < 0)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Mem_Map::ACE_Mem_Map"));
}

// Map a file from an open file descriptor HANDLE.  This function will
// lookup the length of the file if it is not given.

ACE_Mem_Map::ACE_Mem_Map (ACE_HANDLE handle, 
			  int len, 
			  int prot, 
			  int share, 
			  void *addr, 
			  off_t pos)
  : filename_ (0)
{
  ACE_TRACE ("ACE_Mem_Map::ACE_Mem_Map");
  if (this->map (handle, len, prot, share, addr, pos) < 0)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Mem_Map::ACE_Mem_Map"));
}

// Close down and remove the file from the file system.

int
ACE_Mem_Map::remove (void)
{
  ACE_TRACE ("ACE_Mem_Map::remove");
  if (ACE_OS::close (this->handle_) == -1 
      || this->unmap () == -1
      || ACE_OS::ftruncate (this->handle_, 0) == -1)
    return -1;
  else if (this->filename_ != 0)
    return ACE_OS::unlink (this->filename_);
  else
    return 0;
}

