/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ace
// 
// = FILENAME
//    Mem_Map.h 
//
// = AUTHOR
//    Doug Schmidt 
// 
// ============================================================================

#if !defined (ACE_MEM_MAP_H)
#define ACE_MEM_MAP_H

#include "ace/ACE.h"

class ACE_Mem_Map
  // = TITLE
  //     C++ interface to the mmap(2) UNIX system call. 
{
public:
  // = Initialization methods.

  ACE_Mem_Map (void);
  // Default constructor.

  ACE_Mem_Map (ACE_HANDLE handle, 
	       int length = -1, 
	       int prot = PROT_RDWR, 
	       int share = MAP_PRIVATE, 
	       void *addr = 0, 
	       off_t pos = 0);
  // Map a file from an open file descriptor FDES.  This function will
  // lookup the length of the file if it is not given.

  ACE_Mem_Map (const char file_name[], 
	       int len = -1, 
	       int flags = O_RDWR | O_CREAT, 
	       int mode = ACE_DEFAULT_PERMS, 
	       int prot = PROT_RDWR, 
	       int share = MAP_PRIVATE, 
	       void *addr = 0, 
	       off_t pos = 0);
  // Map a file specified by <file_name>.

  int map (ACE_HANDLE handle, 
	   int length = -1, 
	   int prot = PROT_RDWR, 
	   int share = MAP_PRIVATE, 
	   void *addr = 0,
	   off_t pos = 0);
  // Map a file from an open file descriptor <fd>.  This function will
  // lookup the length of the file if it is not given.

  int map (int length = -1, 
	   int prot = PROT_RDWR, 
	   int share = MAP_PRIVATE, 
	   void *addr = 0, 
	   off_t pos = 0);
  // Remap the file associated with <this->handle_>.

  int map (const char file_name[], 
	   int len = -1, 
	   int flags = O_RDWR | O_CREAT, 
	   int mode = ACE_DEFAULT_PERMS, 
	   int prot = PROT_RDWR, 
	   int share = MAP_PRIVATE, 
	   void *addr = 0, 
	   off_t pos = 0);
  // Map a file specified by <file_name>.

  int operator () (void *&addr);
  // This operator passes back the starting address of the mapped file.

  size_t size (void) const;
  // This function returns the number of bytes currently mapped in the
  // file.

  int unmap (int len = -1);
  // Unmap the region starting at <this->base_addr_>.

  int unmap (void *addr, int len);
  // Unmap the region starting at <addr_>.

  int sync (ssize_t len = -1, int flags = MS_SYNC);
  // Sync <len> bytes of the memory region to the backing store
  // starting at <this->base_addr_>.  If <len> == -1 then sync the
  // whole region.

  int sync (void *addr, size_t len, int flags = MS_SYNC);
  // Sync <len> bytes of the memory region to the backing store
  // starting at <addr_>.

  int protect (ssize_t len = -1, int prot = PROT_RDWR);
  // Change the protection of the pages of the mapped region to <prot>
  // starting at <this->base_addr_> up to <len> bytes.  If <len> == -1
  // then change protection of all pages in the mapped region.

  int protect (void *addr, size_t len, int prot = PROT_RDWR);
  // Change the protection of the pages of the mapped region to <prot>
  // starting at <addr> up to <len> bytes.

  int remove (void);
  // Close down and remove the file from the file system.

  int advise (int behavior, int len = -1);
  // Hook into the underlying VM system.

  ACE_HANDLE handle (void) const;
  // Return the underlying <handle_>.

  void dump (void) const;
  // Dump the state of an object.

  ACE_ALLOC_HOOK_DECLARE;
  // Declare the dynamic allocation hooks.

private:
  void *base_addr_;
  // Base address of the memory-mapped file.

  const char *filename_;
  // Name of the file that is mapped.

  size_t length_;
  // Length of the file.
  
  ACE_HANDLE handle_;
  // I/O handle for the open file.

  int map_it (ACE_HANDLE handle, 
	      int len = -1, 
	      int prot = PROT_RDWR, 
	      int share = MAP_SHARED, 
	      void *addr = 0, 
	      off_t pos = 0);
  // This method does the dirty work of actually calling ::mmap to map
  // the file into memory.
};

#if defined (__INLINE__)
#include "ace/Mem_Map.i"
#endif /* __INLINE__ */

#endif /* ACE_MEM_MAP_H */
