#include "ace/Synch.h"
#include "ace/Log_Msg.h"
#include "ace/Get_Opt.h"
#include "ace/Name_Options.h"

// Static Singleton instance.
ACE_Name_Options *ACE_Name_Options::instance_ = 0;

#if defined (ACE_MT_SAFE)
// Synchronize output operations.
ACE_Mutex ACE_Name_Options::lock_;
#endif /* ACE_MT_SAFE */

ACE_Name_Options *
ACE_Name_Options::instance (void)
{
  if (ACE_Name_Options::instance_ == 0)
    {
      ACE_MT (ACE_Guard<ACE_Mutex> g (ACE_Name_Options::lock_));
      if (ACE_Name_Options::instance_ == 0)
	ACE_Name_Options::instance_ = new ACE_Name_Options;
    }
  
  return ACE_Name_Options::instance_;
}

void   
ACE_Name_Options::nameserver_port (int port)
{
  this->nameserver_port_ = port;
}

int
ACE_Name_Options::nameserver_port (void)
{
  return this->nameserver_port_;
}

void   
ACE_Name_Options::nameserver_host (char *host)
{
  this->nameserver_host_ = host;
}

char *
ACE_Name_Options::nameserver_host (void)
{
  return this->nameserver_host_;
}

char *
ACE_Name_Options::process_name (void)
{
  return this->process_name_;
}


void   
ACE_Name_Options::namespace_dir (char *dir)
{
  this->namespace_dir_ = dir;
}

char *
ACE_Name_Options::namespace_dir (void)
{
  return this->namespace_dir_;
}

int 
ACE_Name_Options::debug (void)
{
  return this->debugging_;
}

int 
ACE_Name_Options::verbose (void)
{
  return this->verbosity_;
}

ACE_Name_Options::ACE_Name_Options (void)
  : debugging_ (0),
    verbosity_ (0),
    nameserver_port_ (ACE_DEFAULT_SERVER_PORT),
    nameserver_host_ (ACE_DEFAULT_SERVER_HOST),
    namespace_dir_  (ACE_DEFAULT_NAMESPACE_DIR)
{
}

ACE_Name_Options::~ACE_Name_Options (void)
{
}

void   
ACE_Name_Options::process_name (char *pname)
{
  this->process_name_ = pname;
}

void
ACE_Name_Options::parse_args (int argc, char *argv[])
{
  ACE_LOG_MSG->open (argv[0]);
  this->process_name (argv[0]);

  ACE_Get_Opt get_opt (argc, argv, "P:p:h:l:dvT:");
  for (int c; (c = get_opt ()) != -1; )
    switch (c)
      {
      case 'd':
	this->debugging_ = 1;
	break;
      case 'h':
	this->nameserver_host (get_opt.optarg);
	break;
      case 'l':
	this->namespace_dir (get_opt.optarg);
	break;
      case 'P':
	this->process_name (get_opt.optarg);
	break;
      case 'p':
	this->nameserver_port (ACE_OS::atoi (get_opt.optarg));
	break;
      case 'T':
	if (ACE_OS::strcasecmp (get_opt.optarg, "ON") == 0)
	  ACE_Trace::start_tracing ();
	else if (ACE_OS::strcasecmp (get_opt.optarg, "OFF") == 0)
	  ACE_Trace::stop_tracing ();
	break;
      case 'v':
	this->verbosity_ = 1;
	break;
      default:
	::fprintf (stderr, "%s\n"
		   "\t[-d] (enable debugging)\n"
		   "\t[-h nameserver host]\n"
		   "\t[-l namespace directory]\n"
		   "\t[-P processname]\n"
		   "\t[-p nameserver port]\n"
		   "\t[-v] (verbose) \n",
		   argv[0]);
	::exit (1);
	/* NOTREACHED */
	break;
      }
}


