/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ace
//
// = FILENAME
//    ACE_Name_Options.h
//
// = AUTHOR
//    Gerhard Lenzer, Douglas C. Schmidt, and Prashant Jain
//
// ============================================================================

#if !defined (ACE_NAME_OPTIONS_H)
#define ACE_NAME_OPTIONS_H

#include "ace/ACE.h"
#include "ace/Synch.h"

class ACE_Name_Options
  // = TITLE
  //     Manages the options for the ACE Name_Server.
{
public:
  static ACE_Name_Options *instance (void);
  // Returns Singleton.

  ~ACE_Name_Options (void);
  void parse_args (int argc, char *argv[]);
  // Parse arguments

  void nameserver_port (int port);
  int nameserver_port (void);
  // Set/Get port number

  void nameserver_host (char *host);
  char *nameserver_host (void);
  // Set/Get host name

  void namespace_dir (char *dir);
  char *namespace_dir (void);
  // Set/Get name space directory

  void process_name (char *dir);
  char *process_name (void);
  // Set/Get process name

  void consumer_port (char *port);
  char *consumer_port (void);
  // Set/Get client port number

  int debug (void);
  // Return debug status

  int verbose (void);
  // Return verbose status

protected:
  ACE_Name_Options (void);
  // Ensure Singleton!

private:
  int debugging_;		
  // Extra debugging info

  int verbosity_;		
  // Extra verbose messages

  int nameserver_port_;	
  // Port to connect to nameserver process

  char *nameserver_host_;	
  // Hostname of nameserver

  char *namespace_dir_;	
  // Directory to hold name_bindings

  char *process_name_;         
  // Name of this process

  static ACE_Name_Options *instance_;
  // The one and only one instnace

#if defined (ACE_MT_SAFE)  
  static ACE_Mutex lock_;
  // Double-Check lock.
#endif /* ACE_MT_SAFE */
};

#endif /* ACE_NAME_OPTIONS_H */
