#include "ace/Name_Server.h"
#include "ace/Log_Msg.h"
#include "ace/INET_Addr.h"
#include "ace/Get_Opt.h"

ACE_Name_Server::ACE_Name_Server (int argc, char *argv[])
  : server_port_ (ACE_DEFAULT_SERVER_PORT), // Default server port.
    done_ (0) // Continue until <SIGINT> occurs.
{
  // Use the global Singleton reactor.
  this->reactor (ACE_Service_Config::reactor ());

  // Parse the command line arguments and set options.
  this->parse_args (argc, argv);

  // Create the server address.
  this->addr_.set (this->server_port_);
}

// Run the name server.

ACE_Name_Handler *
ACE_Name_Server::make_svc_handler (void)
{
  return new ACE_Name_Handler (this->reactor ());
}

int
ACE_Name_Server::run (void)
{
  // Set the acceptor endpoint into listen mode (use the Singleton
  // global Reactor...).
  if (this->open (this->addr_, this->reactor ()) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%n: %p on port %d\n", 
		      "acceptor::open failed", 
		      this->addr_.get_port_number ()), -1);

  // Register ourselves to receive SIGINT so we can shutdown
  // gracefully.
  else if (this->reactor ()->register_handler (SIGINT, this) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%n: %p\n", 
		      "register_handler (SIGINT)"), -1);

  // Ignore SIGPIPE so that each ACE_Name_Handler can handle this on its own.
  ACE_Sig_Action sig (ACE_SignalHandler (SIG_IGN), SIGPIPE);

  ACE_DEBUG ((LM_DEBUG, "starting up server test at port %d on handle %d\n", 
	     this->server_port_, this->acceptor ().get_handle ()));

  // Loop until we receive SIGINT.

  while (this->done_ == 0)
    this->reactor ()->handle_events ();

  ACE_DEBUG ((LM_DEBUG, "shutting down server test\n"));
  return 0;
}

// Signal the server to shutdown gracefully.

int 
ACE_Name_Server::handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
)
{
  ACE_DEBUG ((LM_DEBUG, "%S\n", signum));
  this->done_ = 1;
  return 0;
}

int
ACE_Name_Server::parse_args (int argc, char *argv[])
{
  ACE_Get_Opt get_opt (argc, argv, "p:", 1);

  for (int c; (c = get_opt ()) != -1; )
    {
      switch (c)
	{
	case 'p':
	  this->server_port_ = ACE_OS::atoi (get_opt.optarg);
	  break;
	default:
	  ACE_ERROR_RETURN ((LM_ERROR, 
			    "%n:\n[-p server-port]\n%a", 1),
			   -1);
	  break;
	}
    }
  return 0;
}

#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
template class ACE_Acceptor<ACE_Name_Handler, ACE_SOCK_Acceptor, ACE_INET_Addr>;
template class ACE_Svc_Handler<ACE_SOCK_Stream, ACE_INET_Addr, ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
template class ACE_Null_Condition<ACE_Null_Mutex>;
template class ACE_Task<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
template class ACE_Message_Queue<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
template class ACE_Module<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
template class ACE_Task_Exit<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
template class ACE_TSS<ACE_Task_Exit<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> > >;
template class ACE_Thru_Task<ACE_Null_Mutex, ACE_Null_Condition<ACE_Null_Mutex> >;
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */
