/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ACE
// 
// = FILENAME
//    ACE_Name_Server.h
//
// = DESCRIPTION 
//    Main class that integrates all the components of the ACE_Name Server.
//
// = AUTHOR
//    Gerhard Lenzer, Douglas C. Schmidt, and Prashant Jain
// 
// ============================================================================

#if !defined (ACE_NAME_SERVER_H)
#define ACE_NAME_SERVER_H

#include "ace/Acceptor.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/Name_Handler.h"

class ACE_Name_Server : public ACE_Acceptor<ACE_Name_Handler, ACE_SOCK_ACCEPTOR> 
  // = TITLE
  //   Integrate the various components that implement the ACE_Name
  //   Server.
  //
  // = DESCRIPTION
  //   The <ACE_Name_Server> listens to a port number.  When a
  //   connection arrives, the <ACE_Name_Acceptor> creates a new
  //   <ACE_Name_Handler> (one per connection).  The
  //   <ACE_Name_Handler> processes all requests from a client to
  //   bind, rebind, resolve, and unbind a name.  The
  //   <ACE_Name_Server> shuts down gracefully on <SIGINT>.
{
public:
  ACE_Name_Server (int argc, char *argv[]);
  // Initialize the test server.

  int run (void);
  // Run the test server.

private:
  virtual int handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */  
);
  // Handle SIGINT.

  ACE_Name_Handler *make_svc_handler (void);
  // Factory method that creates a new ACE_Name_Handler.

  int parse_args (int argc, char *argv[]);
  // Parse the command line and set the options.

  u_short server_port_;
  // Port number for the server.

  sig_atomic_t done_;
  // Used to shut down the server gracefully when a <SIGINT> occurs.

  ACE_INET_Addr addr_;
  // Address that the <ACE_Name_Server> uses to listen for connections.
};

#endif /* ACE_NAME_SERVER_H */
