#include "ace/Log_Msg.h"
#include "ace/Naming_Context.h"
#include "ace/Name_Options.h"
#include "ace/Local_Name_Space.h"
#include "ace/Remote_Name_Space.h"

int
ACE_Naming_Context::local (void)
{
  return ACE_OS::strcmp (this->netnameserver_host_, "localhost") == 0
    || ACE_OS::strcmp (this->netnameserver_host_, this->hostname_) == 0;
}    

int
ACE_Naming_Context::open (Context_Scope_Type scope_in) 
{
  ACE_OS::hostname (this->hostname_, sizeof this->hostname_);

  this->netnameserver_host_ = ACE_Name_Options::instance ()->nameserver_host ();
  this->netnameserver_port_ = ACE_Name_Options::instance ()->nameserver_port ();

  // Perform factory operation to select appropriate type of Name_Space subclass.

  if (scope_in == ACE_Naming_Context::NET_LOCAL && this->local () == 0)
    // Use NET_LOCAL name space, set up connection with remote server.
    this->name_space_ = new ACE_Remote_Name_Space (this->netnameserver_host_,
						   this->netnameserver_port_);
  else // Use NODE_LOCAL or PROC_LOCAL name space.
    this->name_space_ = new ACE_Local_Name_Space (scope_in);

  return 0;
}

ACE_Naming_Context::ACE_Naming_Context (void)
{
}

ACE_Naming_Context::ACE_Naming_Context (Context_Scope_Type scope_in) 
{
  // Initialize
  if (this->open (scope_in) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Naming_Context::ACE_Naming_Context"));
}

int 
ACE_Naming_Context::bind (const ACE_WString &name_in, 
			  const ACE_WString &value_in, 
			  const char *type_in)
{
  return this->name_space_->bind (name_in, value_in, type_in);
}

int 
ACE_Naming_Context::bind (const char *name_in, 
			  const char *value_in, 
			  const char *type_in)
{
  return this->bind (ACE_WString (name_in), ACE_WString (value_in), type_in);
}

int 
ACE_Naming_Context::rebind (const ACE_WString &name_in, 
			    const ACE_WString &value_in, 
			    const char *type_in)
{
  return this->name_space_->rebind (name_in, value_in, type_in);
}

int 
ACE_Naming_Context::rebind (const char *name_in, 
			    const char *value_in, 
			    const char *type_in)
{
  return rebind (ACE_WString (name_in), ACE_WString (value_in), type_in);
}

int 
ACE_Naming_Context::resolve (const ACE_WString &name_in, 
			     ACE_WString &value_out,
			     char *&type_out)
{
  return this->name_space_->resolve (name_in, value_out, type_out);
}

int 
ACE_Naming_Context::resolve (const char *name_in, 
			     ACE_WString &value_out,
			     char *&type_out)
{
  return resolve (ACE_WString (name_in), value_out, type_out);
}

int 
ACE_Naming_Context::resolve (const char *name_in, 
			     char *&value_out, 
			     char *&type_out)
{
  ACE_WString val_str;

  if (this->resolve (ACE_WString (name_in), val_str, type_out) == -1)
    return -1;

  // Note that char_rep() *allocates* the memory!  Thus, caller is
  // responsible for deleting it!
  value_out = val_str.char_rep ();

  return value_out == 0 ? -1 : 0;
}

int  
ACE_Naming_Context::unbind (const ACE_WString &name_in)
{
  return this->name_space_->unbind (name_in);
}

int 
ACE_Naming_Context::unbind (const char *name_in)
{
  return this->unbind (ACE_WString (name_in));
}

int 
ACE_Naming_Context::list_names (ACE_PWSTRING_SET &set_out, 
				const ACE_WString &pattern_in)
{
  return this->name_space_->list_names (set_out, pattern_in);
}

ACE_Naming_Context::~ACE_Naming_Context (void)
{
  delete this->name_space_;
}

void
ACE_Naming_Context::dump ()
{
  this->name_space_->dump();
}
