/* -*- C++ -*- */

#if !defined (ACE_HAS_INLINED_OSCALLS)
#undef INLINE
#define INLINE
#endif /* ACE_HAS_INLINED_OSCALLS */

#if !defined (ACE_HAS_SYS_SIGLIST)
#if !defined (_sys_siglist)
#define _sys_siglist sis_siglist
#endif /* !defined (sys_siglist) */
extern char **_sys_siglist;
#endif /* !ACE_HAS_SYS_SIGLIST */

#if defined (ACE_HAS_SIZET_RECVFROM)
typedef size_t ACE_RECVFROM_TYPE;
#else
typedef int ACE_RECVFROM_TYPE;
#endif /* ACE_HAS_SIZET_RECVFROM */

#if defined (ACE_HAS_VOIDPTR_SOCKOPT)
typedef void *ACE_SOCKOPT_TYPE1;
#else
typedef const char *ACE_SOCKOPT_TYPE1;
#endif /* ACE_HAS_VOIDPTR_SOCKOPT */

#if defined (ACE_NEEDS_WRITEV)
extern "C" int writev (ACE_HANDLE handle, const struct iovec *iov, int iovcnt);
#endif /* ACE_NEEDS_WRITEV */

#if defined (ACE_NEEDS_READV)
extern "C" ssize_t readv (ACE_HANDLE handle, struct iovec *iov, int iovcnt);
#endif /* ACE_NEEDS_READV */

#if defined (ACE_HAS_VOIDPTR_MMAP)
// Needed for some odd OS's (e.g., SGI).
typedef void *ACE_MMAP_TYPE;
#else
typedef char *ACE_MMAP_TYPE;
#endif /* ACE_HAS_VOIDPTR_MMAP */

#if defined (ACE_HAS_SIZET_SOCKOPT)
typedef size_t *ACE_SOCKOPT_TYPE2;
#else
typedef int ACE_SOCKOPT_TYPE2;
#endif /* ACE_HAS_SIZET_SOCKOPT */

#if defined (ACE_HAS_INT_SOCKNAME)
typedef int ACE_SOCKNAME_TYPE;
#else
typedef int *ACE_SOCKNAME_TYPE;
#endif /* ACE_HAS_INT_SOCKNAME */

#if !defined (ACE_HAS_STRERROR)
#define strerror(err) sys_errlist[err]
#endif /* !ACE_HAS_STERROR */

#if defined (ACE_HAS_XLI)
#include <xliuser.h>
#endif /* ACE_HAS_XLI */

#if !defined (ACE_HAS_CPLUSPLUS_HEADERS)
#include <libc.h>
#include <osfcn.h>
#endif /* ACE_HAS_CPLUSPLUS_HEADERS */

#if defined (ACE_HAS_SYSENT_H)
#include <sysent.h>
#endif /* ACE_HAS_SYSENT_H_*/

#if defined (ACE_HAS_SYS_FILIO_H)
#include <sys/filio.h>
#endif /* ACE_HAS_SYS_FILIO_H */

/* Matthew Stevens 7-10-95 Fix GNU GCC 2.7 for memchr() problem */
#if !defined (ACE_HAS_GNU_CSTRING_H)
#include <rpc/rpc.h>
#include <memory.h>
#endif

#if defined (ACE_HAS_SVR4_GETTIMEOFDAY)
extern "C" int gettimeofday (struct timeval *tp, void * = 0);
#elif defined (ACE_HAS_OSF1_GETTIMEOFDAY)
extern "C" int gettimeofday (struct timeval *tp, struct timezone * = 0);
#elif defined (ACE_HAS_SUNOS4_GETTIMEOFDAY)
#define ACE_HAS_SVR4_GETTIMEOFDAY
/*
#elif defined (ACE_HAS_IRIX53_GETTIMEOFDAY)
extern "C" int gettimeofday (struct timeval *tp, ...);
#else
extern "C" int gettimeofday (struct timeval *tp);
*/
#endif /* ACE_HAS_SVR4_GETTIMEOFDAY */

#include <malloc.h>

#if !defined (ACE_WIN32)

/* These prototypes are chronically lacking from many versions of UNIX. */
extern "C" int t_getname (int, struct netbuf *, int);
extern "C" int isastream (int);
extern "C" int getrusage (int who, struct rusage *rusage);

#if defined (ACE_LACKS_SYSCALL)
extern "C" int syscall (int, ACE_HANDLE, struct rusage *);
#endif /* ACE_LACKS_SYSCALL */

#if defined (ACE_LACKS_MKTEMP)
extern "C" char *mktemp (char *);
#endif /* ACE_LACKS_MKTEMP */

// The following are #defines and #includes that must be visible for
// ACE to compile it's OS wrapper class implementation correctly.  We
// put them inside of here to reduce compiler overhead if we're not
// inlining...

#if defined (ACE_HAS_REGEX)
#include <regexpr.h>
#endif /* ACE_HAS_REGEX */

#if defined (ACE_HAS_SYSINFO)
#include <sys/systeminfo.h>
#endif /* ACE_HAS_SYS_INFO */

#if defined (ACE_HAS_SYSCALL_H)
#include <sys/syscall.h>
#endif /* ACE_HAS_SYSCALL_H */

// Adapt the weird threading and synchronization routines (which don't
// return -1 normally) so that they return -1 and work correctly with
// the ACE_OSCALL macros.
#define ACE_ADAPT_RETVAL(OP,RESULT) ((RESULT = (OP)) != 0 ? (errno = RESULT, -1) : 0)

#if defined (SIGNAL_SAFE_OS_CALLS)
#include "ace/Log_Msg.h"
// The following two macros ensure that system calls are properly
// restarted (if necessary) when interrupts occur.
#define ACE_OSCALL(OP,TYPE,FAILVALUE,RESULT) \
  do \
    RESULT = (TYPE) OP; \
  while (RESULT == FAILVALUE && errno == EINTR && ACE_LOG_MSG->restart ())
#define ACE_OSCALL_RETURN(OP,TYPE,FAILVALUE) \
  do { \
    TYPE _result; \
    do \
      _result = (TYPE) OP; \
    while (_result == FAILVALUE && errno == EINTR && ACE_LOG_MSG->restart ()); \
    return _result; \
  } while (0)
#else
#define ACE_OSCALL_RETURN(OP,TYPE,FAILVALUE) do { TYPE _result; return OP; } while (0)
#define ACE_OSCALL(OP,TYPE,FAILVALUE,RESULT) \
   do { RESULT = (TYPE) OP; } while (0)
#endif /* SIGNAL_SAFE_OS_CALLS */

// Doesn't need a macro since it *never* returns!

INLINE void 
ACE_OS::_exit (int status)
{
  ACE_TRACE ("ACE_OS::_exit");
  ::_exit (status);
}

INLINE int 
ACE_OS::chdir (const char *path)
{
  ACE_TRACE ("ACE_OS::chdir");
  ACE_OSCALL_RETURN (::chdir (path), int, -1);
}

INLINE char *
ACE_OS::cuserid (char *user, size_t)
{
  ACE_TRACE ("ACE_OS::cuserid");
  ACE_OSCALL_RETURN (::cuserid (user), char *, 0);
}

INLINE int 
ACE_OS::dup (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::dup");
  ACE_OSCALL_RETURN (::dup (fd), int, -1);
}

INLINE int 
ACE_OS::dup2 (ACE_HANDLE oldhandle, ACE_HANDLE newfd)
{
  ACE_TRACE ("ACE_OS::dup2");
  ACE_OSCALL_RETURN (::dup2 (oldhandle, newfd), int, -1);
}

INLINE int 
ACE_OS::fcntl (ACE_HANDLE handle, int cmd, int value)
{
  ACE_TRACE ("ACE_OS::fcntl");
  ACE_OSCALL_RETURN (::fcntl (handle, cmd, value), int, -1);
}

INLINE FILE *
ACE_OS::fdopen (ACE_HANDLE handle, const char *mode)
{
  ACE_TRACE ("ACE_OS::fdopen");
  ACE_OSCALL_RETURN (::fdopen (handle, mode), FILE *, 0);
}

INLINE int 
ACE_OS::fstat (ACE_HANDLE handle, struct stat *stp)
{
  ACE_TRACE ("ACE_OS::fstat");
  ACE_OSCALL_RETURN (::fstat (handle, stp), int, -1);
}

INLINE int 
ACE_OS::ftruncate (ACE_HANDLE handle, off_t offset)
{
  ACE_TRACE ("ACE_OS::ftruncate");
  ACE_OSCALL_RETURN (::ftruncate (handle, offset), int, -1);
}

INLINE int 
ACE_OS::fdetach (const char *file)
{
  ACE_TRACE ("ACE_OS::fdetach");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::fdetach (file), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int
ACE_OS::fattach (int handle, const char *path)
{
  ACE_TRACE ("ACE_OS::fattach");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::fattach (handle, path), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE pid_t 
ACE_OS::fork (void)
{
  ACE_TRACE ("ACE_OS::fork");
  ACE_OSCALL_RETURN (::fork (), pid_t, -1);
}

INLINE char *
ACE_OS::getenv (const char *symbol)
{
  ACE_TRACE ("ACE_OS::getenv");
  ACE_OSCALL_RETURN (::getenv (symbol), char *, 0);
}

INLINE uid_t 
ACE_OS::getgid (void)
{
  ACE_TRACE ("ACE_OS::getgid");
  ACE_OSCALL_RETURN (::getgid (), uid_t, -1);
}

INLINE hrtime_t 
ACE_OS::gethrtime (void)
{
  ACE_TRACE ("ACE_OS::gethrtime");
#if defined (ACE_HAS_HI_RES_TIMER)
  ACE_OSCALL_RETURN (::gethrtime (), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_HI_RES_TIMER */
}

INLINE int 
ACE_OS::getopt (int argc, char *const *argv, const char *optstring)
{
  ACE_TRACE ("ACE_OS::getopt");
  ACE_OSCALL_RETURN (::getopt (argc, argv, optstring), int, -1);
}

INLINE pid_t 
ACE_OS::getpid (void)
{
  ACE_TRACE ("ACE_OS::getpid");
  ACE_OSCALL_RETURN (::getpid (), int, -1);
}

INLINE int 
ACE_OS::getrlimit (ACE_HANDLE handle, struct rlimit *rl)
{
  ACE_TRACE ("ACE_OS::getrlimit");
  ACE_OSCALL_RETURN (::getrlimit (handle, rl), int, -1);
}

INLINE uid_t 
ACE_OS::getuid (void)
{
  ACE_TRACE ("ACE_OS::getuid");
  ACE_OSCALL_RETURN (::getuid (), uid_t, -1);
}

INLINE int 
ACE_OS::ioctl (ACE_HANDLE handle, int cmd, void *val)
{
  ACE_TRACE ("ACE_OS::ioctl");
  ACE_OSCALL_RETURN (::ioctl (handle, cmd, val), int, -1);
}

INLINE int 
ACE_OS::isatty (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::isatty");
  ACE_OSCALL_RETURN (::isatty (fd), int, -1);
}

INLINE int 
ACE_OS::kill (pid_t pid, int signum)
{
  ACE_TRACE ("ACE_OS::kill");
  ACE_OSCALL_RETURN (::kill (pid, signum), int, -1);
}

INLINE off_t 
ACE_OS::lseek (ACE_HANDLE handle, off_t offset, int whence)
{
  ACE_TRACE ("ACE_OS::lseek");
  ACE_OSCALL_RETURN (::lseek (handle, offset, whence), int, -1);
}

INLINE int 
ACE_OS::mkfifo (const char *file, mode_t mode)
{
  ACE_TRACE ("ACE_OS::mkfifo");
  ACE_OSCALL_RETURN (::mkfifo (file, mode), int, -1);
}

INLINE char *
ACE_OS::mktemp (char *t)
{
  ACE_TRACE ("ACE_OS::mktemp");
  return ::mktemp (t);
}

INLINE int 
ACE_OS::pipe (ACE_HANDLE fds[])
{
  ACE_TRACE ("ACE_OS::pipe");
  ACE_OSCALL_RETURN (::pipe (fds), int, -1);
}

INLINE int 
ACE_OS::rand (void)
{
  ACE_TRACE ("ACE_OS::rand");
  ACE_OSCALL_RETURN (::rand (), int, -1);
}

INLINE int 
ACE_OS::rand_r (ACE_RANDR_TYPE seed)
{
  ACE_TRACE ("ACE_OS::rand_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
  ACE_OSCALL_RETURN (::rand_r (seed), int, -1);
#else
  ACE_OSCALL_RETURN (::rand (), int, -1);
#endif
}

INLINE void *
ACE_OS::sbrk (int brk)
{
  ACE_TRACE ("ACE_OS::sbrk");
  ACE_OSCALL_RETURN (::sbrk (brk), void *, 0);
}

INLINE int 
ACE_OS::setrlimit (ACE_HANDLE handle, const struct rlimit *rl)
{
  ACE_TRACE ("ACE_OS::setrlimit");
  ACE_OSCALL_RETURN (::setrlimit (handle, rl), int, -1);
}

INLINE pid_t 
ACE_OS::setsid (void)
{
  ACE_TRACE ("ACE_OS::setsid");
  ACE_OSCALL_RETURN (::setsid (), int, -1);
}

INLINE int 
ACE_OS::sigaction (int signum, const struct sigaction *nsa, 
	       struct sigaction *osa)
{
  ACE_TRACE ("ACE_OS::sigaction");
  ACE_OSCALL_RETURN (::sigaction (signum, nsa, osa), int, -1);
}

INLINE int 
ACE_OS::sigaddset (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigaddset");
  ACE_OSCALL_RETURN (::sigaddset (s, signum), int, -1);
}

INLINE int 
ACE_OS::sigdelset (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigdelset");
  ACE_OSCALL_RETURN (::sigdelset (s, signum), int, -1);
}

INLINE int 
ACE_OS::sigemptyset (sigset_t *s)
{
  ACE_TRACE ("ACE_OS::sigemptyset");
  ACE_OSCALL_RETURN (::sigemptyset (s), int, -1);
}

INLINE int 
ACE_OS::sigfillset (sigset_t *s)
{
  ACE_TRACE ("ACE_OS::sigfillset");
  ACE_OSCALL_RETURN (::sigfillset (s), int, -1);
}

INLINE int 
ACE_OS::sigismember (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigismember");
  ACE_OSCALL_RETURN (::sigismember (s, signum), int, -1);
}

INLINE int 
ACE_OS::sigprocmask (int signum, const sigset_t *nsp, sigset_t *osp)
{
  ACE_TRACE ("ACE_OS::sigprocmask");
  ACE_OSCALL_RETURN (::sigprocmask (signum, nsp, osp), int, -1);
}

INLINE int 
ACE_OS::strcasecmp (const char *s, const char *t)
{
  ACE_TRACE ("ACE_OS::strcasecmp");
  ACE_OSCALL_RETURN (::strcasecmp (s, t), int, -1);
}

INLINE u_int 
ACE_OS::sleep (u_int seconds)
{
  ACE_TRACE ("ACE_OS::sleep");
  ACE_OSCALL_RETURN (::sleep (seconds), int, -1);
}

INLINE int 
ACE_OS::socketpair (int domain, int type, 
		    int protocol, ACE_HANDLE sv[2])
{
  ACE_TRACE ("ACE_OS::socketpair");
  ACE_OSCALL_RETURN (::socketpair (domain, type, protocol, sv),
		     int, -1);
}

INLINE void 
ACE_OS::srand (u_int seed)
{
  ACE_TRACE ("ACE_OS::srand");
  ::srand (seed);
}

INLINE time_t 
ACE_OS::time (time_t *tloc)
{
  ACE_TRACE ("ACE_OS::time");
  ACE_OSCALL_RETURN (::time (tloc), time_t, -1);
}

INLINE mode_t 
ACE_OS::umask (mode_t cmask)
{
  ACE_TRACE ("ACE_OS::umask");
  ACE_OSCALL_RETURN (::umask (cmask), mode_t, -1);
}

INLINE int 
ACE_OS::unlink (const char *path)
{
  ACE_TRACE ("ACE_OS::unlink");
  ACE_OSCALL_RETURN (::unlink (path), int, -1);
}

INLINE pid_t 
ACE_OS::wait (int *stat_loc)
{
  ACE_TRACE ("ACE_OS::wait");
#if !defined (AIX)
  ACE_OSCALL_RETURN (::wait (stat_loc), pid_t, -1);
#else
  ACE_OSCALL_RETURN (::wait ((union wait *) stat_loc), pid_t, -1);
#endif /* !defined (AIX) */
}

INLINE pid_t 
ACE_OS::waitpid (pid_t pid, int *stat_loc, int options)
{
  ACE_TRACE ("ACE_OS::waitpid");
  ACE_OSCALL_RETURN (::waitpid (pid, stat_loc, options), 
		     pid_t, -1);
}

#else // ACE_WIN32

// This is for Win32 exclusively!

#define ACE_OSCALL_RETURN(X,TYPE,FAILVALUE) do { return X; } while (0)
#define ACE_OSCALL(X,TYPE,FAILVALUE,RESULT) \
  do { RESULT = (TYPE) X; } while (0)

// Doesn't need a macro since it *never* returns!

INLINE void 
ACE_OS::_exit (int status)
{
  ACE_TRACE ("ACE_OS::_exit");
  ::_exit (status);
}

INLINE int 
ACE_OS::chdir (const char *path)
{
  ACE_TRACE ("ACE_OS::chdir");
  ACE_OSCALL_RETURN (::_chdir (path), int, -1);
}

INLINE int 
ACE_OS::strcasecmp (const char *s, const char *t)
{
  ACE_TRACE ("ACE_OS::strcasecmp");
  ACE_NOTSUP_RETURN (0);
}

INLINE char *
ACE_OS::cuserid (char *user, size_t maxlen)
{
  ACE_TRACE ("ACE_OS::cuserid");
  // Hackish because of missing buffer size!
  return ::GetUserName (user, (unsigned long *) &maxlen) ? user : 0;
}

INLINE int 
ACE_OS::dup (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::dup");
  ACE_NOTSUP_RETURN (-1);
  // ACE_OSCALL_RETURN (::dup (fd), int, -1);   - Should :: DuplicateHandle ()
}

INLINE int 
ACE_OS::dup2 (ACE_HANDLE oldhandle, ACE_HANDLE newfd)
{
  ACE_TRACE ("ACE_OS::dup2");
  ACE_NOTSUP_RETURN (-1);
  // ACE_OSCALL_RETURN (::dup2 (oldhandle, newfd), int, -1);
}

INLINE int 
ACE_OS::fcntl (ACE_HANDLE handle, int cmd, int value)
{
  ACE_TRACE ("ACE_OS::fcntl");
  ACE_NOTSUP_RETURN (0); // We should be able to map this stuff
}

INLINE int 
ACE_OS::fdetach (const char *file)
{
  ACE_TRACE ("ACE_OS::fdetach");
  ACE_NOTSUP_RETURN (0); // STREAMS something, doesn't matter for Win32
}

INLINE pid_t 
ACE_OS::fork (void)
{
  ACE_TRACE ("ACE_OS::fork");
  ACE_NOTSUP_RETURN (0); // Sorry, Win32 can't fork
}

INLINE char *
ACE_OS::getenv (const char *symbol)
{
  ACE_TRACE ("ACE_OS::getenv");
  ACE_OSCALL_RETURN (::getenv (symbol), char *, 0);
}

INLINE uid_t 
ACE_OS::getgid (void)
{
  ACE_TRACE ("ACE_OS::getgid");
  ACE_NOTSUP_RETURN (-1);
}

INLINE hrtime_t 
ACE_OS::gethrtime (void)
{
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::getopt (int argc, char *const *argv, const char *optstring)
{
  ACE_TRACE ("ACE_OS::getopt");
  ACE_NOTSUP_RETURN (-1);
}

INLINE pid_t 
ACE_OS::getpid (void)
{
  ACE_TRACE ("ACE_OS::getpid");
  ACE_NOTSUP_RETURN (0); // this will confuse some callers...
}

INLINE int 
ACE_OS::getrlimit (ACE_HANDLE handle, struct rlimit *rl)
{
  ACE_TRACE ("ACE_OS::getrlimit");
  ACE_NOTSUP_RETURN (-1);
}

INLINE uid_t 
ACE_OS::getuid (void)
{
  ACE_TRACE ("ACE_OS::getuid");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::ioctl (ACE_HANDLE handle, int cmd, void *val)
{
  ACE_TRACE ("ACE_OS::ioctl");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::isatty (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::isatty");
  ACE_NOTSUP_RETURN (-1);
//  ACE_OSCALL_RETURN (::_isatty (fd), int, -1);
}

INLINE int 
ACE_OS::kill (pid_t pid, int signum)
{
  ACE_TRACE ("ACE_OS::kill");
  ACE_NOTSUP_RETURN (-1);
}

INLINE off_t 
ACE_OS::lseek (ACE_HANDLE handle, off_t offset, int whence)
{
  ACE_TRACE ("ACE_OS::lseek");
  ACE_NOTSUP_RETURN (-1);
//  ACE_OSCALL_RETURN (::_lseek (handle, offset, whence), int, -1);
}

INLINE int 
ACE_OS::mkfifo (const char *file, mode_t mode)
{
  ACE_TRACE ("ACE_OS::mkfifo");
  ACE_NOTSUP_RETURN (-1);
}

INLINE char *
ACE_OS::mktemp (char *t)
{
  ACE_TRACE ("ACE_OS::mktemp");
  return ::mktemp (t);
}

INLINE int 
ACE_OS::pipe (ACE_HANDLE fds[])
{
  ACE_TRACE ("ACE_OS::pipe");
  ACE_NOTSUP_RETURN (-1);
//  ACE_OSCALL_RETURN (::_pipe (fds, PIPE_BUF, 0), int, -1);   // Use default mode
}

INLINE int 
ACE_OS::rand (void)
{
  ACE_TRACE ("ACE_OS::rand");
  ACE_OSCALL_RETURN (::rand (), int, -1);
}

INLINE int 
ACE_OS::rand_r (ACE_RANDR_TYPE seed)
{
  ACE_TRACE ("ACE_OS::rand_r");
  ACE_NOTSUP_RETURN (-1);
}

INLINE void *
ACE_OS::sbrk (int brk)
{
  ACE_TRACE ("ACE_OS::sbrk");
  ACE_NOTSUP_RETURN (0);
}

INLINE int 
ACE_OS::setrlimit (ACE_HANDLE handle, const struct rlimit *rl)
{
  ACE_TRACE ("ACE_OS::setrlimit");
  ACE_NOTSUP_RETURN (-1);
}

INLINE pid_t 
ACE_OS::setsid (void)
{
  ACE_TRACE ("ACE_OS::setsid");
  ACE_NOTSUP_RETURN (0);
}

INLINE int 
ACE_OS::sigaction (int signum, const struct sigaction *nsa, 
	       struct sigaction *osa)
{
  ACE_TRACE ("ACE_OS::sigaction");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigaddset (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigaddset");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigdelset (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigdelset");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigemptyset (sigset_t *s)
{
  ACE_TRACE ("ACE_OS::sigemptyset");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigfillset (sigset_t *s)
{
  ACE_TRACE ("ACE_OS::sigfillset");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigismember (sigset_t *s, int signum)
{
  ACE_TRACE ("ACE_OS::sigismember");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::sigprocmask (int signum, const sigset_t *nsp, sigset_t *osp)
{
  ACE_TRACE ("ACE_OS::sigprocmask");
  ACE_NOTSUP_RETURN (-1);
}

INLINE u_int 
ACE_OS::sleep (u_int seconds)
{
  ACE_TRACE ("ACE_OS::sleep");
  ::Sleep (seconds * 1000);
  return 0;
}

INLINE int 
ACE_OS::socketpair (int domain, int type, 
		    int protocol, ACE_HANDLE sv[2])
{
  ACE_TRACE ("ACE_OS::socketpair");
  ACE_NOTSUP_RETURN (-1);
}

INLINE void 
ACE_OS::srand (u_int seed)
{
  ACE_TRACE ("ACE_OS::srand");
  ::srand (seed);
}

INLINE time_t 
ACE_OS::time (time_t *tloc)
{
  ACE_TRACE ("ACE_OS::time");
  ACE_OSCALL_RETURN (::time (tloc), time_t, -1);
}

INLINE mode_t 
ACE_OS::umask (mode_t cmask)
{
  ACE_TRACE ("ACE_OS::umask");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::unlink (const char *path)
{
  ACE_TRACE ("ACE_OS::unlink");
  ACE_OSCALL_RETURN (::unlink (path), int, -1);
}

INLINE pid_t 
ACE_OS::waitpid (pid_t pid, int *stat_loc, int options)
{
  ACE_TRACE ("ACE_OS::waitpid");
  ACE_NOTSUP_RETURN (0);
}

INLINE FILE *
ACE_OS::fdopen (ACE_HANDLE handle, const char *mode)
{
  ACE_TRACE ("ACE_OS::fdopen");
  ACE_NOTSUP_RETURN (0);
}

INLINE int 
ACE_OS::fstat (ACE_HANDLE handle, struct stat *stp)
{
  ACE_TRACE ("ACE_OS::fstat");
  ACE_NOTSUP_RETURN (-1);
}

INLINE int 
ACE_OS::ftruncate (ACE_HANDLE handle, off_t offset)
{
  ACE_TRACE ("ACE_OS::ftruncate");
  ACE_NOTSUP_RETURN (-1);
}

#endif // WIN32

INLINE int 
ACE_OS::cond_broadcast (cond_t *cv)
{
  ACE_TRACE ("ACE_OS::cond_broadcast");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::cond_broadcast (cv), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_cond_broadcast (cv), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  if (ACE_OS::mutex_lock (&cv->internal_mutex_) != 0)
    return -1;
  int result = 0;

  while (cv->waiters_ > 0)
    {
      if (ACE_OS::sema_post (&cv->sema_) != 0)
	{
	  result = -1;
	  break;
	}
      else
	cv->waiters_--;
    }

  ACE_OS::mutex_unlock (&cv->internal_mutex_);
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::cond_destroy (cond_t *cv)
{
  ACE_TRACE ("ACE_OS::cond_destroy");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::cond_destroy (cv), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_cond_destroy (cv), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_OS::sema_destroy (&cv->sema_);
  return ACE_OS::mutex_destroy (&cv->internal_mutex_);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::cond_init (cond_t *cv, int type, const char *name, void *arg)
{
  ACE_TRACE ("ACE_OS::cond_init");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::cond_init (cv, type, arg), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  pthread_condattr_t attributes;
  int result = -1;

  if (::pthread_condattr_init (&attributes) == 0
#if !defined (ACE_LACKS_CONDATTR_PSHARED)
      && ::pthread_condattr_setpshared (&attributes, type) == 0
#endif /* ACE_LACKS_CONDATTR_PSHARED */
      && ::pthread_cond_init (cv, &attributes) == 0)
    result = 0;

  ::pthread_condattr_destroy (&attributes);
  return result;
#elif defined (ACE_HAS_WTHREADS)
  cv->waiters_ = 0;

  if (ACE_OS::sema_init (&cv->sema_, 1, type, name, arg) != 0)
    return -1;

  if (ACE_OS::mutex_init (&cv->internal_mutex_, type, name, arg) != 0)
    {
      ACE_OS::sema_destroy (&cv->sema_);
      return -1;
    }
  return 0;

#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::cond_signal (cond_t *cv)
{
  ACE_TRACE ("ACE_OS::cond_signal");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::cond_signal (cv), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_cond_signal (cv), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  if (ACE_OS::mutex_lock (&cv->internal_mutex_) != 0)
    return -1;
  int result = 0;
  if (cv->waiters_ > 0)
    {
      if (ACE_OS::sema_post (&cv->sema_) != 0)
	result = -1;
      else
	cv->waiters_--;
    }
  ACE_OS::mutex_unlock (&cv->internal_mutex_);
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::cond_timedwait (cond_t *cv, mutex_t *external_mutex, ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_OS::cond_timedwait");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_WTHREADS)
  if (ACE_OS::mutex_lock (&cv->internal_mutex_) != 0)
    return -1;

  cv->waiters_++;

  if (ACE_OS::mutex_unlock (&cv->internal_mutex_) != 0)
    return -1;
  else if (ACE_OS::mutex_unlock (external_mutex) != 0)
    return -1;

  int result = ::WaitForSingleObject (cv->sema_, timeout == 0 ? INFINITE : timeout->msec ());

  if (result == WAIT_TIMEOUT) 
    errno = EBUSY;

  ACE_OS::mutex_lock (external_mutex);
  return result == WAIT_OBJECT_0 ? 0 : -1;
#else /* Either PTHREADS or WTHREADS */
  int result;
  timestruc_t ts = *timeout; // Calls ACE_Time_Value::operator timestruc_t().
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL (ACE_ADAPT_RETVAL ((timeout == 0
				? ::cond_wait (cv, external_mutex) 
				: ::cond_timedwait (cv, external_mutex, &ts)), result),
	      int, -1, result);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL (ACE_ADAPT_RETVAL ((timeout == 0
				? ::pthread_cond_wait (cv, external_mutex)
				: ::pthread_cond_timedwait (cv, external_mutex, &ts)), result),
	      int, -1, result);
#endif /* ACE_HAS_STHREADS */
  timeout->set (ts); // Update the time value before returning.
  return result;
#endif /* ACE_HAS_WTHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::cond_wait (cond_t *cv, mutex_t *external_mutex)
{
  ACE_TRACE ("ACE_OS::cond_wait");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::cond_wait (cv, external_mutex), _result), 
		     int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_cond_wait (cv, external_mutex), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  if (ACE_OS::mutex_lock (&cv->internal_mutex_) != 0)
    return -1;

  cv->waiters_++;

  if (ACE_OS::mutex_unlock (&cv->internal_mutex_) != 0)
    return -1;
  else if (ACE_OS::mutex_unlock (external_mutex) != 0)
    return -1;

  if (ACE_OS::sema_wait (&cv->sema_) != 0)
    return -1;

  ACE_OS::mutex_lock (external_mutex);
  return 0;

#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::mutex_init (mutex_t *m, int type, const char *name, void *arg)
{
  ACE_TRACE ("ACE_OS::mutex_init");

#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::mutex_init (m, type, arg), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  pthread_mutexattr_t attributes;
  int result = -1;

  if (::pthread_mutexattr_init (&attributes) == 0
#if defined (ACE_HAS_SETKIND_NP)
      && ::pthread_mutexattr_setkind_np (&attributes, type) == 0
#endif /* ACE_HAS_SETKIND_NP */
      && ::pthread_mutex_init (m, &attributes) == 0)
    result = 0;

  ::pthread_mutexattr_destroy (&attributes);
  return result;
#elif defined (ACE_HAS_WTHREADS)
  *m = ::CreateMutex (NULL, FALSE, name);
  return *m != 0 ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::mutex_destroy (mutex_t *m)
{
  ACE_TRACE ("ACE_OS::mutex_destroy");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::mutex_destroy (m), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_mutex_destroy (m), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  return ::CloseHandle (*m) ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::mutex_lock (mutex_t *m)
{
  ACE_TRACE ("ACE_OS::mutex_lock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::mutex_lock (m), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_mutex_lock (m), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  // timeout can't occur, so don't check.
  return (::WaitForSingleObject(*m, INFINITE) != WAIT_FAILED) ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::mutex_trylock (mutex_t *m)
{
  ACE_TRACE ("ACE_OS::mutex_trylock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::mutex_trylock (m), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_mutex_trylock (m), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  // Try for 0 milliseconds - i.e. nonblocking.
  DWORD status = ::WaitForSingleObject(*m, 0);
  // if the status is either WAIT_TIMEOUT or WAIT_FAILED we didn't get it.
  return (status == WAIT_ABANDONED || status == WAIT_OBJECT_0) ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::mutex_unlock (mutex_t *m)
{
  ACE_TRACE ("ACE_OS::mutex_unlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::mutex_unlock (m), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_mutex_unlock (m), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  return ::ReleaseMutex (*m) ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rw_rdlock (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rw_rdlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rw_rdlock (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
#if defined (ACE_HAS_PTHREADS)
  ::pthread_cleanup_push (ACE_OS::mutex_lock_cleanup, (void *) &rw->lock_);
#endif
  if (ACE_OS::mutex_lock (&rw->lock_) == -1)
    {
#if defined (ACE_HAS_PTHREADS)
    ::pthread_cleanup_pop (0);
    return -1;
#endif
    }
  // Give preference to writers who are waiting.
  while (rw->ref_count_ < 0 || rw->num_waiting_writers_ > 0)
    {
      rw->num_waiting_readers_++;
      if (ACE_OS::cond_wait (&rw->waiting_readers_, &rw->lock_) == -1)
	{
	  ACE_OS::mutex_unlock (&rw->lock_);
#if defined (ACE_HAS_PTHREADS)
          ::pthread_cleanup_pop (0);
#endif
	  return -1;
	}
      rw->num_waiting_readers_--;
    }

  rw->ref_count_++;
  ACE_OS::mutex_unlock (&rw->lock_);
#if defined (ACE_HAS_PTHREADS)
  ::pthread_cleanup_pop (0);
#endif
  return 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rw_tryrdlock (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rw_tryrdlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rw_tryrdlock (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
  if (ACE_OS::mutex_lock (&rw->lock_) == -1)
    return -1;

  int result;

  if (rw->ref_count_ == -1 || rw->num_waiting_writers_ > 0)
    {
      errno = EBUSY;
      result = -1;
    }
  else
    {
      rw->ref_count_++;
      result = 0;
    }

  ACE_OS::mutex_unlock (&rw->lock_);
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rw_trywrlock (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rw_trywrlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rw_trywrlock (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
  if (ACE_OS::mutex_lock (&rw->lock_) == -1)
    return -1;

  int result;

  if (rw->ref_count_ != 0)
    {
      errno = EBUSY;
      result = -1;
    }
  else
    {
      rw->ref_count_ = -1;
      result = 0;
    }

  ACE_OS::mutex_unlock (&rw->lock_);
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rw_unlock (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rw_unlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rw_unlock (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
  if (ACE_OS::mutex_lock (&rw->lock_) == -1)
    return -1;

  if (rw->ref_count_ > 0) // Releasing a reader.
    rw->ref_count_--;
  else if (rw->ref_count_ == -1) // Releasing a writer.
    rw->ref_count_ = 0;
  else
    assert (!"count should not be 0!\n");

  int result;

  // Give preference to writers over readers...
  if (rw->num_waiting_writers_ > 0)
    result = ACE_OS::cond_signal (&rw->waiting_writers_);
  else if (rw->num_waiting_readers_ > 0)
    result = ACE_OS::cond_broadcast (&rw->waiting_readers_);
  else
    result = 0;

  ACE_OS::mutex_unlock (&rw->lock_);
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rw_wrlock (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rw_wrlock");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rw_wrlock (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
#if defined (ACE_HAS_PTHREADS)
  ::pthread_cleanup_push (ACE_OS::mutex_lock_cleanup, (void *) &rw->lock_);
#endif

  if (ACE_OS::mutex_lock (&rw->lock_) == -1)
  {
#if defined (ACE_HAS_PTHREADS)
    ::pthread_cleanup_pop (0);
#endif
    return -1;
  }

  while (rw->ref_count_ != 0)
    {
      rw->num_waiting_writers_++;
      if (ACE_OS::cond_wait (&rw->waiting_writers_, &rw->lock_) == -1)
	{
	  ACE_OS::mutex_unlock (&rw->lock_);
#if defined (ACE_HAS_PTHREADS)
          ::pthread_cleanup_pop (0);
#endif
	  return -1;
	}
      rw->num_waiting_writers_--;
    }

  rw->ref_count_ = -1;
  ACE_OS::mutex_unlock (&rw->lock_);
#if defined (ACE_HAS_PTHREADS)
  ::pthread_cleanup_pop (0);
#endif
  return 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rwlock_init (rwlock_t *rw, 
		     int type, 
		     const char *name, 
		     void *arg)
{
  ACE_TRACE ("ACE_OS::rwlock_init");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rwlock_init (rw, type, arg), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
  int result = -1;

  if (ACE_OS::mutex_init (&rw->lock_, type, name, arg) == 0
      && ACE_OS::cond_init (&rw->waiting_readers_, type, name, arg) == 0
      && ACE_OS::cond_init (&rw->waiting_writers_, type, name, arg) == 0)
    {
      // Success!  
      rw->ref_count_ = 0;
      rw->num_waiting_writers_ = 0;
      rw->num_waiting_readers_ = 0;

      result = 0;
    }

  if (result == -1)
    {
      ACE_OS::mutex_destroy (&rw->lock_);
      ACE_OS::cond_destroy (&rw->waiting_readers_);
      ACE_OS::cond_destroy (&rw->waiting_writers_);
    }
  return result;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::rwlock_destroy (rwlock_t *rw)
{
  ACE_TRACE ("ACE_OS::rwlock_destroy");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::rwlock_destroy (rw), _result), int, -1);
#else // Both NT and POSIX don't support this natively.
  ACE_OS::mutex_destroy (&rw->lock_);
  ACE_OS::cond_destroy (&rw->waiting_readers_);
  return ACE_OS::cond_destroy (&rw->waiting_writers_);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE ACE_HANDLE
ACE_OS::accept (ACE_HANDLE handle, struct sockaddr *addr, int *addrlen)
{
  ACE_TRACE ("ACE_OS::accept");
  ACE_OSCALL_RETURN ((ACE_HANDLE) ::accept ((ACE_SOCKET) handle, addr, addrlen), int, -1);
}

INLINE int 
ACE_OS::atoi (const char *s)
{
  ACE_TRACE ("ACE_OS::atoi");
  ACE_OSCALL_RETURN (::atoi (s), int, -1);
}

INLINE int 
ACE_OS::bind (ACE_HANDLE handle, struct sockaddr *addr, int addrlen)
{
  ACE_TRACE ("ACE_OS::bind");
  ACE_OSCALL_RETURN (::bind ((ACE_SOCKET) handle, addr, addrlen), int, -1);
}

INLINE int 
ACE_OS::connect (ACE_HANDLE handle, struct sockaddr *addr, int addrlen)
{
  ACE_TRACE ("ACE_OS::connect");
  ACE_OSCALL_RETURN (::connect ((ACE_SOCKET) handle, addr, addrlen), int, -1);
}

INLINE int 
ACE_OS::select (int width, 
		fd_set *rfds, fd_set *wfds, fd_set *efds, 
		ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_OS::select");
  ACE_OSCALL_RETURN (::select (width, 
			       (ACE_FD_SET_TYPE *) rfds, 
			       (ACE_FD_SET_TYPE *) wfds, 
			       (ACE_FD_SET_TYPE *) efds, 
			       (timeval *) timeout) , int, -1);
}

INLINE int 
ACE_OS::recv (ACE_HANDLE handle, char *buf, int len, int flags)
{
  ACE_TRACE ("ACE_OS::recv");
  ACE_OSCALL_RETURN (::recv ((ACE_SOCKET) handle, buf, len, flags), int, -1);
}

INLINE int 
ACE_OS::recvfrom (ACE_HANDLE handle, char *buf, int len, 
		  int flags, struct sockaddr *addr, int *addrlen)
{
  ACE_TRACE ("ACE_OS::recvfrom");
  ACE_RECVFROM_TYPE *internal_addrlen = (ACE_RECVFROM_TYPE *) addrlen;
  ACE_OSCALL_RETURN (::recvfrom ((ACE_SOCKET) handle, buf, len, flags, addr, internal_addrlen), 
		     int, -1);
}

INLINE int
ACE_OS::recvmsg (ACE_HANDLE handle, struct msghdr *msg, int flags)
{
  ACE_TRACE ("ACE_OS::recvmsg");
#if !defined (ACE_LACKS_RECVMSG)
  ACE_OSCALL_RETURN (::recvmsg (handle, msg, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_MSG */
}

INLINE int 
ACE_OS::send (ACE_HANDLE handle, const char *buf, int len, int flags)
{
  ACE_TRACE ("ACE_OS::send");
  ACE_OSCALL_RETURN (::send ((ACE_SOCKET) handle, buf, len, flags), int, -1);
}

INLINE int 
ACE_OS::sendto (ACE_HANDLE handle, const char *buf, int len, 
		int flags, const struct sockaddr *addr, int addrlen)
{
  ACE_TRACE ("ACE_OS::sendto");
  ACE_OSCALL_RETURN (::sendto ((ACE_SOCKET) handle, buf, len, flags, 
			       (struct sockaddr *) addr, addrlen), int, -1);
}

INLINE int 
ACE_OS::sendmsg (ACE_HANDLE handle, ACE_SENDMSG_TYPE *msg, int flags)
{
  ACE_TRACE ("ACE_OS::sendmsg");
#if !defined (ACE_LACKS_SENDMSG)
  ACE_OSCALL_RETURN (::sendmsg (handle, msg, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_MSG */
}

INLINE int 
ACE_OS::fclose (FILE *fp)
{
  ACE_TRACE ("ACE_OS::fclose");
  ACE_OSCALL_RETURN (::fclose (fp), int, -1);
}

INLINE char *
ACE_OS::fgets (char *buf, int size, FILE *fp)
{
  ACE_TRACE ("ACE_OS::fgets");
  ACE_OSCALL_RETURN (::fgets (buf, size, fp), char *, 0);
}

INLINE int 
ACE_OS::fflush (FILE *fp)
{
  ACE_TRACE ("ACE_OS::fflush");
  ACE_OSCALL_RETURN (::fflush (fp), int, -1);
}

INLINE FILE *
ACE_OS::fopen (const char *filename, const char *mode)
{
  ACE_TRACE ("ACE_OS::fopen");
  ACE_OSCALL_RETURN (::fopen (filename, mode), FILE *, 0);
}

INLINE int 
ACE_OS::fprintf (FILE *fp, const char *format, ...)
{
  ACE_TRACE ("ACE_OS::fprintf");
  int result = 0;
  va_list ap;
  va_start (ap, format);
  ACE_OSCALL (::vfprintf (fp, format, ap), int, -1, result);
  va_end (ap);
  return result;
}

INLINE size_t 
ACE_OS::fread (void *ptr, size_t size, size_t nelems, FILE *fp)
{
  ACE_TRACE ("ACE_OS::fread");
  ACE_OSCALL_RETURN (::fread (ptr, size, nelems, fp), int, 0);
}

INLINE size_t 
ACE_OS::fwrite (const void *ptr, size_t size, size_t nitems, FILE *fp)
{
  ACE_TRACE ("ACE_OS::fwrite");
  ACE_OSCALL_RETURN (::fwrite (ptr, size, nitems, fp), int, 0);
}

INLINE void *
ACE_OS::malloc (size_t nbytes)
{
  ACE_TRACE ("ACE_OS::malloc");
  return ::malloc (nbytes);
}

INLINE void *
ACE_OS::realloc (void *ptr, size_t nbytes)
{
  ACE_TRACE ("ACE_OS::realloc");
  return ::realloc (ACE_MALLOC_T (ptr), nbytes);
}

INLINE void 
ACE_OS::free (void *ptr)
{
  ACE_TRACE ("ACE_OS::free");
  ::free (ACE_MALLOC_T (ptr));
}

INLINE int
ACE_OS::memcmp (const void *s, const void *t, size_t len)
{
  ACE_TRACE ("ACE_OS::memcmp");
  return ::memcmp (s, t, len);
}

INLINE void *
ACE_OS::memcpy (void *s, const void *t, size_t len)
{
  ACE_TRACE ("ACE_OS::memcpy");
  return ::memcpy (s, t, len);
}

INLINE void *
ACE_OS::memset (void *s, int c, size_t len)
{
  ACE_TRACE ("ACE_OS::memset");
  return ::memset (s, c, len);
}

INLINE struct hostent *
ACE_OS::gethostbyaddr (const char *addr, int length, int type)
{
  ACE_TRACE ("ACE_OS::gethostbyaddr");
  ACE_OSCALL_RETURN (::gethostbyaddr (addr, length, type), 
		     struct hostent *, 0); 
}

INLINE struct hostent *
ACE_OS::gethostbyaddr_r (const char *addr, int length, int type, 
			 hostent *result, ACE_HOSTENT_DATA buffer, 
			 int *h_errnop)
{
  ACE_TRACE ("ACE_OS::gethostbyaddr_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
#if !defined (AIX)
  ACE_OSCALL_RETURN (::gethostbyaddr_r (addr, length, type, result, 
					buffer, sizeof (ACE_HOSTENT_DATA), 
					h_errnop),
		     struct hostent *, 0);
#else
  ::memset (buffer, 0, sizeof buffer);

  if (::gethostbyaddr_r ((char *) addr, length, type, result,
			 (struct hostent_data *) buffer)== 0)
    return result;
  else
    {
      *h_errnop = h_errno;
      return (struct hostent *) 0;
    }
#endif /* !defined (AIX) */
#else
  ACE_OSCALL_RETURN (::gethostbyaddr (addr, length, type), 
		     struct hostent *, 0); 
#endif
}
		     
INLINE struct hostent *
ACE_OS::gethostbyname (const char *name)
{
  ACE_TRACE ("ACE_OS::gethostbyname");
  ACE_OSCALL_RETURN (::gethostbyname (name), struct hostent *, 0); 
}

INLINE struct hostent *
ACE_OS::gethostbyname_r (const char *name, hostent *result, 
			 ACE_HOSTENT_DATA buffer, 
			 int *h_errnop)
{
  ACE_TRACE ("ACE_OS::gethostbyname_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
#if ! defined (AIX)
  ACE_OSCALL_RETURN (::gethostbyname_r (name, result, buffer, 
					sizeof (ACE_HOSTENT_DATA), h_errnop), 
		     struct hostent *, 0);
#else
  ::memset (buffer, 0, sizeof buffer);

  if (::gethostbyname_r (name, result, (struct hostent_data *) buffer) == 0)
    return result;
  else
    {
      *h_errnop = h_errno;
      return (struct hostent *) 0;
    }
#endif /* ! defined (AIX) */
#else
  ACE_OSCALL_RETURN (::gethostbyname (name), struct hostent *, 0); 
#endif
}

INLINE int 
ACE_OS::getpeername (ACE_HANDLE handle, struct sockaddr *addr, 
		     int *addrlen)
{
  ACE_TRACE ("ACE_OS::getpeername");
  ACE_OSCALL_RETURN (::getpeername ((ACE_SOCKET) handle, addr, addrlen), int, -1);
}

INLINE char *
ACE_OS::gets (char *str)
{
  ACE_TRACE ("ACE_OS::gets");
  ACE_OSCALL_RETURN (::gets (str), char *, 0);
}

INLINE struct servent *
ACE_OS::getservbyname (const char *svc, const char *proto)
{
  ACE_TRACE ("ACE_OS::getservbyname");
  ACE_OSCALL_RETURN (::getservbyname (svc, proto), 
		     struct servent *, 0);
}

INLINE struct servent *
ACE_OS::getservbyname_r (const char *svc, const char *proto, 
			 struct servent *result, ACE_SERVENT_DATA buf)
{
  ACE_TRACE ("ACE_OS::getservbyname_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
#if ! defined (AIX)
  ACE_OSCALL_RETURN (::getservbyname_r (svc, proto, result, buf,
					sizeof (ACE_SERVENT_DATA)),
		     struct servent *, 0);
#else
  ::memset (buf, 0, sizeof buf);

  if (::getservbyname_r (svc, proto, result, (struct servent_data *) buf) == 0)
    return result;
  else
    return (struct servent *) 0;
#endif /* !defined (AIX) */
#else
  ACE_OSCALL_RETURN (::getservbyname (svc, proto), 
		     struct servent *, 0);
#endif
}

INLINE int 
ACE_OS::getsockname (ACE_HANDLE handle, 
		     struct sockaddr *addr, 
		     int *addrlen)
{
  ACE_TRACE ("ACE_OS::getsockname");

#if defined (ACE_HAS_INT_SOCKNAME) // AIX is weird...
  ACE_SOCKNAME_TYPE internal_addrlen = addrlen == 0 ? 0 : *addrlen;
#else
  ACE_SOCKNAME_TYPE internal_addrlen = addrlen;
#endif /* ACE_HAS_INT_SOCKNAME */
  ACE_OSCALL_RETURN (::getsockname ((ACE_SOCKET) handle, addr, internal_addrlen), int, -1);
}

INLINE int 
ACE_OS::getsockopt (ACE_HANDLE handle, int level, 
		    int optname, char *optval, int *optlen)
{
  ACE_TRACE ("ACE_OS::getsockopt");
  ACE_OSCALL_RETURN (::getsockopt ((ACE_SOCKET) handle, level, optname, optval, optlen), 
		     int, -1);
}

INLINE long 
ACE_OS::inet_addr (const char *name)
{
  ACE_TRACE ("ACE_OS::inet_addr");
  return ::inet_addr (name);
}

INLINE char *
ACE_OS::inet_ntoa (const struct in_addr addr)
{
  ACE_TRACE ("ACE_OS::inet_ntoa");
  ACE_OSCALL_RETURN (::inet_ntoa (addr), char *, 0);
}

INLINE int 
ACE_OS::listen (ACE_HANDLE handle, int backlog)
{
  ACE_TRACE ("ACE_OS::listen");
  ACE_OSCALL_RETURN (::listen ((ACE_SOCKET) handle, backlog), int, -1);
}

INLINE void
ACE_OS::perror (const char *s)
{
  ACE_TRACE ("ACE_OS::perror");
  ::perror (s);
}

INLINE int 
ACE_OS::printf (const char *format, ...)
{
  ACE_TRACE ("ACE_OS::printf");
  int result;
  va_list ap;
  va_start (ap, format);
  ACE_OSCALL (::vprintf (format, ap), int, -1, result);
  va_end (ap);
  return result;
}

INLINE int 
ACE_OS::puts (const char *s)
{
  ACE_TRACE ("ACE_OS::puts");
  ACE_OSCALL_RETURN (::puts (s), int, -1);
}

INLINE int 
ACE_OS::sema_destroy (sema_t *s)
{
  ACE_TRACE ("ACE_OS::sema_destroy");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sema_destroy (s), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  int r1 = ACE_OS::mutex_destroy (&s->lock_);
  int r2 = ACE_OS::cond_destroy (&s->count_nonzero_);
  return r1 != 0 || r2 != 0 ? -1 : 0;
#elif defined (ACE_HAS_WTHREADS)
  return ::CloseHandle (*s) == TRUE ? -1 : 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::sema_init (sema_t *s, u_int count, int type, const char *name, void *arg) 
{
  ACE_TRACE ("ACE_OS::sema_init");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sema_init (s, count, type, arg), _result), 
		     int, -1);
#elif defined (ACE_HAS_PTHREADS)
  int result = -1;

  if (ACE_OS::mutex_init (&s->lock_, type, name, arg) == 0
      && ACE_OS::cond_init (&s->count_nonzero_, type, name, arg) == 0
      && ACE_OS::mutex_lock (&s->lock_) == 0)
    {
      s->count_ = count;
      if (ACE_OS::mutex_unlock (&s->lock_) == 0)
	result = 0;
    }

  if (result == -1)
    {
      ACE_OS::mutex_destroy (&s->lock_);
      ACE_OS::cond_destroy (&s->count_nonzero_);
    }
  return result;
#elif defined (ACE_HAS_WTHREADS)
  // Create the semaphore with its value initialized to 1.
  *s = ::CreateSemaphore (0, count, 0x7FFFFFFF, name);
  return (*s == 0) ? -1 : 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::sema_post (sema_t *s)
{
  ACE_TRACE ("ACE_OS::sema_post");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sema_post (s), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  if (ACE_OS::mutex_lock (&s->lock_) != 0)
    return -1;

  // Allow waiter to continue.
  if (s->count_ == 0)
    ACE_OS::cond_signal (&s->count_nonzero_);

  ++s->count_;

  return ACE_OS::mutex_unlock (&s->lock_);
#elif defined (ACE_HAS_WTHREADS)
  return ::ReleaseSemaphore (*s, 1, 0);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int
ACE_OS::sema_trywait (sema_t *s)
{
  ACE_TRACE ("ACE_OS::sema_trywait");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sema_trywait (s), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  if (ACE_OS::mutex_lock (&s->lock_) != 0)
    return -1;

  int result = 0;

  if (s->count_ > 0)
    --s->count_;
  else
    {
      errno = EBUSY;
      result = -1;
    }

  ACE_OS::mutex_unlock (&s->lock_);
  return result;
#elif defined (ACE_HAS_WTHREADS)
  int result = ::WaitForSingleObject (*s, 0);

  if (result == WAIT_TIMEOUT) 
    errno = EBUSY;

  return result == WAIT_OBJECT_0 ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::sema_wait (sema_t *s)
{
  ACE_TRACE ("ACE_OS::sema_wait");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sema_wait (s), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ::pthread_cleanup_push (ACE_OS::mutex_lock_cleanup, (void *) &s->lock_);

  if (ACE_OS::mutex_lock (&s->lock_) != 0)
    {
      ::pthread_cleanup_pop (0);
      return -1;
    }

  while (s->count_ == 0)
    if (ACE_OS::cond_wait (&s->count_nonzero_, &s->lock_) == -1)
      {
	::pthread_cleanup_pop (1);
	return -1;
      }

  --s->count_;

  int result = ACE_OS::mutex_unlock (&s->lock_);
  ::pthread_cleanup_pop (1);
  return result;
#elif defined (ACE_HAS_WTHREADS)
  return ::WaitForSingleObject (*s, INFINITE) == WAIT_OBJECT_0 ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::setsockopt (ACE_HANDLE handle, int level, int optname, 
		    const char *optval, int optlen)
{
  ACE_TRACE ("ACE_OS::setsockopt");

  ACE_SOCKOPT_TYPE1 internal_optval = (ACE_SOCKOPT_TYPE1) optval;
#if defined (ACE_HAS_SIZET_SOCKOPT)
  ACE_SOCKOPT_TYPE2 internal_optlen = (ACE_SOCKOPT_TYPE2) &optlen;
#else
  ACE_SOCKOPT_TYPE2 internal_optlen = optlen;
#endif /* ACE_HAS_SIZET_SOCKOPT */
  ACE_OSCALL_RETURN (::setsockopt ((ACE_SOCKET) handle, level, optname, 
				   internal_optval, internal_optlen),
		     int, -1);
}

INLINE int 
ACE_OS::shutdown (ACE_HANDLE handle, int how)
{
  ACE_TRACE ("ACE_OS::shutdown");
  ACE_OSCALL_RETURN (::shutdown ((ACE_SOCKET) handle, how), int, -1);
}

INLINE ACE_SignalHandler
ACE_OS::signal (int signum, ACE_SignalHandler func)
{
  return ::signal (signum, func);
}

INLINE int 
ACE_OS::sprintf (char *buf, const char *format, ...)
{
  ACE_TRACE ("ACE_OS::sprintf");
  int result;
  va_list ap;
  va_start (ap, format);
  ACE_OSCALL (::vsprintf (buf, format, ap), int, -1, result);
  va_end (ap);
  return result;
}

INLINE char *
ACE_OS::strcat (char *s, const char *t)
{
  ACE_TRACE ("ACE_OS::strcat");
  ACE_OSCALL_RETURN (::strcat (s, t), char *, 0);
}

INLINE char *
ACE_OS::strchr (const char *s, int c)
{
  ACE_TRACE ("ACE_OS::strchr");
  ACE_OSCALL_RETURN (::strchr (s, c), char *, 0);
}

INLINE int 
ACE_OS::strcmp (const char *s, const char *t)
{
  ACE_TRACE ("ACE_OS::strcmp");
  ACE_OSCALL_RETURN (::strcmp (s, t), int, -1);
}

INLINE char *
ACE_OS::strcpy (char *s, const char *t)
{
  ACE_TRACE ("ACE_OS::strcpy");
  ACE_OSCALL_RETURN (::strcpy (s, t), char *, 0);
}

INLINE char *
ACE_OS::strdup (const char *s)
{
  ACE_TRACE ("ACE_OS::strdup");
  ACE_OSCALL_RETURN (::strdup (s), char *, 0);
}

INLINE size_t 
ACE_OS::strlen (const char *s)
{
  ACE_TRACE ("ACE_OS::strlen");
  ACE_OSCALL_RETURN (::strlen (s), int, -1);
}

INLINE int 
ACE_OS::strncmp (const char *s, const char *t, size_t len)
{
  ACE_TRACE ("ACE_OS::strncmp");
  ACE_OSCALL_RETURN (::strncmp (s, t, len), int, -1);
}

INLINE char *
ACE_OS::strncpy (char *s, const char *t, size_t len)
{
  ACE_TRACE ("ACE_OS::strncpy");
  return ::strncpy (s, t, len);
}

INLINE char *
ACE_OS::strtok (char *s, const char *tokens)
{
  ACE_TRACE ("ACE_OS::strtok");
  return ::strtok (s, tokens);
}

INLINE long 
ACE_OS::strtol (const char *s, char **ptr, int base)
{
  ACE_TRACE ("ACE_OS::strtol");
  return ::strtol (s, ptr, base);
}

INLINE int 
ACE_OS::system (const char *s)
{
  ACE_TRACE ("ACE_OS::system");
  ACE_OSCALL_RETURN (::system (s), int, -1);
}

INLINE int 
ACE_OS::thr_continue (hthread_t target_thread)
{
  ACE_TRACE ("ACE_OS::thr_continue");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_continue (target_thread), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  return ::ResumeThread (target_thread) != 0xFFFFFFFF ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int
ACE_OS::thr_equal (thread_t t1, thread_t t2)
{
#if defined (ACE_HAS_PTHREADS)
#if defined (pthread_equal)
// If it's a macro we can't say "::pthread_equal"...
  return pthread_equal (t1, t2);
#else
  return ::pthread_equal (t1, t2);
#endif /* pthread_equal */
#else // For both STHREADS and WTHREADS...
// Hum, Do we need to treat WTHREAD differently?
  return t1 == t2;
#endif /* ACE_HAS_PTHREADS */
}

INLINE void 
ACE_OS::thr_exit (void *status)
{
  ACE_TRACE ("ACE_OS::thr_exit");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ::thr_exit (status);
#elif defined (ACE_HAS_PTHREADS)
  ::pthread_exit (status);
#elif defined (ACE_HAS_WTHREADS)
  ACE_OS::thr_destroy_tss ();
  ::ExitThread ((DWORD) status);
#endif /* ACE_HAS_STHREADS */
#else
  ;
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_getconcurrency (void)
{
  ACE_TRACE ("ACE_OS::thr_getconcurrency");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_getconcurrency (), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_getprio (hthread_t thr_id, int *prio)
{
  ACE_TRACE ("ACE_OS::thr_getprio");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_getprio (thr_id, prio), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  return ::GetThreadPriority (thr_id);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_getspecific (thread_key_t key, void **data)
{
  ACE_TRACE ("ACE_OS::thr_getspecific");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_getspecific (key, data), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  *data = ::pthread_getspecific (key);
  return 0;
#elif defined (ACE_HAS_WTHREADS)
  *data = ::TlsGetValue (key);
  return 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_join (thread_t waiter_id, thread_t *thr_id, void **status)
{
  ACE_TRACE ("ACE_OS::thr_join");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_join (waiter_id, thr_id, status), _result), 
		     int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_join (waiter_id, status), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_join (hthread_t thr_id, void **status)
{
  ACE_TRACE ("ACE_OS::thr_join");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  return ::WaitForSingleObject (thr_id, INFINITE) == WAIT_OBJECT_0
    ? (::GetExitCodeThread (thr_id, (LPDWORD) status) ? 0 : -1) : 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int
ACE_OS::thr_setcancelstate (int new_state, int *old_state)
{
  ACE_TRACE ("ACE_OS::thr_setcancelstate");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_setcancelstate (new_state, old_state), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE int
ACE_OS::thr_setcanceltype (int new_type, int *old_type)
{
  ACE_TRACE ("ACE_OS::thr_setcanceltype");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_setcanceltype (new_type, old_type), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE int
ACE_OS::thr_cancel (thread_t t_id)
{
  ACE_TRACE ("ACE_OS::thr_cancel");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_cancel(t_id), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE int
ACE_OS::sigwait (sigset_t *set, int *sig)
{
  ACE_TRACE ("ACE_OS::sigwait");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sigwait (set), _result), 
		     int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sigwait (set, sig), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE void
ACE_OS::thr_testcancel (void)
{
  ACE_TRACE ("ACE_OS::thr_testcancel");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
#elif defined (ACE_HAS_PTHREADS)
  ::pthread_testcancel ();
#elif defined (ACE_HAS_WTHREADS)
#endif /* ACE_HAS_STHREADS */
#else
#endif /* ACE_HAS_THREADS */		
}

INLINE int
ACE_OS::thr_sigsetmask (int signum,
			const sigset_t *nsm,
			sigset_t *osm)
{
  ACE_TRACE ("ACE_OS::thr_sigsetmask");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_sigsetmask (signum, nsm, osm), _result),
		     int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::sigthreadmask (signum, nsm, osm), _result),
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE size_t
ACE_OS::thr_min_stack (void)
{
  ACE_TRACE ("ACE_OS::thr_min_stack");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_min_stack (), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  return PTHREAD_STACK_MIN;
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_THREADS */		
}

INLINE int
ACE_OS::thr_kill (thread_t thr_id, int signum)
{
  ACE_TRACE ("ACE_OS::thr_kill");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_kill (thr_id, signum), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_kill (thr_id, signum), _result), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		
}

INLINE void 
ACE_OS::thr_self (hthread_t &self)
{
  ACE_TRACE ("ACE_OS::thr_self");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  self = ::thr_self ();
#elif defined (ACE_HAS_PTHREADS)
  self = ::thread_self ();
#elif defined (ACE_HAS_WTHREADS)
  self = ::GetCurrentThread ();
#endif /* ACE_HAS_STHREADS */
#else
#endif /* ACE_HAS_THREADS */		     
}

INLINE thread_t 
ACE_OS::thr_self (void)
{
  ACE_TRACE ("ACE_OS::thr_self");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (::thr_self (), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (::pthread_self (), int, -1);
#elif defined (ACE_HAS_WTHREADS)
  return ::GetCurrentThreadId ();
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_setconcurrency (int hint)
{
  ACE_TRACE ("ACE_OS::thr_setconcurrency");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_setconcurrency (hint), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_setprio (hthread_t thr_id, int prio)
{
  ACE_TRACE ("ACE_OS::thr_setprio");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_setprio (thr_id, prio), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  return ::SetThreadPriority (thr_id, prio) ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_setspecific (thread_key_t key, void *data)
{
  ACE_TRACE ("ACE_OS::thr_setspecific");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_setspecific (key, data), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::pthread_setspecific (key, data), _result), 
		     int, -1);
#elif defined (ACE_HAS_WTHREADS)
  ::TlsSetValue (key, data);
  return 0;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE int 
ACE_OS::thr_suspend (hthread_t target_thread)
{
  ACE_TRACE ("ACE_OS::thr_suspend");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ACE_OSCALL_RETURN (ACE_ADAPT_RETVAL (::thr_suspend (target_thread), _result), int, -1);
#elif defined (ACE_HAS_PTHREADS)
  ACE_NOTSUP_RETURN (-1);
#elif defined (ACE_HAS_WTHREADS)
  return ::SuspendThread (target_thread) != 0xFFFFFFFF ? 0 : -1;
#endif /* ACE_HAS_STHREADS */
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_THREADS */		     
}

INLINE void 
ACE_OS::thr_yield (void)
{
  ACE_TRACE ("ACE_OS::thr_yield");
#if defined (ACE_HAS_THREADS)
#if defined (ACE_HAS_STHREADS)
  ::thr_yield ();
#elif defined (ACE_HAS_PTHREADS)
  ::pthread_yield ();
#elif defined (ACE_HAS_WTHREADS)
  ::Sleep (0);
#endif /* ACE_HAS_STHREADS */
#else
  ;
#endif /* ACE_HAS_THREADS */		     
}

INLINE void 
ACE_OS::rewind (FILE *fp)
{
  ACE_TRACE ("ACE_OS::rewind");
  ::rewind (fp);
}

// This function returns the number of bytes in the file referenced by
// FD.

INLINE long
ACE_OS::filesize (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE::filesize");
#if defined (ACE_WIN32)
  return ::GetFileSize (handle, NULL);
#else /* !ACE_WIN32 */
  struct stat sb;

  return ACE_OS::fstat (handle, &sb) < 0 ? -1 : sb.st_size;
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::writev (ACE_HANDLE handle, 
		const struct iovec *iov, 
		int iovcnt)
{
  ACE_TRACE ("ACE_OS::writev");
  ACE_OSCALL_RETURN (::writev (handle, iov, iovcnt), int, -1);
}

INLINE ssize_t
ACE_OS::readv (ACE_HANDLE handle, 
	       struct iovec *iov, 
	       int iovlen)
{
  ACE_TRACE ("ACE_OS::readv");
  ACE_OSCALL_RETURN (::readv (handle, iov, iovlen), ssize_t, -1);
}

INLINE ACE_Time_Value
ACE_OS::gettimeofday (void)
{
  ACE_TRACE ("ACE_OS::gettimeofday");
  timeval tv;
#if 0
  // From Todd Montgomery...
  struct _timeb tb;
  ::_ftime (&tb);
  tv.tv_sec = tb.time;
  tv.tv_usec = 1000 * tb.millitm;
#endif /* 0 */
#if defined (ACE_WIN32)
  // Alternative form.
  SYSTEMTIME system_time;
  FILETIME   file_time;

  ::GetSystemTime (&system_time);
  ::SystemTimeToFileTime (&system_time, &file_time);
  QWORD _100ns = ACE_MAKE_QWORD (file_time.dwLowDateTime, 
				 file_time.dwHighDateTime);
  // Convert 100ns units to seconds;
  tv.tv_sec = long (_100ns / (10000 * 1000));
  // Convert remainder to microseconds;
  tv.tv_usec = long ((_100ns - (tv.tv_sec * (10000 * 1000))) * 10);
#else
  int result;
#if defined (ACE_HAS_SVR4_GETTIMEOFDAY)
  ACE_OSCALL (::gettimeofday (&tv, 0), int, -1, result);
#else
  ACE_OSCALL (::gettimeofday (&tv), int, -1, result);
#endif /* ACE_HAS_SVR4_GETTIMEOFDAY */
#endif /* ACE_WIN32 */
  return ACE_Time_Value (tv);
}

INLINE int 
ACE_OS::poll (struct pollfd *pollfds, u_long len, ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_OS::poll");
#if defined (ACE_HAS_POLL)
  int to = timeout == 0 ? -1 : int (timeout->msec ());
  ACE_OSCALL_RETURN (::poll (pollfds, len, to), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_POLL */
}

INLINE int 
ACE_OS::poll (struct pollfd *pollfds, u_long len, const ACE_Time_Value &timeout)
{
  ACE_TRACE ("ACE_OS::poll");
#if defined (ACE_HAS_POLL)
  ACE_OSCALL_RETURN (::poll (pollfds, len, int (timeout.msec ())), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_POLL */
}

INLINE int 
ACE_OS::t_accept (ACE_HANDLE handle, int reshandle, 
		  struct t_call *call)
{
  ACE_TRACE ("ACE_OS::t_accept");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_accept (handle, reshandle, call), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE char *
ACE_OS::t_alloc (ACE_HANDLE handle, int struct_type, 
	     int fields)
{
  ACE_TRACE ("ACE_OS::t_alloc");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_alloc (handle, struct_type, fields), 
		     char *, 0);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_bind (ACE_HANDLE handle, struct t_bind *req, 
	    struct t_bind *ret)
{
  ACE_TRACE ("ACE_OS::t_bind");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_bind (handle, req, ret), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_close (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_close");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_close (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_connect(int fildes, 
		  struct t_call *sndcall,
		  struct t_call *rcvcall)
{
  ACE_TRACE ("ACE_OS::t_connect");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_connect (fildes, sndcall, rcvcall), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE void 
ACE_OS::t_error (char *errmsg)
{
  ACE_TRACE ("ACE_OS::t_error");
#if defined (ACE_HAS_TLI)
  ::t_error (errmsg);
#else
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_free (char *ptr, int struct_type)
{
  ACE_TRACE ("ACE_OS::t_free");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_free (ptr, struct_type), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_getinfo (ACE_HANDLE handle, struct t_info *info)
{
  ACE_TRACE ("ACE_OS::t_getinfo");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_getinfo (handle, info), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_getname (ACE_HANDLE handle, 
	       struct netbuf *namep, 
	       int type)
{
  ACE_TRACE ("ACE_OS::t_getname");
#if defined (ACE_HAS_SVR4_TLI)
  ACE_OSCALL_RETURN (::t_getname (handle, namep, type), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int 
ACE_OS::t_getstate (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_getstate");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_getstate (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_listen (ACE_HANDLE handle, struct t_call *call)
{
  ACE_TRACE ("ACE_OS::t_listen");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_listen (handle, call), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_look (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_look");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_look (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_open (char *path, int oflag, struct t_info *info)
{
  ACE_TRACE ("ACE_OS::t_open");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_open (path, oflag, info), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_optmgmt (ACE_HANDLE handle, 
	       struct t_optmgmt *req, 
	       struct t_optmgmt *ret)
{
  ACE_TRACE ("ACE_OS::t_optmgmt");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_optmgmt (handle, req, ret), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_rcv (ACE_HANDLE handle, 
	   char *buf, 
	   unsigned nbytes, 
	   int *flags)
{
  ACE_TRACE ("ACE_OS::t_rcv");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_rcv (handle, buf, nbytes, flags), 
		     int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_rcvdis (ACE_HANDLE handle, struct t_discon *discon)
{
  ACE_TRACE ("ACE_OS::t_rcvdis");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_rcvdis (handle, discon), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_rcvrel (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_rcvrel");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_rcvrel (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_rcvudata (ACE_HANDLE handle, 
		struct t_unitdata *unitdata, 
		int *flags)
{
  ACE_TRACE ("ACE_OS::t_rcvudata");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_rcvudata (handle, unitdata, flags), 
		     int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_rcvuderr (ACE_HANDLE handle, struct t_uderr *uderr)
{
  ACE_TRACE ("ACE_OS::t_rcvuderr");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_rcvuderr (handle, uderr), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_snd (ACE_HANDLE handle, char *buf, unsigned nbytes, int flags)
{
  ACE_TRACE ("ACE_OS::t_snd");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_snd (handle, buf, nbytes, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_snddis (ACE_HANDLE handle, struct t_call *call)
{
  ACE_TRACE ("ACE_OS::t_snddis");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_snddis (handle, call), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_sndrel (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_sndrel");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_sndrel (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_sync (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_sync");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_sync (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE int 
ACE_OS::t_unbind (ACE_HANDLE handle)
{
  ACE_TRACE ("ACE_OS::t_unbind");
#if defined (ACE_HAS_TLI)
  ACE_OSCALL_RETURN (::t_unbind (handle), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_TLI */
}

INLINE char * 
ACE_OS::compile (const char *instring, char *expbuf, char *endbuf)
{
  ACE_TRACE ("ACE_OS::compile");
#if defined (ACE_HAS_REGEX)
  ACE_OSCALL_RETURN (::compile (instring, expbuf, endbuf), char *, 0);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_REGEX */		     
  return 0;
}

INLINE int 
ACE_OS::close (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::close");
#if defined (ACE_WIN32)  
  return ::CloseHandle (fd) ? 0 : -1;
#else
  ACE_OSCALL_RETURN (::close (fd), int, -1);
#endif /* ACE_WIN32 */
}

INLINE ACE_HANDLE
ACE_OS::creat (const char *filename, mode_t mode)
{
  ACE_TRACE ("ACE_OS::creat");
#if defined (ACE_WIN32)  
  ACE_OSCALL_RETURN (ACE_OS::open (filename, mode), ACE_HANDLE, -1);
#else
  ACE_OSCALL_RETURN (::creat (filename, mode), ACE_HANDLE, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::uname (struct utsname *name)
{
  ACE_TRACE ("ACE_OS::uname");
#if defined (ACE_WIN32)
  size_t maxnamelen = sizeof name->nodename;
  ::strcpy (name->sysname, "Win32");
  // Any ideas what these should be?
  ::strcpy (name->release, "???");
  ::strcpy (name->version, "???");
  ::strcpy (name->machine, "???");
  return ::GetComputerName (name->nodename, LPDWORD (&maxnamelen));
#else
  ACE_OSCALL_RETURN (::uname (name), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::hostname (char name[], size_t maxnamelen)
{
  ACE_TRACE ("ACE_OS::uname");
#if defined (ACE_WIN32)
  return ::GetComputerName (name, LPDWORD (&maxnamelen)) ? 0 : -1;
#else /* !ACE_WIN32 */
  struct utsname host_info;
  
  if (ACE_OS::uname (&host_info) == -1)
    return -1;
  else
    {
      ACE_OS::strncpy (name, host_info.nodename, maxnamelen);
      return 0;
    }
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::msgctl (int msqid, int cmd, msqid_ds *val)
{
  ACE_TRACE ("ACE_OS::msgctl");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::msgctl (msqid, cmd, val), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::msgget (key_t key, int msgflg)
{
  ACE_TRACE ("ACE_OS::msgget");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::msgget (key, msgflg), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int
ACE_OS::msgrcv (int int_id, void *buf, size_t len, 
		long type, int flags)
{
  ACE_TRACE ("ACE_OS::msgrcv");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::msgrcv (int_id, buf, len, type, flags), 
		     int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::msgsnd (int int_id, const void *buf, size_t len, int flags)
{
  ACE_TRACE ("ACE_OS::msgsnd");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::msgsnd (int_id, buf, len, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE u_int 
ACE_OS::alarm (u_int delay)
{
  ACE_TRACE ("ACE_OS::alarm");

#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (0);
#else
  return ::alarm (delay);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::dlclose (void *handle)
{
  ACE_TRACE ("ACE_OS::dlclose");
#if defined (ACE_HAS_SVR4_DYNAMIC_LINKING)

#if !defined (ACE_HAS_AUTOMATIC_INIT_FINI)
  // SunOS4 does not automatically call _fini()! 
  void *ptr;

  ACE_OSCALL (::dlsym (handle, "_fini"), void *, 0, ptr);

  if (ptr != 0) 
    (*((int (*)(void)) ptr)) (); // Call _fini hook explicitly.
#endif /* ACE_HAS_AUTOMATIC_INIT_FINI */
  ACE_OSCALL_RETURN (::dlclose (handle), int, -1);
#elif defined (ACE_WIN32)
  return ::FreeLibrary (handle);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SVR4_DYNAMIC_LINKING */
}

INLINE char *
ACE_OS::dlerror (void)
{
  ACE_TRACE ("ACE_OS::dlerror");
#if defined (ACE_HAS_SVR4_DYNAMIC_LINKING)
  ACE_OSCALL_RETURN (::dlerror (), char *, 0);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_SVR4_DYNAMIC_LINKING */
}

INLINE void *
ACE_OS::dlopen (ACE_DL_TYPE filename, int mode)
{
  ACE_TRACE ("ACE_OS::dlopen");
#if defined (ACE_HAS_SVR4_DYNAMIC_LINKING)
  void *handle;
  ACE_OSCALL (::dlopen (filename, mode), void *, 0, handle);
#if !defined (ACE_HAS_AUTOMATIC_INIT_FINI)
  // Some systems (e.g., SunOS4) do not automatically call _init(), so
  // we'll have to call it manually.

  void *ptr;

  ACE_OSCALL (::dlsym (handle, "_init"), void *, 0, ptr);

  if (ptr != 0 && (*((int (*)(void)) ptr)) () == -1) // Call _init hook explicitly.
    return 0; 
#endif /* ACE_HAS_AUTOMATIC_INIT_FINI */
  return handle;
#elif defined (ACE_WIN32)
  return ::LoadLibrary (filename);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_SVR4_DYNAMIC_LINKING */
}

INLINE void *
ACE_OS::dlsym (void *handle, ACE_DL_TYPE symbolname)
{
  ACE_TRACE ("ACE_OS::dlsym");
#if defined (ACE_HAS_SVR4_DYNAMIC_LINKING)
  ACE_OSCALL_RETURN (::dlsym (handle, symbolname), void *, 0);
#elif defined (ACE_WIN32)
  return ::GetProcAddress (handle, symbolname);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_SVR4_DYNAMIC_LINKING */
}

INLINE void 
ACE_OS::exit (int status)
{
  ACE_TRACE ("ACE_OS::exit");
#if defined (ACE_WIN32)
  ::ExitProcess ((UINT) status);
#else
  ::exit (status);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::step (const char *str, char *expbuf)
{
  ACE_TRACE ("ACE_OS::step");
#if defined (ACE_HAS_REGEX)
  ACE_OSCALL_RETURN (::step (str, expbuf), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_REGEX */		     
}

INLINE long 
ACE_OS::sysinfo (int cmd, char *buf, long count)
{
  ACE_TRACE ("ACE_OS::sysinfo");
#if defined (ACE_HAS_SYSINFO)
  ACE_OSCALL_RETURN (::sysinfo (cmd, buf, count), long, -1);
#else
  ACE_NOTSUP_RETURN (0);
#endif /* ACE_HAS_SYSINFO */
}

INLINE ssize_t 
ACE_OS::write (ACE_HANDLE handle, const void *buf, size_t nbyte)
{
  ACE_TRACE ("ACE_OS::write");
#if defined (ACE_WIN32)
  DWORD bytes_written; // This is set to 0 byte WriteFile.

  if (::WriteFile (handle, buf, nbyte, &bytes_written, 0))
    return (ssize_t) bytes_written;
  else
    return -1;
#else
  ACE_OSCALL_RETURN (::write (handle, buf, nbyte), ssize_t, -1);
#endif /* ACE_WIN32 */
}

INLINE ssize_t 
ACE_OS::read (ACE_HANDLE handle, void *buf, size_t len)
{
  ACE_TRACE ("ACE_OS::read");
#if defined (ACE_WIN32)
  DWORD ok_len;
  return ::ReadFile (handle, buf, len, &ok_len, 0) ? (ssize_t) ok_len : -1;
#else
  ACE_OSCALL_RETURN (::read (handle, buf, len), ssize_t, -1);
#endif /* ACE_WIN32 */
}

INLINE ACE_HANDLE
ACE_OS::socket (int domain, int type, int proto)
{
  ACE_TRACE ("ACE_OS::socket");
#if defined (ACE_WIN32)
  ACE_SOCKET s = ::socket (domain, type, proto);
  return s == INVALID_SOCKET ? ACE_INVALID_HANDLE : (ACE_HANDLE) s;
#else
  ACE_OSCALL_RETURN (::socket (domain, type, proto), ACE_HANDLE, ACE_INVALID_HANDLE);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::getmsg (ACE_HANDLE handle, struct strbuf *ctl, 
		struct strbuf *data, int *flags) 
{
  ACE_TRACE ("ACE_OS::getmsg");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::getmsg (handle, ctl, data, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int
ACE_OS::getpmsg (ACE_HANDLE handle, struct strbuf *ctl, 
	     struct strbuf *data, int *band, int *flags)
{
  ACE_TRACE ("ACE_OS::getpmsg");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::getpmsg (handle, ctl, data, band, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int 
ACE_OS::getrusage (ACE_HANDLE who, struct rusage *ru)
{
  ACE_TRACE ("ACE_OS::getrusage");

#if defined (ACE_HAS_SYSCALL_GETRUSAGE)
// This nonsense is necessary for HP/UX...
  ACE_OSCALL_RETURN (::syscall (SYS_GETRUSAGE, who, ru), int, -1);
#elif defined (ACE_HAS_GETRUSAGE)
  ACE_OSCALL_RETURN (::getrusage (who, ru), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);  
#endif /* ACE_HAS_SYSCALL_GETRUSAGE */
}

INLINE int 
ACE_OS::isastream (ACE_HANDLE fd)
{
  ACE_TRACE ("ACE_OS::isastream");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::isastream (fd), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE void *
ACE_OS::mmap (void *addr, size_t len, 
	      int prot, int flags, ACE_HANDLE file_handle, 
	      off_t off, ACE_HANDLE *file_mapping)
{
  ACE_TRACE ("ACE_OS::mmap");
#if defined (ACE_WIN32)
  if (file_mapping == 0)
    {
      errno = EINVAL;
      return 0;
    }
  *file_mapping = ::CreateFileMapping (file_handle, 0, prot, 0, len, 0);
  if (*file_mapping == 0)
    return 0;
  return (void *) ::MapViewOfFile (*file_mapping, flags, 0, off, len);
#else
  ACE_OSCALL_RETURN ((void *) ::mmap ((ACE_MMAP_TYPE) addr, len, 
				      prot, flags, file_handle, off),
		     void *, (void *) -1);
#endif /*ACE_WIN32 */
}

INLINE int 
ACE_OS::mprotect (void *addr, size_t len, int prot)
{
  ACE_TRACE ("ACE_OS::mprotect");
#if defined (ACE_WIN32)
  // @@ I'm not sure what goes here...
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_OSCALL_RETURN (::mprotect ((ACE_MMAP_TYPE) addr, len, prot), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::msync (void *addr, size_t len, int sync)
{
  ACE_TRACE ("ACE_OS::msync");
#if defined (ACE_WIN32)
  return ::FlushViewOfFile (addr, len);
#elif !defined (ACE_LACKS_MSYNC)
  ACE_OSCALL_RETURN (::msync ((ACE_MMAP_TYPE) addr, len, sync), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::munmap (void *addr, size_t len)
{
  ACE_TRACE ("ACE_OS::munmap");
#if defined (ACE_WIN32)
  return ::UnmapViewOfFile (addr);
#else
  ACE_OSCALL_RETURN (::munmap ((ACE_MMAP_TYPE) addr, len), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::madvise (caddr_t addr, size_t len, int advice)
{
  ACE_TRACE ("ACE_OS::madvise");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#elif !defined (ACE_LACKS_MADVISE)
  ACE_OSCALL_RETURN (::madvise (addr, len, advice), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::putmsg (ACE_HANDLE handle, const struct strbuf *ctl, 
		const struct strbuf *data, int flags) 
{
  ACE_TRACE ("ACE_OS::putmsg");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::putmsg (handle, ctl, data, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int
ACE_OS::putpmsg (ACE_HANDLE handle, const struct strbuf *ctl, 
		 const struct strbuf *data, int band, int flags)
{
  ACE_TRACE ("ACE_OS::putpmsg");
#if defined (ACE_HAS_STREAM_PIPES)
  ACE_OSCALL_RETURN (::putpmsg (handle, ctl, data, band, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_STREAM_PIPES */
}

INLINE int 
ACE_OS::semctl (int int_id, int semnum, int cmd, semun value)
{
  ACE_TRACE ("ACE_OS::semctl");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::semctl (int_id, semnum, cmd, value), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::semget (key_t key, int nsems, int flags)
{
  ACE_TRACE ("ACE_OS::semget");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::semget (key, nsems, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::semop (int int_id, struct sembuf *sops, size_t nsops)
{
  ACE_TRACE ("ACE_OS::semop");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::semop (int_id, sops, nsops), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE void *
ACE_OS::shmat (int int_id, void *shmaddr, int shmflg)
{
  ACE_TRACE ("ACE_OS::shmat");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::shmat (int_id, shmaddr, shmflg), void *, (void *) -1);
#else
  ACE_NOTSUP_RETURN ((void *) -1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::shmctl (int int_id, int cmd, struct shmid_ds *buf)
{
  ACE_TRACE ("ACE_OS::shmctl");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::shmctl (int_id, cmd, buf), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::shmdt (void *shmaddr)
{
  ACE_TRACE ("ACE_OS::shmdt");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::shmdt ((char *) shmaddr), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE int 
ACE_OS::shmget (key_t key, int size, int flags)
{
  ACE_TRACE ("ACE_OS::shmget");
#if defined (ACE_HAS_SYSV_IPC)
  ACE_OSCALL_RETURN (::shmget (key, size, flags), int, -1);
#else
  ACE_NOTSUP_RETURN (-1);
#endif /* ACE_HAS_SYSV_IPC */
}

INLINE ACE_HANDLE 
ACE_OS::open (const char *filename, int mode, int perms)
{
  ACE_TRACE ("ACE_OS::open");
#if defined (ACE_WIN32)
  // Warning: This function ignores _O_APPEND

  DWORD access = ((mode & _O_RDONLY) ? GENERIC_READ : 0)
               | ((mode & _O_WRONLY) ? GENERIC_WRITE : 0)
               | (((mode & _O_RDWR) == _O_RDWR) ? (GENERIC_READ | GENERIC_WRITE) : 0);

  DWORD creation;
  if ((mode & (_O_CREAT | _O_EXCL)) == (_O_CREAT | _O_EXCL))
    creation = CREATE_NEW;
  else if (mode & _O_CREAT)
    creation = CREATE_ALWAYS;
  else if (mode & _O_TRUNC)
    creation = TRUNCATE_EXISTING;
  else if (access & GENERIC_WRITE)
    creation = OPEN_ALWAYS;
  else
    creation = OPEN_EXISTING;

  return ::CreateFile (filename, access, (FILE_SHARE_READ | FILE_SHARE_WRITE),
		       0, creation, (mode & _O_TEMPORARY) ? FILE_FLAG_DELETE_ON_CLOSE : 0, 0);
#else
  ACE_OSCALL_RETURN (::open (filename, mode, perms), ACE_HANDLE, -1);
#endif /* ACE_WIN32 */
}

INLINE char *
ACE_OS::ctime (const time_t *t)
{
  ACE_TRACE ("ACE_OS::ctime");
  ACE_OSCALL_RETURN (::ctime (t), char *, 0);
}

INLINE char *
ACE_OS::ctime_r (const time_t *t, char *buf, int buflen)
{
  ACE_TRACE ("ACE_OS::ctime_r");
  char *result;
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
#if ! defined (AIX)
  ACE_OSCALL (::ctime_r (t, buf, buflen), char *, 0, result);
#else
  ACE_OSCALL (::ctime_r (t, buf), char *, 0, result);
#endif /* !defined (AIX) */
#else
  ACE_OSCALL (::ctime (t), char *, 0, result);
#endif
  ::strncpy (buf, result, buflen);
  return buf;
}

INLINE struct tm *
ACE_OS::localtime (const time_t *t)
{
  ACE_TRACE ("ACE_OS::localtime");
  ACE_OSCALL_RETURN (::localtime (t), struct tm *, 0);
}

INLINE struct tm *
ACE_OS::localtime_r (const time_t *t, struct tm *res)
{
  ACE_TRACE ("ACE_OS::localtime_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
  ACE_OSCALL_RETURN (::localtime_r (t, res), struct tm *, 0);
#else
  ACE_OSCALL_RETURN (::localtime (t), struct tm *, 0);
#endif
}

INLINE char *
ACE_OS::asctime (const struct tm *t)
{
  ACE_TRACE ("ACE_OS::asctime");
  ACE_OSCALL_RETURN (::asctime (t), char *, 0);
}

INLINE char *
ACE_OS::asctime_r (const struct tm *t, char *buf, int buflen)
{
  ACE_TRACE ("ACE_OS::asctime_r");
#if defined (ACE_HAS_REENTRANT_FUNCTIONS) && defined (ACE_MT_SAFE)
#if !defined (AIX)
  ACE_OSCALL_RETURN (::asctime_r (t, buf, buflen), char *, 0);
#else
  ACE_OSCALL_RETURN (::asctime_r (t, buf), char *, 0);
#endif /* !defined (AIX) */
#else
  ACE_OSCALL_RETURN (::asctime (t), char *, 0);
#endif
}

INLINE int 
ACE_OS::flock_init (ACE_OS::flock_t *lock, 
		    int flags, 
		    const char *name, 
		    mode_t perms)
{
  ACE_TRACE ("ACE_OS::flock_init");
#if defined (ACE_WIN32)
  // Once initialized, these values are never changed.
  lock->overlapped_.Internal = 0;
  lock->overlapped_.InternalHigh = 0;
  lock->overlapped_.OffsetHigh = 0;
  lock->overlapped_.hEvent = INVALID_HANDLE_VALUE;
#endif /* ACE_WIN32 */
  lock->handle_ = ACE_INVALID_HANDLE;

  if (name != 0)
    {
      ACE_OSCALL (ACE_OS::open (name, flags, perms), ACE_HANDLE, ACE_INVALID_HANDLE, lock->handle_);
      return lock->handle_ == ACE_INVALID_HANDLE ? -1 : 0;
    }
  else
    return 0;
}

INLINE int 
ACE_OS::flock_wrlock (ACE_OS::flock_t *lock, short whence, off_t start, off_t len)
{
  ACE_TRACE ("ACE_OS::flock_wrlock");
#if defined (ACE_WIN32)
  lock->overlapped_.Offset = start;
  return ::LockFileEx (lock->handle_, LOCKFILE_EXCLUSIVE_LOCK, 0, len, 0, &lock->overlapped_) ? 0 : -1;
#else
  lock->lock_.l_whence = whence;
  lock->lock_.l_start = start;
  lock->lock_.l_len = len;
  lock->lock_.l_type = F_WRLCK;         // set write lock
  // block, if no access
  ACE_OSCALL_RETURN (::fcntl (lock->handle_, F_SETLKW, &lock->lock_), int, -1);
#endif /* ACE_WIN32 */
}
  
INLINE int 
ACE_OS::flock_rdlock (ACE_OS::flock_t *lock, short whence, off_t start, off_t len)
{
  ACE_TRACE ("ACE_OS::flock_rdlock");
#if defined (ACE_WIN32)
  lock->overlapped_.Offset = start;
  return ::LockFileEx (lock->handle_, 0, 0, len, 0, &lock->overlapped_) ? 0 : -1;
#else
  lock->lock_.l_whence = whence;
  lock->lock_.l_start = start;
  lock->lock_.l_len = len;
  lock->lock_.l_type = F_RDLCK;         // set read lock
  // block, if no access
  ACE_OSCALL_RETURN (::fcntl (lock->handle_, F_SETLKW, &lock->lock_), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::flock_trywrlock (ACE_OS::flock_t *lock, short whence, off_t start, off_t len)
{
  ACE_TRACE ("ACE_OS::flock_trywrlock");
#if defined (ACE_WIN32)
  lock->overlapped_.Offset = start;
  return ::LockFileEx (lock->handle_, LOCKFILE_FAIL_IMMEDIATELY | LOCKFILE_EXCLUSIVE_LOCK, 0, len, 0, &lock->overlapped_) ? 0 : -1;
#else
  lock->lock_.l_whence = whence;
  lock->lock_.l_start = start;
  lock->lock_.l_len = len;
  lock->lock_.l_type = F_WRLCK;         // set write lock

  // Does not block, if no access, returns -1.
  ACE_OSCALL_RETURN (::fcntl (lock->handle_, F_SETLK, &lock->lock_), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::flock_tryrdlock (ACE_OS::flock_t *lock, short whence, off_t start, off_t len)
{
  ACE_TRACE ("ACE_OS::flock_tryrdlock");
#if defined (ACE_WIN32)
  lock->overlapped_.Offset = start;
  return ::LockFileEx (lock->handle_, LOCKFILE_FAIL_IMMEDIATELY, 0, len, 0, &lock->overlapped_) ? 0 : -1;
#else
  lock->lock_.l_whence = whence;
  lock->lock_.l_start = start;
  lock->lock_.l_len = len;
  lock->lock_.l_type = F_RDLCK;         // set read lock

  // Does not block, if no access, returns -1.
  ACE_OSCALL_RETURN (::fcntl (lock->handle_, F_SETLK, &lock->lock_), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int
ACE_OS::flock_unlock (ACE_OS::flock_t *lock, short whence, off_t start, off_t len)
{
  ACE_TRACE ("ACE_OS::flock_unlock");
#if defined (ACE_WIN32)
  lock->overlapped_.Offset = start;
  return ::UnlockFileEx (lock->handle_, 0, len, 0, &lock->overlapped_) ? 0 : -1;
#else
  lock->lock_.l_whence = whence;
  lock->lock_.l_start = start;
  lock->lock_.l_len = len;
  lock->lock_.l_type = F_UNLCK;   // Unlock file.

  // release lock
  ACE_OSCALL_RETURN (::fcntl (lock->handle_, F_SETLK, &lock->lock_), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::flock_destroy (ACE_OS::flock_t *lock)
{
  ACE_TRACE ("ACE_OS::flock_destroy");
  if (lock->handle_ != ACE_INVALID_HANDLE)
    {
      ACE_OS::flock_unlock (lock);
      ACE_OS::close (lock->handle_);
      lock->handle_ = ACE_INVALID_HANDLE;
    }
  return 0;
}

INLINE int 
ACE_OS::execv (const char *path, char *const argv[])
{
  ACE_TRACE ("ACE_OS::execv");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_OSCALL_RETURN (::execv (path, argv), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::execve (const char *path, char *const argv[], char *const envp[])
{
  ACE_TRACE ("ACE_OS::execve");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_OSCALL_RETURN (::execve (path, argv, envp), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::execvp (const char *file, char *const argv[])
{
  ACE_TRACE ("ACE_OS::execvp");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_OSCALL_RETURN (::execvp (file, argv), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::execl (const char *path, const char *arg0, ...)
{
  ACE_TRACE ("ACE_OS::execl");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_NOTSUP_RETURN (-1);
// Need to write this code.
// ACE_OSCALL_RETURN (::execv (path, argv), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::execle (const char *path, const char *arg0, ...)
{
  ACE_TRACE ("ACE_OS::execle");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_NOTSUP_RETURN (-1);
// Need to write this code.
//  ACE_OSCALL_RETURN (::execve (path, argv, envp), int, -1);
#endif /* ACE_WIN32 */
}

INLINE int 
ACE_OS::execlp (const char *file, const char *arg0, ...)
{
  ACE_TRACE ("ACE_OS::execlp");
#if defined (ACE_WIN32)
  ACE_NOTSUP_RETURN (-1);
#else
  ACE_NOTSUP_RETURN (-1);
// Need to write this code.
//  ACE_OSCALL_RETURN (::execvp (file, argv), int, -1);
#endif /* ACE_WIN32 */
}

