#include "ace/Obstack.h"

ACE_ALLOC_HOOK_DEFINE(ACE_Obstack)

void
ACE_Obstack::dump (void) const
{
  ACE_TRACE ("ACE_Obstack::dump");
}

ACE_ALLOC_HOOK_DEFINE(ACE_Obchunk)

void
ACE_Obchunk::dump (void) const
{
  ACE_TRACE ("ACE_Obchunk::dump");
}

class ACE_Obchunk *
ACE_Obstack::new_chunk (void)
{
  ACE_TRACE ("ACE_Obstack::new_chunk");
  class ACE_Obchunk *temp = (class ACE_Obchunk *) 
    new char[sizeof (class ACE_Obchunk) + this->size_];

  temp->next_ = 0;
  temp->end_  = temp->contents_ + this->size_;
  temp->cur_  = temp->contents_;

  return temp;
}

ACE_Obstack::ACE_Obstack (int sz)
  : size_ (sz), 
    head_ (0)
{
  ACE_TRACE ("ACE_Obstack::ACE_Obstack");
  this->head_ = this->new_chunk ();
  this->curr_ = this->head_;
}

ACE_Obstack::~ACE_Obstack (void)
{
  ACE_TRACE ("ACE_Obstack::~ACE_Obstack");
  class ACE_Obchunk *temp = this->head_;

  while (temp != 0)
    {
      class ACE_Obchunk *next = temp->next_;
      temp->next_  = 0;
      delete [] temp;
      temp = next;
    }
}

char *
ACE_Obstack::copy (const char *s, 
		   size_t len)
{
  ACE_TRACE ("ACE_Obstack::copy");
  char *result;

  assert (this->size_ >= len + 1);

  // Check whether we need to grow our chunk...

  if (this->curr_->cur_ + len + 1 >= this->curr_->end_)
    {
      // Check whether we can just reuse previously allocated memory.

      if (this->curr_->next_ == 0)
	{
	  this->curr_->next_ = this->new_chunk ();
	  this->curr_ = this->curr_->next_;
	}
      else
	{
	  this->curr_ = this->curr_->next_;
	  this->curr_->cur_ = this->curr_->contents_;
	}
    }

  result = this->curr_->cur_;
  ACE_OS::memcpy (result, s, len);
  result[len] = '\0';
  this->curr_->cur_ += (len + 1);
  return result;
}

void 
ACE_Obstack::release (void)
{
  ACE_TRACE ("ACE_Obstack::release");
  this->curr_ = this->head_;
  this->curr_->cur_ = this->curr_->contents_;
}


