#include "ace/Log_Msg.h"
#include "ace/Remote_Name_Space.h"
#include "ace/Name_Options.h"

int
ACE_Remote_Name_Space::open (const char *servername, int port)
{
  ACE_INET_Addr servaddr;
  
  // Initialize Addr
  if (servaddr.set (port, servername) == -1)
    return -1;

  // Connect to Name Server process.
  if (this->ns_proxy_.open (servaddr) == -1)
    return -1;
      
  return 0;
}

ACE_Remote_Name_Space::ACE_Remote_Name_Space (void)
{
}

ACE_Remote_Name_Space::ACE_Remote_Name_Space (const char *hostname, int port)
{
  if (this->open (hostname, port) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Remote_Name_Space::ACE_Remote_Name_Space"));
}

int 
ACE_Remote_Name_Space::bind (const ACE_WString &name_, 
			     const ACE_WString &value_, 
			     const char *type_)
{
  ACE_Name_Request request (ACE_Name_Request::BIND, name_.rep (),
			    name_.length () * sizeof (wchar_t),
			    value_.rep (),
			    value_.length () * sizeof (wchar_t),
			    type_, strlen (type_));
  int result = this->ns_proxy_.request_reply (request);
  return result == ACE_Name_Reply::SUCCESS ? 0 : result;
}

int 
ACE_Remote_Name_Space::rebind (const ACE_WString &name_, 
			       const ACE_WString &value_, 
			       const char *type_)
{
  ACE_Name_Request request (ACE_Name_Request::REBIND, name_.rep (),
			    name_.length () * sizeof (wchar_t),
			    value_.rep (),
			    value_.length () * sizeof (wchar_t),
			    type_, strlen (type_));
  int result = this->ns_proxy_.request_reply (request);
  return result == ACE_Name_Reply::SUCCESS ? 0 : result;
}

int 
ACE_Remote_Name_Space::resolve (const ACE_WString &name_, 
				ACE_WString &value_, 
				char *&type_)
{
  ACE_Name_Request request (ACE_Name_Request::RESOLVE, name_.rep (),
			    name_.length () * sizeof (wchar_t),
			    NULL, 0, NULL, 0);

  if (this->ns_proxy_.send_request (request) == -1)
    return -1;

  ACE_Name_Request reply;

  if (this->ns_proxy_.recv_reply (reply) == -1)
    return -1;

  ACE_WString temp (reply.value (), reply.value_len () / sizeof (wchar_t));
  value_ = temp;
  type_ = new char[reply.type_len () + 1];
  ACE_OS::strcpy (type_, reply.type ());

  return 0;
}

int  
ACE_Remote_Name_Space::unbind (const ACE_WString &name_)
{
  ACE_Name_Request request (ACE_Name_Request::UNBIND, name_.rep (),
			    name_.length () * sizeof (wchar_t),
			    NULL, 0, NULL, 0);

  int result = this->ns_proxy_.request_reply (request);
  return result == ACE_Name_Reply::SUCCESS ? 0 : result;
}

int 
ACE_Remote_Name_Space::list_names (ACE_WSTRING_SET &set_, 
				   const ACE_WString &pattern_)
{
// Fix later on
#if 0
  ACE_Collectable_String *a_name;
  ACE_WString *aw_name;
  ACE_Name_Request reply;
  set_ = new ACE_WSTRING_SET;
  wchar_t *pat_ptr = NULL;
  int pat_len = 0;

  ACE_Name_Request request (ACE_Name_Request::LISTNAMES, NULL, 0,
			    NULL, 0, NULL, 0);
  if (pattern_ != NULL)
    {
      request.name (pattern_->rep ());
      request.name_len (pattern_->length ());
    }
  if (this->ns_proxy_.send_request (request) == -1)
    return -1;

  while (reply.msg_type () != ACE_Name_Request::LAST)
    {
      if (this->ns_proxy_.recv_reply (reply) == -1)
	{
	  ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Remote_Name_Space::list_names"));
	  return -1;
	}
      if (reply.msg_type () != ACE_Name_Request::LAST)
	{
	  a_name = new ACE_Collectable_String;
	  aw_name = new ACE_WString (reply.name (),
				     reply.name_len () / sizeof (wchar_t));
	  a_name->insert (0, aw_name->to_multi_byte ());
	  set_->insert (a_name); 
	  delete aw_name;
	}
    }
#endif 
  return 0;
}

ACE_Remote_Name_Space::~ACE_Remote_Name_Space (void)
{
}

void
ACE_Remote_Name_Space::dump (void) const
{
}
