#include "ace/Log_Msg.h"
#include "ace/Remote_Tokens.h"
#include "ace/Synch.h"
#include "ace/Synch_T.h"
#include "ace/SOCK_Connector.h"
#include "ace/Time_Value.h"
#include "ace/Singleton.h"

#if !defined (__INLINE__)
#include "ace/Remote_Tokens.i"
#endif /* __INLINE__ */

class ACE_TSS_Connection : public ACE_TSS<ACE_SOCK_Stream>
{
public:
  ACE_SOCK_Stream *get_connection (void);

  virtual ACE_SOCK_Stream *make_TSS_TYPE (void) const;

  operator ACE_SOCK_Stream *(void);

  static void set_server_address (const ACE_INET_Addr &server_address);
  // set the server address

protected:
  static ACE_SOCK_Connector connector_;
  // ACE_Connector factory used to establish connections actively.

  static ACE_INET_Addr server_address_;
  // The address of the Token Server used by all instances of Token_Proxy.
};

static ACE_Singleton <ACE_TSS_Connection, ACE_Mutex> ACE_T_C;
typedef ACE_Singleton<ACE_TSS_Connection,ACE_Mutex> ACE_Token_Connections;

ACE_TSS_Connection *ACE_Singleton<ACE_TSS_Connection,ACE_Mutex>::instance_;
ACE_Mutex ACE_Singleton<ACE_TSS_Connection,ACE_Mutex>::lock_;
ACE_SOCK_Connector ACE_TSS_Connection::connector_;
ACE_INET_Addr ACE_TSS_Connection::server_address_;

// ************************************************************

void 
ACE_TSS_Connection::set_server_address (const ACE_INET_Addr &server_address)
{
  ACE_TRACE ("ACE_TSS_Connection::set_server_address");
  server_address_ = server_address;
}

ACE_SOCK_Stream *
ACE_TSS_Connection::get_connection (void)
{
  return ACE_TSS<ACE_SOCK_Stream>::operator-> ();
}

ACE_SOCK_Stream *
ACE_TSS_Connection::make_TSS_TYPE (void) const
{
  ACE_TRACE ("ACE_TSS_Connection::make_TSS_TYPE");
  ACE_SOCK_Stream *stream = 0;
  stream = new ACE_SOCK_Stream;
  // This compiler sucks.
  if (((ACE_SOCK_Connector) connector_).connect (*stream, server_address_) == -1)
    {
      delete stream;
      ACE_ERROR_RETURN ((LM_ERROR, "%p\n",
			 "ACE_TSS_Connection"), 0);
    }
  ACE_DEBUG ((LM_DEBUG, "ACE_TSS_Connection new connection\n"));

  return stream;
}

ACE_TSS_Connection::operator ACE_SOCK_Stream *(void)
{
  return this->get_connection ();
}

// ************************************************************
// ************************************************************
// ************************************************************

ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy (ACE_Token_Proxy::PROXY_TYPE proxy_type,
						ACE_Tokens::TOKEN_TYPE token_type)
: ACE_Token_Proxy (proxy_type,
		   token_type)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy");
}

ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy (const ACE_Remote_Token_Proxy &proxy)
: ACE_Token_Proxy (proxy)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy");
}

ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy (const char *token_name,
						int ignore_deadlock,
						int debug,
						ACE_Token_Proxy::PROXY_TYPE proxy_type,
						ACE_Tokens::TOKEN_TYPE token_type)
: ACE_Token_Proxy (proxy_type, token_type),
  ignore_shadow_deadlock_ (ignore_deadlock)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::ACE_Remote_Token_Proxy");
  ACE_Token_Proxy::open (token_name, 0, debug);
}

ACE_Remote_Token_Proxy::~ACE_Remote_Token_Proxy (void)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::~ACE_Remote_Token_Proxy");
}

void 
ACE_Remote_Token_Proxy::set_server_address (const ACE_INET_Addr &server_address)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::set_server_address");
  ACE_Token_Connections::instance ()->set_server_address (server_address);
}

int 
ACE_Remote_Token_Proxy::initiate_connection (void)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::initiate_connection");
  if (mutex_ == 0)
    {
      errno = ENOENT;
      ACE_ERROR_RETURN ((LM_ERROR, "ACE_Remote_Token_Proxy not open.\n"), -1);
    }

  ACE_SOCK_Stream *peer = ACE_Token_Connections::instance ()->get_connection ();
  return peer == 0 ? 0 : 1;
}

// Do the work of sending a request and getting a reply.

int
ACE_Remote_Token_Proxy::request_reply (ACE_Token_Request &request,
				 ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::request_reply");
  void    *buffer;
  ssize_t length;

//  request.dump ();

  if ((length = request.encode (buffer)) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "encode failed"), -1);    

  ACE_SOCK_Stream *peer = ACE_Token_Connections::instance ()->get_connection ();

  if (peer == 0)
    return -1;

  // Transmit request via a blocking send.

  if (peer->send_n (buffer, length) != length)
    {
      ACE_ERROR ((LM_ERROR, "%p\n", "send_n failed"));
      return -1;
    }
  else
    {
      ACE_Token_Reply reply;
      
      // Receive reply via blocking read.

      if (peer->recv (&reply, sizeof reply) == -1)
	{
	  ACE_ERROR ((LM_ERROR, "%p\n", "recv failed"));
	  return -1;
	}

      if (reply.decode () == -1)
	{
	  ACE_ERROR ((LM_ERROR, "%p\n", "decode failed"));
	  return -1;
	}

      errno = int (reply.errnum ());
      if (errno != 0)
	{
	  ACE_ERROR ((LM_ERROR, ""));
	  return -1;
	}
      else
	return 0;
    }
}

int 
ACE_Remote_Token_Proxy::acquire (int notify,
			   void (*sleep_hook)(void *),
			   ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::acquire");

  // first grab the local shadow mutex.
  
  if (ACE_Token_Proxy::acquire (notify, sleep_hook, ACE_Synch_Options::asynch) == -1)
    // acquire failed
    {
      switch (ACE_LOG_MSG->errnum ())
	{
	case EWOULDBLOCK :
	  // whoah, we detected wouldblock via the shadow mutex!
	  if (debug_)
	    ACE_DEBUG ((LM_DEBUG, "(%t) shadow: acquire will block, owner is %s\n",
			this->mutex_->owner_id ()));
	  // no error, but would block,
	  break;

	case EDEADLK :
	  if (debug_)
	    ACE_DEBUG ((LM_DEBUG, "(%t) shadow: deadlock detected\n"));

	  if (ignore_shadow_deadlock_)
	    break;
	  else
	    return -1;

	  default :
	  ACE_ERROR_RETURN ((LM_ERROR, 
			     "%p\n","(%t) shadow: ACE_Remote_Token_Proxy acquire."), -1);
	  break;
	}
    }

  ACE_Token_Request request (token_name_.type (),
			     this->type (),
			     ACE_Token_Request::ACQUIRE,
			     this->name (),
			     this->client_id (), 
			     options);

  request.notify (notify);

  int result = this->request_reply (request, options);

  if (result == -1)
    // update the local shadow copy
    {
      ACE_DEBUG ((LM_DEBUG, "error on remote acquire, releasing shadow mutex.\n"));
      ACE_Token_Proxy::release ();
    }
  else
    ACE_DEBUG ((LM_DEBUG, "(%t) acquired %s remotely.\n", this->name ()));

  return result;
}

int 
ACE_Remote_Token_Proxy::tryacquire (void (*sleep_hook)(void *))
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::tryacquire");

  // if we can detect locally that the tryacquire will fail, there is
  // no need to go remote.
  if (ACE_Token_Proxy::tryacquire (sleep_hook) == -1)
    {
      ACE_DEBUG ((LM_DEBUG, "shadow try acquire failed\n"));
      return -1;
    }

  ACE_Token_Request request (token_name_.type (),
			     this->type (),
			     ACE_Token_Request::RELEASE, 
			     this->name (),
			     this->client_id (),
			     ACE_Synch_Options::synch);

  return this->request_reply (request, ACE_Synch_Options::synch);
}


int
ACE_Remote_Token_Proxy::renew (int requeue_position,
			 ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::renew");

  if (ACE_Token_Proxy::renew (requeue_position, ACE_Synch_Options::asynch) == -1)
    {
      // check for error
      if (ACE_LOG_MSG->errnum () != EWOULDBLOCK)
	ACE_ERROR_RETURN ((LM_ERROR, 
			   "(%t) shadow: renew failed."), -1);

      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "(%t) shadow: renew would block. owner %s.\n",
		    this->owner_id ()));
    }

  ACE_Token_Request request (token_name_.type (),
			     this->type (),
			     ACE_Token_Request::RENEW, 
			     this->name (),
			     this->client_id (), 
			     options);

  request.requeue_position (requeue_position);

  int result = this->request_reply (request, options);
  if (result == -1)
    {
      ACE_DEBUG ((LM_DEBUG, "error on remote renew, releasing shadow mutex.\n"));
      ACE_Token_Proxy::release ();
    }
  else
    ACE_DEBUG ((LM_DEBUG, "(%t) renewed %s remotely.\n", this->name ()));

  return result;
}

int
ACE_Remote_Token_Proxy::release (ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::release");

  if (ACE_Token_Proxy::release () == -1)
    ACE_ERROR ((LM_ERROR, "(%t) shadow: release failed\n"));

  ACE_Token_Request request (token_name_.type (),
			     this->type (),
			     ACE_Token_Request::RELEASE, 
			     this->name (),
			     this->client_id (), 
			     options);

  int result = this->request_reply (request, options);
  if (result == 0)
    ACE_DEBUG ((LM_DEBUG, "(%t) released %s remotely.\n", this->name ()));

  return result;
}

int 
ACE_Remote_Token_Proxy::remove (ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::remove");
  return 0;
}

ACE_Token_Proxy* 
ACE_Remote_Token_Proxy::clone (void)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::clone");
  return new ACE_Remote_Token_Proxy (*this);
}

void
ACE_Remote_Token_Proxy::token_acquired (void* vp)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::token_acquired");
  ACE_DEBUG ((LM_DEBUG, "(%s) token_acquired %s\n", 
	      this->client_id (),
	      this->name ()));
//  ACE_Token_Proxy::token_acquired (vp);
}

const char*
ACE_Remote_Token_Proxy::owner_id (void)
{
  ACE_TRACE ("ACE_Remote_Token_Proxy::owner_id");
  ACE_DEBUG ((LM_DEBUG, "owner_id called\n"));
  // @@ special operation
  return 0;
}

#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
 template class ACE_TSS <ACE_SOCK_Stream>;
 template class ACE_Singleton <ACE_TSS_Connection>;
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */

