/* Defines the member functions for the base class of the ACE_SOCK
   abstraction. */

#include "ace/SOCK.h"
#include "ace/Log_Msg.h"

ACE_ALLOC_HOOK_DEFINE(ACE_SOCK)

void
ACE_SOCK::dump (void) const
{
  ACE_TRACE ("ACE_SOCK::dump");
}

ACE_SOCK::ACE_SOCK (void)
{
  ACE_TRACE ("ACE_SOCK::ACE_SOCK");
}

/* Returns information about the remote peer endpoint (if there is
   one). */

int
ACE_SOCK::get_remote_addr (ACE_Addr &sa) const
{
  ACE_TRACE ("ACE_SOCK::get_remote_addr");
  int len = sa.get_size ();
  sockaddr *addr = (sockaddr *) sa.get_addr ();

  if (ACE_OS::getpeername (this->get_handle (), addr, &len) == -1)
    return -1;
  
  sa.set_size (len);
  return 0;
}

int
ACE_SOCK::get_local_addr (ACE_Addr &sa) const
{
  ACE_TRACE ("ACE_SOCK::get_local_addr");
  int len = sa.get_size ();
  sockaddr *addr = (sockaddr *) sa.get_addr ();

  if (ACE_OS::getsockname (this->get_handle (), addr, &len) == -1)
    return -1;

  sa.set_size (len);
  return 0;
}

int
ACE_SOCK::open (int type, 
		int protocol_family, 
		int protocol)
{
  ACE_TRACE ("ACE_SOCK::open");
  this->set_handle (ACE_OS::socket (protocol_family, type, protocol));
  return this->get_handle () == ACE_INVALID_HANDLE ? -1 : 0;
}

/* Close down a ACE_SOCK. */

int
ACE_SOCK::close (void)
{
  ACE_TRACE ("ACE_SOCK::close");
  int result = 0;

  if (this->get_handle () != ACE_INVALID_HANDLE)
    {
      result = ACE_OS::close (this->get_handle ());
      this->set_handle (ACE_INVALID_HANDLE);
    }
  return result;
}

/* General purpose constructor for performing server ACE_SOCK
   creation. */

ACE_SOCK::ACE_SOCK (int type, 
		    int protocol_family, 
		    int protocol)
{
  ACE_TRACE ("ACE_SOCK::ACE_SOCK");
  if (this->open (type, protocol_family, protocol) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_SOCK::ACE_SOCK"));
}
