#include "ace/Parse_Node.h"
#include "ace/Service_Object.h"
#include "ace/Svc_Conf.h"
#include "ace/sc_tokens.h"
#include "ace/Get_Opt.h"
#include "ace/ARGV.h"
#include "ace/Malloc.h"
#include "ace/Service_Manager.h"
#include "ace/Service_Repository.h"
#include "ace/Service_Config.h"

#if !defined (__INLINE__)
#include "ace/Service_Config.i"
#endif /* __INLINE__ */

ACE_ALLOC_HOOK_DEFINE(ACE_Service_Config)

void
ACE_Service_Config::dump (void) const
{
  ACE_TRACE ("ACE_Service_Config::dump");
}

// All the factory functions that allocate default statically linked
// services should be placed below.

// Allocate a Service Manager.

static ACE_Service_Object *
alloc_service_manager (void)
{
  ACE_TRACE ("alloc_service_manager");
  return new ACE_Service_Manager;
}

// ----------------------------------------
// Process-wide ACE_Reactor.
/* static */
ACE_Reactor *ACE_Service_Config::reactor_ = 0;

// Process-wide Service Repository.
/* static */
ACE_Service_Repository *ACE_Service_Config::svc_rep_ = 0;

// Process-wide Thread Manager.
/* static */
ACE_Thread_Manager *ACE_Service_Config::thr_mgr_ = 0;

// Process-wide ACE_Allocator.
/* static */
ACE_Allocator *ACE_Service_Config::allocator_ = 0;

// Make this the default.
typedef ACE_Malloc <ACE_Local_Memory_Pool, ACE_Null_Mutex> 
	ACE_DEFAULT_MALLOC;

// Terminate the eventloop.
/* static */
sig_atomic_t ACE_Service_Config::end_event_loop_ = 0;

// Trigger a reconfiguration.
/* static */
sig_atomic_t ACE_Service_Config::reconfig_occurred_ = 0;

  // = Set by command-line options. 
/* static */
char ACE_Service_Config::debug_ = 0;
char ACE_Service_Config::be_a_daemon_ = 0;
char ACE_Service_Config::no_defaults_ = 0;

// Number of the signal used to trigger reconfiguration.
/* static */
int ACE_Service_Config::signum_ = SIGHUP;

// Name of the service configuration file. 
const char *ACE_Service_Config::service_config_file_ = "./svc.conf";

// Name of file used to store messages.
const char *ACE_Service_Config::logger_key_ = "/tmp/server_daemon";

// List of statically configured services.
ACE_Static_Svc_Descriptor ACE_Service_Config::service_list_[] =
{
  {"ACE_Svc_Manager", SVC_OBJ_T, &alloc_service_manager, 
   ACE_Service_Type::DELETE_THIS | ACE_Service_Type::DELETE_OBJ, 0},
  {(char *) 0, 0, (ACE_Service_Object *(*)(void)) 0, 0, 0}
};

ACE_Allocator *
ACE_Service_Config::allocator (void)
{
  ACE_TRACE ("ACE_Service_Config::allocator");
  if (ACE_Service_Config::allocator_ == 0)
    ACE_Service_Config::allocator_ 
      = new ACE_Allocator_Adapter <ACE_DEFAULT_MALLOC>;

  return ACE_Service_Config::allocator_;
}

ACE_Allocator *
ACE_Service_Config::allocator (ACE_Allocator *r)
{
  ACE_TRACE ("ACE_Service_Config::allocator");

  ACE_Allocator *t = ACE_Service_Config::allocator_;
  ACE_Service_Config::allocator_ = r;
  return t;
}

ACE_Reactor *
ACE_Service_Config::reactor (void)
{
  ACE_TRACE ("ACE_Service_Config::reactor");
  if (ACE_Service_Config::reactor_ == 0)
    ACE_Service_Config::reactor_ = new ACE_Reactor;

  return ACE_Service_Config::reactor_;
}

ACE_Reactor *
ACE_Service_Config::reactor (ACE_Reactor *r)
{
  ACE_TRACE ("ACE_Service_Config::reactor");

  ACE_Reactor *t = ACE_Service_Config::reactor_;
  ACE_Service_Config::reactor_ = r;
  return t;
}

ACE_Service_Repository *
ACE_Service_Config::svc_rep (void)
{
  ACE_TRACE ("ACE_Service_Config::svc_rep");

  if (ACE_Service_Config::svc_rep_ == 0)
    ACE_Service_Config::svc_rep_ = new ACE_Service_Repository;

  return ACE_Service_Config::svc_rep_;
}

ACE_Service_Repository *
ACE_Service_Config::svc_rep (ACE_Service_Repository *s)
{
  ACE_TRACE ("ACE_Service_Config::svc_rep");

  ACE_Service_Repository *t = ACE_Service_Config::svc_rep_;
  ACE_Service_Config::svc_rep_ = s;
  return t;
}

ACE_Thread_Manager *
ACE_Service_Config::thr_mgr (void)
{
  ACE_TRACE ("ACE_Service_Config::thr_mgr");

  if (ACE_Service_Config::thr_mgr_ == 0)
    ACE_Service_Config::thr_mgr_ = new ACE_Thread_Manager;

  return ACE_Service_Config::thr_mgr_;
}

ACE_Thread_Manager *
ACE_Service_Config::thr_mgr (ACE_Thread_Manager *tm)
{
  ACE_TRACE ("ACE_Service_Config::thr_mgr");

  ACE_Thread_Manager *t = ACE_Service_Config::thr_mgr_;
  ACE_Service_Config::thr_mgr_ = tm;
  return t;
}

// Totally remove <svc_name> from the daemon by removing it 
// from the ACE_Reactor, and unlinking it if necessary.

int
ACE_Service_Config::remove (const char svc_name[])
{
  ACE_TRACE ("ACE_Service_Config::remove");
  return ACE_Service_Config::svc_rep ()->remove (svc_name);
}

// Suspend SVC_NAME.  Note that this will not unlink the service 
// from the daemon if it was dynamically linked, it will mark it 
// as being suspended in the Service Repository and call the
// suspend() member function on the appropriate ACE_Service_Object.  
// A service can be resumed later on by calling the RESUME() 
// member function... 

int
ACE_Service_Config::suspend (const char svc_name[])
{
  ACE_TRACE ("ACE_Service_Config::suspend");
  return ACE_Service_Config::svc_rep ()->suspend (svc_name);
}

// Resume a SVC_NAME that was previously suspended or has not yet
// been resumed (e.g., a static service). 

int
ACE_Service_Config::resume (const char svc_name[])
{
  ACE_TRACE ("ACE_Service_Config::resume");
  return ACE_Service_Config::svc_rep ()->resume (svc_name);
}

// Initialize the Service Repository.  Note that this *must*
// be performed in the constructor (rather than open()) since
// otherwise the repository will not be properly initialized
// to allow static configuration of services... 

ACE_Service_Config::ACE_Service_Config (int ignore_defaults, 
					size_t size, 
					int signum)
{
  ACE_TRACE ("ACE_Service_Config::ACE_Service_Config");
  ACE_Service_Config::no_defaults_ = ignore_defaults;
  ACE_Service_Config::signum_      = signum;

  /* Initialize the Service Repository. */

  if (ACE_Service_Config::svc_rep_ == 0 &&
      (ACE_Service_Config::svc_rep_ = new ACE_Service_Repository (size)) == 0)
    ACE_ERROR ((LM_ERROR, "%p\n", "can't allocate a ACE_Service_Repository\n"));

  /* Initialize the ACE_Reactor (the ACE_Reactor should be the same
     size as the ACE_Service_Repository). */

  if (ACE_Service_Config::reactor_ == 0 && 
      (ACE_Service_Config::reactor_ = new ACE_Reactor (size)) == 0)
    ACE_ERROR ((LM_ERROR, "%p\n", "cannot allocate a ACE_Reactor\n"));

  // This really ought to be a singleton I suspect...
  if (ACE_Service_Config::reactor_->register_handler (ACE_Service_Config::signum_, this) == -1)
    ACE_ERROR ((LM_ERROR, "can't register signal handler\n"));
}

// Handle the command-line options intended for the ACE_Service_Config.

void
ACE_Service_Config::parse_args (int argc, char *argv[])
{
  ACE_TRACE ("ACE_Service_Config::parse_args");
  ACE_Get_Opt getopt (argc, argv, "bdf:ns:", 1); // Start at argv[1]

  for (int c; (c = getopt ()) != -1; )
    switch (c)
      {
      case 'b':
	ACE_Service_Config::be_a_daemon_ = 1;
	break;
      case 'd': 
	ACE_Service_Config::debug_ = 1;
	break;
      case 'f':
	ACE_Service_Config::service_config_file_ = getopt.optarg;
	break;
      case 'n':
	ACE_Service_Config::no_defaults_ = 1;
	break;
      case 's':
	{
	  ACE_Event_Handler *eh = 0;

	  if (ACE_Service_Config::reactor ()->handler (ACE_Service_Config::signum_, &eh) == -1)
	    ACE_ERROR ((LM_ERROR, "cannot obtain signal handler\n"));

	  ACE_Service_Config::signum_ = ACE_OS::atoi (getopt.optarg);

	  if (ACE_Service_Config::reactor ()->register_handler (ACE_Service_Config::signum_, 
								eh) == -1)
	    ACE_ERROR ((LM_ERROR, "cannot obtain signal handler\n"));
	  break;
	}
      default:
	ACE_ERROR ((LM_ERROR, "%c is not a ACE_Service_Config option\n", c));
	break;
      }
}

// Initialize and activate a statically linked service. 

 int
ACE_Service_Config::initialize (const char svc_name[], char *parameters)
{
  ACE_TRACE ("ACE_Service_Config::initialize");
  ACE_ARGV argv (parameters);
  ACE_Service_Record  *srp = 0;

  ACE_DEBUG ((LM_DEBUG, "opening static service %s\n", svc_name));

  if (ACE_Service_Config::svc_rep ()->find (svc_name, (const ACE_Service_Record **) &srp) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%s not found\n", svc_name), -1);    
  else if (srp->type ()->init (argv.count (), &argv) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "static initialization failed, %p\n", svc_name), -1);
  else
    {
      srp->active (1);
      return 0;
    }
}

// Dynamically link the shared object file and retrieve
// a pointer to the designated shared object in this file. 

int
ACE_Service_Config::initialize (const ACE_Service_Record *sr,
				char  parameters[])
{
  ACE_TRACE ("ACE_Service_Config::initialize");
  ACE_ARGV argv (parameters);

  ACE_DEBUG ((LM_DEBUG, "opening dynamic service %s\n", sr->name ()));
  if (ACE_Service_Config::svc_rep ()->insert (sr) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "insertion failed, %p\n", sr->name ()), -1);
  else if (sr->type ()->init (argv.count (), &argv) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "dynamic initialization failed for %s\n", 
		      sr->name ()), -1);
  else
    return 0;
}

// Process service configuration requests as indicated 
// in the <service_config_file>.

int 
ACE_Service_Config::process_directives (void)
{
  ACE_TRACE ("ACE_Service_Config::process_directives");
  FILE *fp;

  if ((fp = ACE_OS::fopen (ACE_Service_Config::service_config_file_, "r")) == 0)
    {
      errno = ENOENT;
      return -1; // No service configuration file
    }
  else
    {
      ace_yyrestart (fp);

      ace_yyerrno = 0;
      ace_yylineno = 1;
  
      ace_obstack = new ACE_Obstack;

      ace_yyparse ();

      delete ace_obstack;
      if (ace_yyerrno > 0)
	errno = EINVAL;		/* This is a hack, better errors should be provided... */

      return ace_yyerrno;
    }
}

/* Add the default statically-linked services to the Service Repository. */

int
ACE_Service_Config::load_defaults (void)
{
  ACE_TRACE ("ACE_Service_Config::load_defaults");
  for (ACE_Static_Svc_Descriptor *sl = ACE_Service_Config::service_list_; 
       sl->name_ != 0; 
       sl++)
    {
      ACE_Service_Type *stp = ace_create_service_type (sl->name_, sl->type_, 
						   (const void *) (*sl->alloc_)(),
						   sl->flags_);
      if (stp == 0)
	continue;

      const ACE_Service_Record *sr = new ACE_Service_Record (sl->name_, stp, 0, sl->active_);

      if (ACE_Service_Config::svc_rep ()->insert (sr) == -1)
	return -1;
    }
  return 0;
}

// Performs an open without parsing command-line arguments.

int
ACE_Service_Config::open (const char program_name[])
{   
  ACE_TRACE ("ACE_Service_Config::open");
  if (ACE_LOG_MSG->open (program_name, ACE_Log_Msg::STDERR, 
			 ACE_Service_Config::logger_key_) == -1)
    return -1;
  ACE_DEBUG ((LM_STARTUP, "starting up daemon %n\n"));

  /* Initialize the Service Repository (this will still work if user forgets
     to define an object of type ACE_Service_Config). */

  if (ACE_Service_Config::svc_rep_ == 0 &&
      (ACE_Service_Config::svc_rep_ = new ACE_Service_Repository) == 0)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "cannot allocate a ACE_Service_Repository\n"), -1);

  /* Initialize the ACE_Reactor (the ACE_Reactor should be the same 
     size as the ACE_Service_Repository). */

  if (ACE_Service_Config::reactor_ == 0 && 
      (ACE_Service_Config::reactor_ = new ACE_Reactor) == 0)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "cannot allocate a ACE_Reactor\n"), -1);

  if (ACE_Service_Config::be_a_daemon_)
    ACE_Service_Config::start_daemon ();	 

  /* Register ourselves to receive reconfiguration requests via signals! */

  if (ACE_Service_Config::no_defaults_ == 0 && ACE_Service_Config::load_defaults () == -1)
    return -1;
  return ACE_Service_Config::process_directives ();
}

ACE_Service_Config::ACE_Service_Config (const char program_name[])
{
  ACE_TRACE ("ACE_Service_Config::ACE_Service_Config");
  if (this->open (program_name) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", program_name));
}

// Signal handling API to trigger dynamic reconfiguration.

int
ACE_Service_Config::handle_signal (int sig
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
)
{
  ACE_TRACE ("ACE_Service_Config::handle_signal");
  if (ACE_Service_Config::signum_ != sig)
    ACE_ERROR ((LM_ERROR, "error, signal %S does match %S\n", sig, ACE_Service_Config::signum_));
  if (ACE_Service_Config::debug_)
    ACE_DEBUG ((LM_DEBUG, "signal %S occurred\n", sig));
  ACE_Service_Config::reconfig_occurred_ = 1;
  return 0;
}

// Trigger the reconfiguration process.

void
ACE_Service_Config::reconfigure (void)
{
  ACE_TRACE ("ACE_Service_Config::reconfigure");
  ACE_Service_Config::reconfig_occurred_ = 0;
  if (ACE_Service_Config::debug_)
    {
      time_t t = ACE_OS::time (0);
      ACE_DEBUG ((LM_DEBUG, "beginning reconfiguration at %s", ACE_OS::ctime (&t)));
    }
  if (ACE_Service_Config::process_directives () == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "process_directives"));
}

// Run the event loop until the <ACE_Reactor::handle_events>
// method returns -1 or the <end_event_loop> method
// is invoked.

int
ACE_Service_Config::run_event_loop (void)
{
  ACE_TRACE ("ACE_Service_Config::run_event_loop");
  while (ACE_Service_Config::end_event_loop_ == 0)
    {
      int result = ACE_Service_Config::reactor ()->handle_events ();
      if (ACE_Service_Config::reconfig_occurred_)
	ACE_Service_Config::reconfigure ();
      else if (result == -1)
	return -1;
    }
  /* NOTREACHED */
  return 0;
}

// Run the event loop until the <ACE_Reactor::handle_events>
// method returns -1, the <end_event_loop> method
// is invoked, or the <ACE_Time_Value> expires.

int
ACE_Service_Config::run_event_loop (ACE_Time_Value &tv)
{
  ACE_TRACE ("ACE_Service_Config::run_event_loop");
  while (ACE_Service_Config::end_event_loop_ == 0)
    {
      int result = ACE_Service_Config::reactor ()->handle_events (tv);
      if (ACE_Service_Config::reconfig_occurred_)
	ACE_Service_Config::reconfigure ();
      else if (result == -1)
	return -1;
    }
  /* NOTREACHED */
  return 0;
}

// Tidy up and perform last rites on a terminating ACE_Service_Config.

int
ACE_Service_Config::close (void)
{
  ACE_TRACE ("ACE_Service_Config::close");
  if (ACE_Service_Config::svc_rep_ != 0)
    {
      ACE_DEBUG ((LM_SHUTDOWN, "shutting down daemon %n\n"));

      // ACE_Service_Config must be deleted first so that an object's
      // fini() method may reference a valid ACE_Reactor.
      delete ACE_Service_Config::svc_rep_;
      ACE_Service_Config::svc_rep_ = 0;
      delete ACE_Service_Config::reactor_;
      ACE_Service_Config::reactor_ = 0;
      delete ACE_Service_Config::thr_mgr_;
      ACE_Service_Config::thr_mgr_ = 0;
      delete ACE_Service_Config::allocator_;
      ACE_Service_Config::allocator_ = 0;
    }
  return 0;
}

// Perform user-specified close activities and remove dynamic memory. 

 
ACE_Service_Config::~ACE_Service_Config (void)
{
  ACE_TRACE ("ACE_Service_Config::~ACE_Service_Config");
  ACE_Service_Config::close ();
}

/* static */
void
ACE_Service_Config::end_event_loop (sig_atomic_t value)
{
  ACE_TRACE ("ACE_Service_Config::end_event_loop");
  ACE_Service_Config::end_event_loop_ = value;
  ACE_Service_Config::reactor ()->notify ();
}

#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
template class ACE_Malloc<ACE_Local_Memory_Pool, ACE_Null_Mutex>;
template class ACE_Allocator_Adapter<ACE_Malloc<ACE_Local_Memory_Pool, ACE_Null_Mutex> >;
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */
