#include "ace/Service_Record.h"

#if !defined (__INLINE__)
#include "ace/Service_Record.i"
#endif /* __INLINE__ */

ACE_ALLOC_HOOK_DEFINE(ACE_Service_Type)

void
ACE_Service_Type::dump (void) const
{
  ACE_TRACE ("ACE_Service_Type::dump");
}

ACE_Service_Type::ACE_Service_Type (const void *so, 
				    const char *s_name, 
				    unsigned int f)
  : obj_ (so), 
    flags_ (f)
{
  ACE_TRACE ("ACE_Service_Type::ACE_Service_Type");
  this->name (ACE_OS::strcpy (new char[::strlen (s_name) + 1], s_name));
}

int
ACE_Service_Type::fini (void) const
{
  ACE_TRACE ("ACE_Service_Type::fini");
  ACE_DEBUG ((LM_DEBUG, "destroying %s, flags = %d\n", 
	     this->name_, this->flags_));

  delete [] (char *) this->name_;
  if (ACE_BIT_ENABLED (this->flags_, ACE_Service_Type::DELETE_OBJ))
    delete (void *) this->object ();
  if (ACE_BIT_ENABLED (this->flags_, ACE_Service_Type::DELETE_THIS))
    delete (void *) this; // Prevent object's destructor from being called...
  return 0;
}

ACE_Service_Object_Type::ACE_Service_Object_Type (ACE_Service_Object *so, 
						  const char *s_name, 
						  unsigned int f)
  : ACE_Service_Type ((const void *) so, s_name, f)
{
  ACE_TRACE ("ACE_Service_Object_Type::ACE_Service_Object_Type");
}

int
ACE_Service_Object_Type::init (int argc, char *argv[]) const
{
  ACE_TRACE ("ACE_Service_Object_Type::init");
  ACE_Service_Object *so = (ACE_Service_Object *) this->object ();

  if (so == 0)
    return -1;
  else
    return so->init (argc, argv);
}

#if defined (ACE_HAS_THREADS)

#if 0 // defined (__GNUG__)
// XXXX hack by Todd to bring in ACE_Stream...
template class ACE_Stream<ACE_MT_SYNCH>;
template class ACE_Task<ACE_MT_SYNCH>;
template class ACE_Stream_Head<ACE_MT_SYNCH>;
template class ACE_Stream_Tail<ACE_MT_SYNCH>;
template class ACE_Thru_Task<ACE_MT_SYNCH>;
#endif /* __GNUG__ */

ACE_ALLOC_HOOK_DEFINE(ACE_Module_Type)

void
ACE_Module_Type::dump (void) const
{
  ACE_TRACE ("ACE_Module_Type::dump");
}

ACE_Module_Type::ACE_Module_Type (MT_Module *m, 
				  const char *m_name, 
				  u_int f)
  : ACE_Service_Type ((const void *) m, m_name, f)
{
  ACE_TRACE ("ACE_Module_Type::ACE_Module_Type");
}

int
ACE_Module_Type::init (int argc, char *argv[]) const
{
  ACE_TRACE ("ACE_Module_Type::init");
  if (((MT_Module *) this->object ())->reader ()->init (argc, argv) == -1
      || ((MT_Module *) this->object ())->writer ()->init (argc, argv) == -1)
    return -1;
  else
    return 0;
}

int
ACE_Module_Type::suspend (void) const
{
  ACE_TRACE ("ACE_Module_Type::suspend");
  if (((MT_Module *) this->object ())->reader ()->suspend () == -1
      || ((MT_Module *) this->object ())->writer ()->suspend () == -1)
    return -1;
  else
    return 0;
}

int
ACE_Module_Type::resume (void) const
{
  ACE_TRACE ("ACE_Module_Type::resume");
  if (((MT_Module *) this->object ())->reader ()->resume () == -1
      || ((MT_Module *) this->object ())->writer ()->resume () == -1)
    return -1;
  else
    return 0;
}

/* Note, these operations are somewhat too familiar with the
   implementation of ACE_Module and ACE_Module::close... */

int
ACE_Module_Type::fini (void) const
{
  ACE_TRACE ("ACE_Module_Type::fini");
  MT_Task *reader = ((MT_Module *) this->object ())->reader ();
  MT_Task *writer = ((MT_Module *) this->object ())->writer ();

  reader->fini ();
  writer->fini ();
  delete reader;
  delete writer;
  return ACE_Service_Type::fini ();     
}

int
ACE_Module_Type::info (char **str, size_t len) const
{
  ACE_TRACE ("ACE_Module_Type::info");
  char buf[BUFSIZ];

  ACE_OS::sprintf (buf, "%s\t %s", this->name (), "# ACE_Module\n");

  if (*str == 0 && (*str = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*str, buf, len);
  return ACE_OS::strlen (buf);
}

void
ACE_Module_Type::link (ACE_Module_Type *n)
{
  ACE_TRACE ("ACE_Module_Type::link");
  this->next_ = n;
}

ACE_Module_Type *
ACE_Module_Type::link (void) const
{
  ACE_TRACE ("ACE_Module_Type::link");
  return this->next_;
}

ACE_ALLOC_HOOK_DEFINE(ACE_STREAM_Type)

void
ACE_STREAM_Type::dump (void) const
{
  ACE_TRACE ("ACE_STREAM_Type::dump");
}

int
ACE_STREAM_Type::init (int, char *[]) const
{
  ACE_TRACE ("ACE_STREAM_Type::init");
  return 0;
}

int
ACE_STREAM_Type::suspend (void) const
{
  ACE_TRACE ("ACE_STREAM_Type::suspend");
  for (ACE_Module_Type *m = this->head_; m != 0; m = m->link ())
    m->suspend ();

  return 0;
}

int
ACE_STREAM_Type::resume (void) const
{
  ACE_TRACE ("ACE_STREAM_Type::resume");
  for (ACE_Module_Type *m = this->head_; m != 0; m = m->link ())
    m->resume ();

  return 0;
}

ACE_STREAM_Type::ACE_STREAM_Type (MT_Stream *s, 
				  const char *s_name, 
				  unsigned int f)
  : ACE_Service_Type ((const void *) s, s_name, f), 
    head_ (0)
{
  ACE_TRACE ("ACE_STREAM_Type::ACE_STREAM_Type");
}

int
ACE_STREAM_Type::info (char **str, size_t len) const
{
  ACE_TRACE ("ACE_STREAM_Type::info");
  char buf[BUFSIZ];

  ACE_OS::sprintf (buf, "%s\t %s", this->name (), "# STREAM\n");

  if (*str == 0 && (*str = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*str, buf, len);
  return ACE_OS::strlen (buf);
}

int
ACE_STREAM_Type::fini (void) const
{
  ACE_TRACE ("ACE_STREAM_Type::fini");
  MT_Stream *str = (MT_Stream *) this->object ();

  for (ACE_Module_Type *m = this->head_; m != 0; )
    {
      ACE_Module_Type *t = m->link ();

      // Final 0 arg disables ACE_Module::DELETE_THIS 
      str->remove (m->name (), 0); 
      m->fini ();
      m = t;
    }

  str->close ();
  return ACE_Service_Type::fini ();
}

// Locate and remove MOD_NAME from the ACE_Stream.

int
ACE_STREAM_Type::remove (ACE_Module_Type *mod)
{
  ACE_TRACE ("ACE_STREAM_Type::remove");
  ACE_Module_Type *prev = 0;
  MT_Stream *str = (MT_Stream *) this->object ();
  int	      result = 0;

  for (ACE_Module_Type *m = this->head_; m != 0; m = m->link ())
    if (m == mod)
      {
	if (prev == 0)
	  this->head_ = m->link ();
        else	  
	  prev->link (m->link ());

	// Final 0 arg disables ACE_Module::DELETE_THIS 
	if (str->remove (m->name (), 0) == -1) 
	  result = -1; 
	m->fini ();
      }
    else
      prev = m;

  return result;
}

int
ACE_STREAM_Type::push (ACE_Module_Type *new_module)
{
  ACE_TRACE ("ACE_STREAM_Type::push");
  MT_Stream *str = (MT_Stream *) this->object ();

  new_module->link (this->head_);
  this->head_ = new_module;
  return str->push ((MT_Module *) new_module->object ());
}

ACE_Module_Type *
ACE_STREAM_Type::find (const char *mod_name) const
{
  ACE_TRACE ("ACE_STREAM_Type::find");
  for (ACE_Module_Type *m = this->head_; m != 0; m = m->link ())
    if (ACE_OS::strcmp (m->name (), mod_name) == 0)
      return m;

  return 0;
}

#endif /* ACE_HAS_THREADS */

ACE_ALLOC_HOOK_DEFINE(ACE_Service_Record)

void
ACE_Service_Record::dump (void) const
{
  ACE_TRACE ("ACE_Service_Record::dump");
}

ACE_Service_Record::ACE_Service_Record (const char *n, 
					ACE_Service_Type *t, 
					const void *h, 
					int active)
  : type_ (t), handle_ (h), active_ (active)
{
  ACE_TRACE ("ACE_Service_Record::ACE_Service_Record");
  this->name (n);
}

ACE_Service_Record::~ACE_Service_Record (void)
{
  ACE_TRACE ("ACE_Service_Record::~ACE_Service_Record");
  this->type_->fini ();
  if (this->handle_ != 0)
    ACE_OS::dlclose ((void *) this->handle_);
  delete [] (char *) this->name_;
}

void 
ACE_Service_Record::suspend (void) const
{
  ACE_TRACE ("ACE_Service_Record::suspend");
  ((ACE_Service_Record *) this)->active_ = 0;
  this->type_->suspend ();
}

void 
ACE_Service_Record::resume (void) const
{
  ACE_TRACE ("ACE_Service_Record::resume");
  ((ACE_Service_Record *) this)->active_ = 1;
  this->type_->resume ();
}

#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
template class ACE_TSS<ACE_Client_ID>;
#if defined(ACE_HAS_THREADS)
template class ACE_Condition<ACE_Mutex>;
template class ACE_Module<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Stream<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Task<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Message_Queue<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Task_Exit<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_TSS<ACE_Task_Exit<ACE_Mutex, ACE_Condition<ACE_Mutex> > >;
template class ACE_Thru_Task<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Stream_Head<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
template class ACE_Stream_Tail<ACE_Mutex, ACE_Condition<ACE_Mutex> >;
#endif /* ACE_HAS_THREADS */
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */
