/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ACE
// 
// = FILENAME
//    ACE_Singleton.h
//
// = DESCRIPTION
//
// = AUTHOR
//    Tim Harrison (harrison@cs.wustl.edu)
// 
// ============================================================================

#if !defined (ACE_SINGLETON_H)
#define ACE_SINGLETON_H

template <class TYPE, class LOCK>
class ACE_Singleton
  // = TITLE
  //   A Singleton Adapter.
  //   
  // = DESCRIPTION
  //   Uses the Adapter pattern to turn ordinary classes into
  //   Singletons optimized with the Double-Check pattern.
{
public:
  static TYPE *instance (void);
  // Global access point to the Singleton.

protected:
  static TYPE *instance_;
  // Pointer to the Singleton instance.

  static LOCK lock_;
  // Lock used to ensure atomic initialization.
};

#if defined (__INLINE__)
#include "ace/Singleton.i"
#endif /* __INLINE__ */

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "ace/Singleton.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */

#if defined (ACE_TEMPLATES_REQUIRE_PRAGMA)
#pragma implementation ("Singleton.cpp")
#endif /* ACE_TEMPLATES_REQUIRE_PRAGMA */

#endif /* ACE_SINGLETON_H */


