#if !defined (ACE_STREAM_C)
#define ACE_STREAM_C

#if defined (ACE_HAS_THREADS)

//#include "ace/Module.h"
#include "ace/Stream.h"
#include "ace/Stream_Modules.h"

#if !defined (__INLINE__)
#include "ace/Stream.i"
#endif /* __INLINE__ */

ACE_ALLOC_HOOK_DEFINE(ACE_Stream)

/* Give some idea of what the heck is going on in a stream! */

template <ACE_SYNCH_1> void
ACE_Stream<ACE_SYNCH_2>::dump (void) const
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::dump");
  ACE_DEBUG ((LM_INFO, "-------- module links --------\n"));

  for (ACE_Module<ACE_SYNCH_2> *mp = this->stream_head_; ; mp = mp->next ())
    {
      ACE_DEBUG ((LM_INFO, "module name = %s\n", mp->name ()));
      if (mp == this->stream_tail_)
	break;
    }

  ACE_DEBUG ((LM_INFO, "-------- writer links --------\n"));

  ACE_Task<ACE_SYNCH_2> *qp;

  for (qp = this->stream_head_->writer (); ; qp = qp->next ())
    {
      ACE_DEBUG ((LM_INFO, "writer queue name = %s\n", qp->name ()));
      qp->dump ();
      ACE_DEBUG ((LM_INFO, "-------\n"));
      if (qp == this->stream_tail_->writer ()
	  || (this->linked_us_ && qp == this->linked_us_->stream_head_->reader ()))
	break;
    }

  ACE_DEBUG ((LM_INFO, "-------- reader links --------\n"));
  for (qp = this->stream_tail_->reader (); ; qp = qp->next ())
    {
      ACE_DEBUG ((LM_INFO, "reader queue name = %s\n", qp->name ()));
      qp->dump ();
      ACE_DEBUG ((LM_INFO, "-------\n"));
      if (qp == this->stream_head_->reader ()
	  || (this->linked_us_ && qp == this->linked_us_->stream_head_->writer ()))
	break;
    }
}

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::push (ACE_Module<ACE_SYNCH_2> *new_top)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::push");
  if (this->push_module  (new_top, 
			  this->stream_head_->next (), 
			  this->stream_head_) == -1)
    return -1;
  else
    return 0;
}

template <ACE_SYNCH_1> int
ACE_Stream_Iterator<ACE_SYNCH_2>::advance (void)
{
  ACE_TRACE ("ACE_Stream_Iterator<ACE_SYNCH_2>::advance");
  this->next_ = this->next_->next ();
  return 0;
}

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::put (ACE_Message_Block *mb, ACE_Time_Value *tv)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::put");
  return this->stream_head_->writer ()->put (mb, tv);
}

template <ACE_SYNCH_1> int 
ACE_Stream<ACE_SYNCH_2>::get (ACE_Message_Block *&mb, ACE_Time_Value *tv)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::get");
  return this->stream_head_->reader ()->getq (mb, tv);
}

/* Return the "top" ACE_Module in a ACE_Stream, skipping over the
   stream_head. */

template <ACE_SYNCH_1> int 
ACE_Stream<ACE_SYNCH_2>::top (ACE_Module<ACE_SYNCH_2> *&m)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::top");
  if (this->stream_head_->next () == this->stream_tail_)
    return -1;
  else
    {
      m = this->stream_head_->next ();
      return 0;
    }
}

/* Remove the "top" ACE_Module in a ACE_Stream, skipping over the stream_head. */

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::pop (u_long flags)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::pop");
  if (this->stream_head_->next () == this->stream_tail_)
    return -1;
  else
    {
      /* Skip over the ACE_Stream head */
      ACE_Module<ACE_SYNCH_2> *top = this->stream_head_->next ();
      ACE_Module<ACE_SYNCH_2> *new_top = top->next ();

      this->stream_head_->next (new_top);

      // Close the top ACE_Module.

      top->close (flags);

      this->stream_head_->writer ()->next (new_top->writer ());
      new_top->reader ()->next (this->stream_head_->reader ());

      return 0;
    }
}

/* Remove a named ACE_Module from an arbitrary place in the ACE_Stream */

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::remove (const char *name, u_long flags)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::remove");
  ACE_Module<ACE_SYNCH_2> *prev = 0;

  for (ACE_Module<ACE_SYNCH_2> *mod = this->stream_head_; mod != 0; mod = mod->next ())
    if (ACE_OS::strcmp (mod->name (), name) == 0)
      {
	if (prev == 0) // Deleting ACE_Stream Head
	  this->stream_head_->link (mod->next ());
	else
	  prev->link (mod->next ());

	mod->close (flags);
	return 0;
      }
    else
      prev = mod;

  return -1;
}

template <ACE_SYNCH_1> ACE_Module<ACE_SYNCH_2> *
ACE_Stream<ACE_SYNCH_2>::find (const char *name)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::find");
  for (ACE_Module<ACE_SYNCH_2> *mod = this->stream_head_; 
       mod != 0; 
       mod = mod->next ())
    if (ACE_OS::strcmp (mod->name (), name) == 0)
	return mod;

  return 0;
}

/* Actually push a module onto the stack... */

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::push_module (ACE_Module<ACE_SYNCH_2> *new_top, 
				      ACE_Module<ACE_SYNCH_2> *current_top, 
				      ACE_Module<ACE_SYNCH_2> *head)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::push_module");
  ACE_Task<ACE_SYNCH_2> *nt_reader = new_top->reader ();
  ACE_Task<ACE_SYNCH_2> *nt_writer = new_top->writer ();
  ACE_Task<ACE_SYNCH_2> *ct_reader = 0;
  ACE_Task<ACE_SYNCH_2> *ct_writer = 0;

  if (current_top)
    {
      ct_reader = current_top->reader ();
      ct_writer = current_top->writer ();
      ct_reader->next (nt_reader);
    }

  nt_writer->next (ct_writer);

  if (head)
    {
      if (head != new_top)
	head->link (new_top);
    }
  else
    nt_reader->next (0);

  new_top->next (current_top);

  if (nt_reader->open (new_top->arg ()) == -1)
    return -1;

  if (nt_writer->open (new_top->arg ()) == -1)
    return -1;
  return 0;
}

#if 0
int
ACE_Stream<ACE_SYNCH_2>::open (void *a, 
			   ACE_Multiplexor &muxer, 
			   ACE_Module<ACE_SYNCH_2> *head)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::open");
  this->stream_head_ = head == 0
    ? new ACE_Module<ACE_SYNCH_2> ("ACE_Stream_Head", 
			       new ACE_Stream_Head<ACE_SYNCH_2>, 
			       new ACE_Stream_Head<ACE_SYNCH_2>, a) : head;
  this->stream_tail_ = 0;
  return muxer.link_from_below (this->stream_head_);
}
#endif 

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::open (void *a, 
			   ACE_Module<ACE_SYNCH_2> *head, 
			   ACE_Module<ACE_SYNCH_2> *tail)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::open");
  ACE_MT (ACE_Guard<ACE_SYNCH_MUTEX> m (this->lock_));

  if (head == 0)
    {
      ACE_Task<ACE_SYNCH_2> *t1 = new ACE_Stream_Head<ACE_SYNCH_2>;
      ACE_Task<ACE_SYNCH_2> *t2 = new ACE_Stream_Head<ACE_SYNCH_2>;
      head = new ACE_Module<ACE_SYNCH_2> ("ACE_Stream_Head", t1, t2, a);
    }
  this->stream_head_ = head;

  if (tail == 0)
    tail = new ACE_Module<ACE_SYNCH_2> ("ACE_Stream_Tail", 
				    new ACE_Stream_Tail<ACE_SYNCH_2>, 
				    new ACE_Stream_Tail<ACE_SYNCH_2>, a);
  this->stream_tail_ = tail;
  if (this->push_module (this->stream_tail_) == -1)
    return -1;
  if (this->push_module (this->stream_head_, 
			 this->stream_tail_, 
			 this->stream_head_) == -1)
    return -1;
  return 0;
}

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::close (u_long flags)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::close");
  ACE_MT (ACE_Guard<ACE_SYNCH_MUTEX> m (this->lock_));
  int res = 0;

  /* Don't bother checking return value here */
  this->unlink ();

  /* Remove and cleanup all the intermediate modules */

  while (this->stream_head_->next () != this->stream_tail_)
    if (this->pop (flags) == -1)
      res = -1;
  
  /* Clean up the head and tail of the stream */
  if (this->stream_head_->close (flags) == -1)
    res = -1;
  if (this->stream_tail_->close (flags) == -1)
    res = -1;

  this->stream_head_ = 0;
  this->stream_tail_ = 0;

  /* Tell all threads waiting on the close that we are done */
  this->final_close_.broadcast ();
  return res;
}

template <ACE_SYNCH_1> int 
ACE_Stream<ACE_SYNCH_2>::control (ACE_IO_Cntl_Msg::ACE_IO_Cntl_Cmds cmd, void *a)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::control");
  ACE_IO_Cntl_Msg ioc (cmd);
  int	      res = 0;

  ACE_Message_Block *db = 
    new ACE_Message_Block (sizeof (int), ACE_Message_Block::MB_IOCTL, 0, (char *) a);
  ACE_Message_Block	*cb = 
    new ACE_Message_Block (sizeof ioc, ACE_Message_Block::MB_IOCTL, db, (char *) &ioc);

  if (this->stream_head_->writer ()->put (cb) == -1)
    res = -1;
  else if (this->stream_head_->reader ()->getq (cb) == -1)
    res = -1;
  else
    res = ((ACE_IO_Cntl_Msg *) cb->rd_ptr ())->rval ();

  delete cb;
  return res;
}

/* Link two streams together at their bottom-most Modules */

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::link (ACE_Stream<ACE_SYNCH_2> &us)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::link");
  this->linked_us_ = &us;
  ACE_Module<ACE_SYNCH_2> *my_tail;
  ACE_Module<ACE_SYNCH_2> *other_tail;

  for (my_tail = this->stream_head_;
       my_tail->next () != this->stream_tail_;
       my_tail = my_tail->next ())
    continue;

  for (other_tail = us.stream_head_;
       other_tail->next () != us.stream_tail_;
       other_tail = other_tail->next ())
    continue;

  my_tail->writer ()->next (other_tail->reader ());
  other_tail->writer ()->next (my_tail->reader ());
  return 0;
}

template <ACE_SYNCH_1> int
ACE_Stream<ACE_SYNCH_2>::unlink (void)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::unlink");
  if (this->linked_us_ != 0)
    {
      ACE_Module<ACE_SYNCH_2> *my_tail;
      ACE_Module<ACE_SYNCH_2> *other_tail;

      for (my_tail = this->stream_head_;
	   my_tail->next () != this->stream_tail_;
	   my_tail = my_tail->next ())
	continue;

      for (other_tail = this->linked_us_->stream_head_;
	   other_tail->next () != this->linked_us_->stream_tail_;
	   other_tail = other_tail->next ())
	continue;

      my_tail->writer ()->next (this->stream_tail_->writer ());
      other_tail->writer ()->next (this->linked_us_->stream_tail_->writer ());
      this->linked_us_ = 0;
      return 0;
    }
  else
    return -1;
}

template <ACE_SYNCH_1> INLINE
ACE_Stream<ACE_SYNCH_2>::ACE_Stream (void * a, 
			     ACE_Module<ACE_SYNCH_2> *head, 
			     ACE_Module<ACE_SYNCH_2> *tail)
  : final_close_ (this->lock_), 
    linked_us_ (0)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::ACE_Stream");
  if (this->open (a, head, tail) == -1)
    ACE_ERROR ((LM_ERROR, "ACE_Stream<ACE_SYNCH_2>::open (%s, %s)\n", 
	       head->name (), tail->name ()));
}

#if 0
INLINE
ACE_Stream<ACE_SYNCH_2>::ACE_Stream (void *a, 
				     ACE_Multiplexor &muxer, 
				     ACE_Module<ACE_SYNCH_2> *head)
  : final_close_ (this->lock_), 
    linked_us_ (0)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::ACE_Stream");
  if (this->open (a, muxer, head) == -1)
    ACE_ERROR ((LM_ERROR, "ACE_Stream<ACE_SYNCH_2>::open (%s, %s)\n", 
		head->name ()));
}
#endif 

template <ACE_SYNCH_1> INLINE
ACE_Stream<ACE_SYNCH_2>::~ACE_Stream (void)
{
  ACE_TRACE ("ACE_Stream<ACE_SYNCH_2>::~ACE_Stream");
  if (this->stream_head_ != 0)
    this->close ();
}

template <ACE_SYNCH_1> INLINE
ACE_Stream_Iterator<ACE_SYNCH_2>::ACE_Stream_Iterator (const ACE_Stream<ACE_SYNCH_2> &sr)
  : next_ (sr.stream_head_)
{
  ACE_TRACE ("ACE_Stream_Iterator<ACE_SYNCH_2>::ACE_Stream_Iterator");
}

#endif /* ACE_HAS_THREADS */
#endif /* ACE_STREAM_C */
