%{
/* Sample lexical analysis for regular expression subset.  
   Must be compiled with FLEX and an ANSI C++ compiler. */

/* Lexical tokens values defined by BISON. */   
#include "ace/Svc_Conf.h"
#include "ace/sc_tokens.h"

/* Keeps track of the current line for debugging output. */
int yylineno = 1;

/* Keeps track of the number of errors encountered so far. */
int yyerrno = 0;

#if defined (DEBUGGING) /* Debugging version - if assert ok. */
#include <assert.h>

/* Perform the `stringizing' function provided by ANSI C preprocessors. */
#define token(x)  #x

YYSTYPE yylval;

main (int argc, char **argv)
{ 
  char *p;
  assert (sizeof (int) >= sizeof (char *));

  while (p = yylex ())
    ACE_DEBUG ((LM_DEBUG, "%-12.12s is \"%s\"\n", 
		p, yytext[0] == '\n' ? "\\n" : yytext));

  return 0;
}
#else 
/* Production version, no DEBUGGING. */
#define token(x)  x
#endif /* DEBUGGING */
%}

%s PARAMETERS
%s NORMAL

letter	        [a-zA-Z_]
letter_or_digit	[a-zA-Z_0-9]
digit           [0-9]
ident           {letter}{letter_or_digit}*
pathname        [a-zA-Z_0-9/\.-]+
symbol		[ -~]
string		\"{symbol}+\"
white_space	[ \t]
newline         \n
other		.

%%

^#{other}*$   	      ; /* EMPTY */
dynamic	              { return token (DYNAMIC); }
static	              { return token (STATIC); }
suspend	              { return token (SUSPEND); }
resume	              { return token (RESUME); }
remove	              { return token (REMOVE); }
stream	              { return token (USTREAM); }
Module		      { return token (MODULE_T); }
Service_Object	      { return token (SVC_OBJ_T); }
STREAM		      { return token (STREAM_T); }
active	              { return token (ACTIVE); }
inactive	      { return token (INACTIVE); }
":"		      { return token (COLON); }
"*"		      { return token (STAR); }
"("                   { return token (LPAREN); }
")"                   { return token (RPAREN); }
"{"                   { return token (LBRACE); }
"}"                   { return token (RBRACE); }
{string}	      { // Eliminate the opening and closing double quotes
		        *strrchr (yytext, '"') = '\0';
			yyleng -= 1;
                        yylval.ident_ = ace_obstack->copy (yytext + 1, yyleng); 
			return token (STRING); }
{ident}		      { 
		        yylval.ident_ = ace_obstack->copy (yytext, yyleng); 
			return token (IDENT); 
		      }
{pathname}	      { 
		        yylval.ident_ = ace_obstack->copy (yytext, yyleng); 
			return token (PATHNAME); 
		      }
{white_space}+	      ; /* EMPTY */
{newline}	      { yylineno++; }
{other}		      { ACE_ERROR ((LM_ERROR, "unknown char = %d\n", *yytext)); }
<<EOF>>               { YY_NEW_FILE; yyterminate(); }
%%
int
yywrap (void)
{
  ::fflush (yyin);
  yytext[0] = '#';
  yyleng = 0;    
  return 1;
}
