%{
#include "ace/ARGV.h"
#include "ace/Svc_Conf.h"
#include "ace/Module.h"
#include "ace/Stream.h"

#if defined (ACE_HAS_THREADS)
static ACE_Module_Type *get_module (ACE_Static_Node *str_rec, ACE_Static_Node *svc_type);
static ACE_Module_Type *get_module (ACE_Static_Node *str_rec, const char *svc_name);
#endif /* ACE_HAS_THREADS */

#define YYDEBUG_LEXER_TEXT (yytext[yyleng] = '\0', yytext) 
/* Force the pretty debugging code to compile*/
#define YYDEBUG 1 

/* Efficient memory allocation technique */
ACE_Obstack *ace_obstack;

%}
%token DYNAMIC STATIC SUSPEND RESUME REMOVE USTREAM
%token MODULE_T STREAM_T SVC_OBJ_T ACTIVE INACTIVE
%token PATHNAME IDENT STRING
%token LPAREN RPAREN LBRACE RBRACE STAR COLON

%start svc_config_entries
%union 
{
  int type_;
  ACE_Location_Node *location_node_;
  ACE_Parse_Node *parse_node_;
  ACE_Static_Node *ACE_Static_Node_;
  ACE_Service_Record *svc_record_;
  char *ident_; 
}

%type <ident_> IDENT STRING PATHNAME parameters_opt 
%type <type_> type status
%type <parse_node_> dynamic static suspend resume remove module_list stream
%type <parse_node_> stream_modules module svc_config_entry 
%type <ACE_Static_Node_> stream_ops
%type <svc_record_> svc_location
%type <location_node_> svc_initializer

%%

svc_config_entries
  : svc_config_entries svc_config_entry 
    {
      $2->apply (); delete $2; ace_obstack->release (); 
    }
  | svc_config_entries error 
    { 
      ace_obstack->release (); 
    }
  | /* EMPTY */
  ;

svc_config_entry 
  : dynamic 
  | static  
  | suspend 
  | resume  
  | remove  
  | stream
  ;

dynamic 
  : DYNAMIC svc_location parameters_opt
    { 
      $$ = new ACE_Dynamic_Node ($2, $3);
    }
  ;

static 
  : STATIC IDENT parameters_opt 
    { 
      $$ = new ACE_Static_Node ($2, $3);
    }
  ;

suspend
  : SUSPEND IDENT 
    { 
      $$ = new ACE_Suspend_Node ($2);
    }
  ;

resume 
  : RESUME IDENT 
    { 
      $$ = new ACE_Resume_Node ($2);
    }
  ;

remove 
  : REMOVE IDENT 
    { 
      $$ = new ACE_Remove_Node ($2);
    }
  ;

stream 
  : USTREAM stream_ops stream_modules
    {
      $$ = new ACE_Stream_Node ($2, $3);
    }
  | USTREAM IDENT { $<ACE_Static_Node_>$ = new ACE_Static_Node ($2); } stream_modules
    {
      $$ = new ACE_Dummy_Node ($<ACE_Static_Node_>3, $4);
    }
  ;

stream_ops
  : dynamic
    {
    }	
  | static
    {
    }	
  ;

stream_modules
  : LBRACE 
    { 
      // Initialize left context...
      $<ACE_Static_Node_>$ = $<ACE_Static_Node_>0; 
    } 
   module_list RBRACE
    {
      $$ = $3;
    }
  | /* EMPTY */ { $$ = 0; }
  ;

module_list 
  : module_list module { $2->link ($1); $$ = $2; }
  | /* EMPTY */ { $$ = 0; }
  ;

module 
  : dynamic 
    {
#if defined (ACE_HAS_THREADS)
      ACE_ARGV argv ($<ACE_Static_Node_>1->parameters ());
      ACE_Module_Type     *mt = get_module ($<ACE_Static_Node_>-1, $<ACE_Static_Node_>1);

      if (::strcmp ($<ACE_Static_Node_>1->name (), 
		    ((MT_Module *) mt->object ())->name ()) != 0)
	ACE_ERROR ((LM_ERROR, "warning, service name %s is different from Module name %s\n",
		   $<ACE_Static_Node_>1->name (), ((MT_Module *) mt->object ())->name ()));

      if (mt->init (argv.count (), &argv) == -1
	  || ((ACE_STREAM_Type *) ($<ACE_Static_Node_>-1)->record ()->type ())->push (mt) == -1)
	{
	  ACE_ERROR ((LM_ERROR, "dynamic initialization failed for Module %s\n",
		     $<ACE_Static_Node_>1->name ()));
	  yyerrno++;
	}
#else
      ACE_ERROR ((LM_ERROR, "Streams are not supported on this platform...\n"));
#endif				/* ACE_HAS_THREADS */
    }
  | static  
    { 
#if defined (ACE_HAS_THREADS)
      ACE_Module_Type *mt = get_module ($<ACE_Static_Node_>-1, $<ACE_Static_Node_>1->name ());
      if (::strcmp ($<ACE_Static_Node_>1->name (), 
		    ((MT_Module *) mt->object ())->name ()) != 0)
	ACE_ERROR ((LM_ERROR, "warning, service name %s is different from Module name %s\n",
		   $<ACE_Static_Node_>1->name (), ((MT_Module *) mt->object ())->name ()));
      if (((ACE_STREAM_Type *) ($<ACE_Static_Node_>-1)->record ()->type ())->push (mt) == -1)
	yyerrno++;
#else
      ACE_ERROR ((LM_ERROR, "Streams are not supported on this platform...\n"));
#endif				/* ACE_HAS_THREADS */
    }
  | suspend 
    { 
#if defined (ACE_HAS_THREADS)
      ACE_Module_Type *mt = get_module ($<ACE_Static_Node_>-1, $<ACE_Static_Node_>1->name ());
      if (mt != 0)
	mt->suspend ();
#else
      ACE_ERROR ((LM_ERROR, "Streams are not supported on this platform...\n"));
#endif				/* ACE_HAS_THREADS */
    }
  | resume  
    {
#if defined (ACE_HAS_THREADS)
      ACE_Module_Type *mt = get_module ($<ACE_Static_Node_>-1, $<ACE_Static_Node_>1->name ());
      if (mt != 0)
	mt->resume ();
#else
      ACE_ERROR ((LM_ERROR, "Streams are not supported on this platform...\n"));
#endif				/* ACE_HAS_THREADS */
    }
  | remove  
    { 
#if defined (ACE_HAS_THREADS)
      ACE_Module_Type *mt = get_module ($<ACE_Static_Node_>-1, $<ACE_Static_Node_>1->name ());
      if (mt != 0 
	  && ((ACE_STREAM_Type *) ($<ACE_Static_Node_>-1)->record ()->type ())->remove (mt) == -1)
	{
	  ACE_ERROR ((LM_ERROR, "cannot remove Module_Type %s from STREAM_Type %s\n",
		     $<ACE_Static_Node_>1->name (), ($<ACE_Static_Node_>-1)->name ()));
	  yyerrno++;
	}
#else
      ACE_ERROR ((LM_ERROR, "Streams are not supported on this platform...\n"));
#endif				/* ACE_HAS_THREADS */
    }
  ;

svc_location
  : IDENT type svc_initializer status
    {
      unsigned int flags 
	= ACE_Service_Type::DELETE_THIS | ($3->dispose () == 0 ? 0 : ACE_Service_Type::DELETE_OBJ);
      ACE_Service_Type *stp = ace_create_service_type ($1, $2, $3->symbol (), flags);
      $$ = new ACE_Service_Record ($1, stp, $3->handle (), $4);
    }
  ;

status
  : ACTIVE
    {
      $$ = 1;
    }  
  | INACTIVE
    {
      $$ = 0;
    }  
  | /* EMPTY */
    {
      $$ = 1;
    }  
  ;

svc_initializer
  : PATHNAME COLON IDENT
    {
      $$ = new ACE_Object_Node ($1, $3);
    }
  | PATHNAME COLON IDENT LPAREN RPAREN
    {
      $$ = new ACE_Function_Node ($1, $3);
    }
  ;

type
  : MODULE_T STAR
    {
      $$ = MODULE_T;
    }    
  | SVC_OBJ_T STAR
    {
      $$ = SVC_OBJ_T;
    }    
  | STREAM_T STAR
    {
      $$ = STREAM_T;
    }    
  ;

parameters_opt
  : STRING
  | /* EMPTY */ { $$ = 0; } 
  ;

%%
/* Prints the error string to standard output.  
   Cleans up the error messages. */

void
yyerror (char *s)
{
  ACE_ERROR ((LM_ERROR, "[error %d] on line %d: %s\n", 
	      ++yyerrno, yylineno, s));
}

/* Note that SRC_REC represents left context, which is the STREAM * record. */

#if defined (ACE_HAS_THREADS)

static ACE_Module_Type *
get_module (ACE_Static_Node *str_rec, const char *svc_name)
{
  const ACE_Service_Record *sr = str_rec->record ();
  ACE_STREAM_Type	       *st = sr == 0 ? 0 : (ACE_STREAM_Type *) sr->type ();
  ACE_Module_Type	       *mt = st == 0 ? 0 : st->find (svc_name);

  if (sr == 0 || st == 0 || mt == 0)
    {
      ACE_ERROR ((LM_ERROR, "cannot locate Module_Type %s in STREAM_Type %s\n",
		 svc_name, str_rec->name ()));
      yyerrno++;
    }
  return mt;
}

static ACE_Module_Type *
get_module (ACE_Static_Node *str_rec, ACE_Static_Node *svc_type)
{
  const ACE_Service_Record *sr = str_rec->record ();
  ACE_STREAM_Type	       *st = sr == 0 ? 0 : (ACE_STREAM_Type *) sr->type ();
  ACE_Module_Type	       *mt = (ACE_Module_Type *) svc_type->record ()->type ();

  if (sr == 0 || st == 0 || mt == 0)
    {
      ACE_ERROR ((LM_ERROR, "cannot locate Module_Type %s or STREAM_Type %s\n",
		 svc_type->name (), str_rec->name ()));
      yyerrno++;
    }
  return mt;
}
#endif				/* ACE_HAS_THREADS */

ACE_Service_Type *
ace_create_service_type (const char *name, 
			 int type, 
			 const void *symbol, 
			 unsigned int flags)
{
  ACE_Service_Type *stp = 0;

  /* Note, the only place we need to put a case statement.
     This is also the place where we'd put the RTTI tests,
     if the compiler actually supported them! */

  switch (type)
    {
    case SVC_OBJ_T:
      stp = new ACE_Service_Object_Type ((ACE_Service_Object *) symbol, name, flags);
      break;
    case MODULE_T:
#if defined (ACE_HAS_THREADS)
      stp = new ACE_Module_Type ((MT_Module *) symbol, name, flags);
      break;
#else
      ACE_ERROR ((LM_ERROR, "Platform does not support Modules...\n"));
#endif				/* ACE_HAS_THREADS */
    case STREAM_T:
#if defined (ACE_HAS_THREADS)
      stp = new ACE_STREAM_Type ((MT_Stream *) symbol, name, flags);
      break;
#else
      ACE_ERROR ((LM_ERROR, "Platform does not support STREAMs...\n"));
#endif				/* ACE_HAS_THREADS */
    default:
      ACE_ERROR ((LM_ERROR, "unknown case\n"));
      yyerrno++;
      break;
    }
  return stp;
}

#if defined (DEBUGGING)
/* Current line number. */	
int yylineno = 1;

/* Name given on the command-line to envoke the program. */
char *program_name;

/* Main driver program. */

int 
main (int argc, char **argv)
{ 
  yyin = stdin;

  /* Try to reopen any filename argument to use YYIN. */
  if (argc > 1 && (yyin = freopen (argv[1], "r", stdin)) == 0)
    (void) ::fprintf (stderr, "usage: %s [file]\n", argv[0]), exit (1);

  return yyparse ();
}
#endif /* DEBUGGING */
