#if !defined (ACE_SYNCH_T_C)
#define ACE_SYNCH_T_C

#include "ace/Log_Msg.h"
#include "ace/Thread.h"
#include "ace/Time_Value.h"
#include "ace/Synch_T.h"

#if !defined (__INLINE__)
#include "ace/Synch_T.i"
#endif /* __INLINE__ */

ACE_ALLOC_HOOK_DEFINE(ACE_TSS)

template <class TYPE> void
ACE_TSS<TYPE>::dump (void) const
{
}

#if defined (ACE_HAS_THREADS)

#if defined (__osf__) && ! defined (__GNUG__)
#pragma define_template ACE_Condition<ACE_Mutex>
#pragma define_template ACE_Recursive_Lock<ACE_Mutex>
#pragma define_template ACE_Guard< ACE_Recursive_Lock <ACE_Mutex> >
#endif

ACE_ALLOC_HOOK_DEFINE(ACE_Atomic_Op)

template <class LOCK, class TYPE> void
ACE_Atomic_Op<LOCK, TYPE>::dump (void) const
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::dump");
}

template <class LOCK, class TYPE>
ACE_Atomic_Op<LOCK, TYPE>::ACE_Atomic_Op (void)
  : type_ (0)
{ 
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::ACE_Atomic_Op");
}

template <class LOCK, class TYPE>
ACE_Atomic_Op<LOCK, TYPE>::ACE_Atomic_Op (TYPE c) 
{ 
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::ACE_Atomic_Op");
  this->type_ = c; 
}

ACE_ALLOC_HOOK_DEFINE(ACE_Null_Condition)

ACE_ALLOC_HOOK_DEFINE(ACE_Guard)

template <class LOCK> void
ACE_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_Guard<LOCK>::dump");
}

ACE_ALLOC_HOOK_DEFINE(ACE_Write_Guard)

template <class LOCK> void
ACE_Write_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_Write_Guard<LOCK>::dump");
}

ACE_ALLOC_HOOK_DEFINE(ACE_Read_Guard)

template <class LOCK> void
ACE_Read_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_Read_Guard<LOCK>::dump");
}

ACE_ALLOC_HOOK_DEFINE(ACE_Recursive_Lock)

template <class LOCK> void
ACE_Recursive_Lock<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::dump");
}

template <class LOCK>
ACE_Recursive_Lock<LOCK>::ACE_Recursive_Lock (int type, 
					      const char *name,
					      void *arg)
  : nesting_level_ (0), 
    thr_id_ (ACE_OS::NULL_thread),
    lock_ (type, name, arg),
    mutex_ (type, name, arg)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::ACE_Recursive_Lock");
}

template <class LOCK>
ACE_Recursive_Lock<LOCK>::~ACE_Recursive_Lock (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::~ACE_Recursive_Lock");
}

template <class LOCK> int
ACE_Recursive_Lock<LOCK>::acquire (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::acquire");
  this->mutex_.acquire ();
  thread_t t_id = ACE_Thread::self ();

  if (ACE_OS::thr_equal (t_id, this->thr_id_))
    {
      ++this->nesting_level_;
      this->mutex_.release ();
      return 0;
    }
  else
    {
      this->mutex_.release ();
      int result = this->lock_.acquire ();

      if (result == 0)
	{
	  this->set_thread_id (t_id);
	  this->set_nesting_level (0);
	}
      return result;
    }
}
 
template <class LOCK> int 
ACE_Recursive_Lock<LOCK>::release (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::release");
  this->mutex_.acquire ();

  // assert (this->get_thread_id () == ACE_Thread::self ());
  if (this->nesting_level_ > 0)
    {
      --this->nesting_level_;
      this->mutex_.release ();
      return 0;
    }
  else
    {
      this->set_thread_id (ACE_OS::NULL_thread);
      this->mutex_.release ();
      return this->lock_.release ();
    }
}

template <class LOCK> int
ACE_Recursive_Lock<LOCK>::tryacquire (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::tryacquire");
  this->mutex_.acquire ();
  thread_t t_id = ACE_Thread::self ();

  if (ACE_OS::thr_equal (t_id, this->thr_id_))
    {
      ++this->nesting_level_;
      this->mutex_.release ();
      return 0;
    }
  else
    {
      this->mutex_.release ();
      int result = this->lock_.tryacquire ();

      if (result == 0)
	{
	  this->set_thread_id (t_id);
	  this->set_nesting_level (0);
	}
      return result;
    }
}

ACE_ALLOC_HOOK_DEFINE(ACE_Condition)

template <class MUTEX> void
ACE_Condition<MUTEX>::dump (void) const
{
  ACE_TRACE ("ACE_Condition<MUTEX>::dump");
}

template <class MUTEX>
ACE_Thread_Condition<MUTEX>::ACE_Thread_Condition (MUTEX &m, 
						   const char *name,
						   void *arg)
  : ACE_Condition<MUTEX> (m, USYNC_THREAD, name, arg)
{
  ACE_TRACE ("ACE_Thread_Condition<MUTEX>::ACE_Thread_Condition");
}

template <class MUTEX> void
ACE_Thread_Condition<MUTEX>::dump (void) const
{
  ACE_TRACE ("ACE_Thread_Condition<MUTEX>::dump");
}

template <class MUTEX> void
ACE_Process_Condition<MUTEX>::dump (void) const
{
  ACE_TRACE ("ACE_Process_Condition<MUTEX>::dump");
}

template <class MUTEX>
ACE_Process_Condition<MUTEX>::ACE_Process_Condition (MUTEX &m, 
						   const char *name,
						   void *arg)
  : ACE_Condition<MUTEX> (m, USYNC_PROCESS, name, arg)
{
  ACE_TRACE ("ACE_Process_Condition<MUTEX>::ACE_Process_Condition");
}

template <class MUTEX>
ACE_Condition<MUTEX>::ACE_Condition (MUTEX &m, 
				     int type, 
				     const char *name,
				     void *arg)
  : mutex_ (m)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::ACE_Condition");
  if (ACE_OS::cond_init (&this->cond_, type, name, arg) != 0)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Condition::ACE_Condition"));
}

template <class MUTEX>
ACE_Condition<MUTEX>::~ACE_Condition (void)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::~ACE_Condition");
  if (this->remove () == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "ACE_Condition::~ACE_Condition"));
}

// Peform an "alertable" timed wait.  If the argument ABSTIME == 0
// then we do a regular cond_wait(), else we do a timed wait for up to
// ABSTIME using the Solaris cond_timedwait() function.

template <class MUTEX> int
ACE_Condition<MUTEX>::wait (ACE_Time_Value *abstime)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::wait");
  if (abstime == 0)
    return ACE_OS::cond_wait (&this->cond_, 
			      &this->mutex_.lock_);
  else
    return ACE_OS::cond_timedwait (&this->cond_, &this->mutex_.lock_, abstime);
}
#endif /* ACE_HAS_THREADS */

#if (defined (ACE_HAS_THREADS) && defined (ACE_HAS_THREAD_SPECIFIC_STORAGE))
template <class TYPE> 
ACE_TSS<TYPE>::ACE_TSS (TYPE *ts_obj)
  : once_ (0),
    key_ (0)    
{
  // If caller has passed us a non-NULL TYPE *, then we'll just use
  // this to initialize the thread-specific value.  Thus, subsequent
  // calls to operator->() will return this value.  This is useful
  // since it enables us to assign objects to thread-specific data
  // that have arbitrarily complex constructors!

  if (ts_obj != 0)
    {
      assert (this->once_ == 0);

      if (ACE_Thread::keycreate (&this->key_,
				 &ACE_TSS<TYPE>::cleanup) != 0)
	// What should we do if this call fails?!
	assert (!"ACE_Thread::keycreate() failed!");

      this->once_ = 1;

      if (ACE_Thread::setspecific (this->key_, (void *) ts_obj) != 0)
	assert (!"ACE_Thread::setspecific() failed!");
    }
}

// Get the thread-specific object for the key associated with this
// object.  Returns 0 if the ts_obj has never been initialized,
// otherwise returns a pointer to the ts_obj.

template <class TYPE> TYPE *
ACE_TSS<TYPE>::ts_object (void) const
{
  // Ensure that we are serialized!
  ACE_Guard <ACE_Mutex> m ((ACE_Mutex &) this->keylock_);

  if (this->once_ == 0) // Return 0 if we've never been initialized.
    return 0;
  else
    {
      TYPE *ts_obj = 0;

      if (ACE_Thread::getspecific (this->key_, (void **) &ts_obj) == -1)
	return 0; // This should not happen!
      else
	return ts_obj;
    }
}

template <class TYPE> TYPE *
ACE_TSS<TYPE>::ts_object (TYPE *new_ts_obj) 
{
  // Ensure that we are serialized!
  ACE_Guard <ACE_Mutex> m (this->keylock_);

  if (this->once_ == 0) // Return 0 if we've never been initialized.
    return 0;
  else
    {
      TYPE *ts_obj = 0;

      if (ACE_Thread::getspecific (this->key_, (void **) &ts_obj) == -1)
	return 0; // This should not happen!
      if (ACE_Thread::setspecific (this->key_, (void *) new_ts_obj) == -1)
	return ts_obj; // This should not happen!
      else
	return ts_obj;
    }
}

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::ts_get (void) const
{ 
  // Create and initialize thread-specific ts_obj.
  if (this->once_ == 0)
    {
      // Insure that we are serialized!
      ACE_Guard <ACE_Mutex> m ((ACE_Mutex &) (this->keylock_));

      // Use the Double-Check pattern to make sure we only create the
      // key once!
      if (this->once_ == 0)
	{
	  if (ACE_Thread::keycreate ((thread_key_t *) &this->key_,
				     &ACE_TSS<TYPE>::cleanup) != 0)
	    return 0; // Major problems, this should *never* happen!

	  // This *must* come last to avoid race conditions!  Note
	  // that we need to "cast away const..."
	  *(int *) &this->once_ = 1;
	}
    }

  TYPE *ts_obj = 0;

  // Get the ts_obj from thread-specific storage.  Note that no locks
  // are required here...
  if (ACE_Thread::getspecific (this->key_, (void **) &ts_obj) == -1)
    return 0; // This should not happen!

  // Check to see if this is the first time in for this thread.
  if (ts_obj == 0)
    {
      // Allocate memory off the heap and store it in a pointer in
      // thread-specific storage (on the stack...).

      ts_obj = this->make_TSS_TYPE ();
	
      if (ts_obj == 0)
	return 0;

      // Store the dynamically allocated pointer in thread-specific
      // storage.
      if (ACE_Thread::setspecific (this->key_, (void *) ts_obj) != 0)
	return 0; // Major problems, this should *never* happen!
    }

  return ts_obj;
}

/* static */
template <class TYPE> void
ACE_TSS<TYPE>::cleanup (void *ptr)
{
  // This cast is necessary to invoke the destructor (if necessary).
  delete (TYPE *) ptr;
}

ACE_ALLOC_HOOK_DEFINE(ACE_TSS_Guard)

template <class LOCK> void
ACE_TSS_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::dump");
}

template <class LOCK> void
ACE_TSS_Guard<LOCK>::init_key (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::init_key");

  this->key_ = 0;
  ACE_Thread::keycreate (&this->key_, 
			 &ACE_TSS_Guard<LOCK>::cleanup);
}

template <class LOCK> 
ACE_TSS_Guard<LOCK>::ACE_TSS_Guard (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::ACE_TSS_Guard");
  this->init_key ();
}

template <class LOCK> int
ACE_TSS_Guard<LOCK>::release (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::release");

  ACE_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->release ();
}

template <class LOCK> int
ACE_TSS_Guard<LOCK>::remove (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::remove");

  ACE_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->remove ();
}

template <class LOCK> 
ACE_TSS_Guard<LOCK>::~ACE_TSS_Guard (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::~ACE_TSS_Guard");

  ACE_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  ACE_Thread::setspecific (this->key_, (void *) 0);

  // Destructor releases lock.
  delete guard;
}

template <class LOCK> void
ACE_TSS_Guard<LOCK>::cleanup (void *ptr)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::dump");

  // Destructor releases lock.
  delete (ACE_Guard<LOCK> *) ptr;
}

template <class LOCK> 
ACE_TSS_Guard<LOCK>::ACE_TSS_Guard (LOCK &lock, int block)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::ACE_TSS_Guard");

  this->init_key ();
  ACE_Guard<LOCK> *guard = new ACE_Guard<LOCK> (lock, block);
  ACE_Thread::setspecific (this->key_, (void *) guard);
}

template <class LOCK> int
ACE_TSS_Guard<LOCK>::acquire (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::acquire");

  ACE_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->acquire ();
}

template <class LOCK> int
ACE_TSS_Guard<LOCK>::tryacquire (void)
{
  ACE_TRACE ("ACE_TSS_Guard<LOCK>::tryacquire");

  ACE_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->tryacquire ();
}

template <class LOCK> 
ACE_TSS_Write_Guard<LOCK>::ACE_TSS_Write_Guard (LOCK &lock, int block)
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::ACE_TSS_Write_Guard");

  this->init_key ();
  ACE_Guard<LOCK> *guard = new ACE_Write_Guard<LOCK> (lock, block);
  ACE_Thread::setspecific (this->key_, (void *) guard);
}

template <class LOCK> int
ACE_TSS_Write_Guard<LOCK>::acquire (void)
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::acquire");

  ACE_Write_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->acquire_write ();
}

template <class LOCK> int
ACE_TSS_Write_Guard<LOCK>::tryacquire (void)
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::tryacquire");

  ACE_Write_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->tryacquire_write ();
}

template <class LOCK> int
ACE_TSS_Write_Guard<LOCK>::acquire_write (void)
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::acquire_write");

  return this->acquire ();
}

template <class LOCK> int
ACE_TSS_Write_Guard<LOCK>::tryacquire_write (void)
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::tryacquire_write");

  return this->tryacquire ();
}

template <class LOCK> void
ACE_TSS_Write_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_TSS_Write_Guard<LOCK>::dump");
}

template <class LOCK> 
ACE_TSS_Read_Guard<LOCK>::ACE_TSS_Read_Guard (LOCK &lock, int block)
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::ACE_TSS_Read_Guard");

  this->init_key ();
  ACE_Guard<LOCK> *guard = new ACE_Read_Guard<LOCK> (lock, block);
  ACE_Thread::setspecific (this->key_, (void *) guard);
}

template <class LOCK> int
ACE_TSS_Read_Guard<LOCK>::acquire (void)
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::acquire");

  ACE_Read_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->acquire_read ();
}

template <class LOCK> int
ACE_TSS_Read_Guard<LOCK>::tryacquire (void)
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::tryacquire");

  ACE_Read_Guard<LOCK> *guard = 0;
  ACE_Thread::getspecific (this->key_, (void **) &guard);
  return guard->tryacquire_read ();
}

template <class LOCK> int
ACE_TSS_Read_Guard<LOCK>::acquire_read (void)
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::acquire_read");

  return this->acquire ();
}

template <class LOCK> int
ACE_TSS_Read_Guard<LOCK>::tryacquire_read (void)
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::tryacquire_read");

  return this->tryacquire ();
}

template <class LOCK> void
ACE_TSS_Read_Guard<LOCK>::dump (void) const
{
  ACE_TRACE ("ACE_TSS_Read_Guard<LOCK>::dump");
}

#endif /* defined (ACE_HAS_THREADS) && defined (ACE_HAS_THREAD_SPECIFIC_STORAGE) */
#endif /* ACE_SYNCH_T_C */
