/* -*- C++ -*- */

#include "ace/Thread.h"

template <class MUTEX> INLINE int
ACE_Null_Condition<MUTEX>::wait (ACE_Time_Value *)
{
  errno = ETIME;
  return -1;
}

#if !(defined (ACE_HAS_THREADS) && defined (ACE_HAS_THREAD_SPECIFIC_STORAGE))
template <class TYPE> INLINE
ACE_TSS<TYPE>::ACE_TSS (TYPE *)
{
}

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::ts_object (void) const
{ 
  return (TYPE *) &this->type_;
}

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::ts_object (TYPE *) 
{ 
  return &this->type_;
}

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::ts_get (void) const
{ 
  return (TYPE *) &this->type_;
}
#endif /* !(defined (ACE_HAS_THREADS) && defined (ACE_HAS_THREAD_SPECIFIC_STORAGE)) */

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::operator-> () const
{
  return this->ts_get ();
}

template <class TYPE> INLINE
ACE_TSS<TYPE>::operator TYPE *(void) const
{ 
  return this->ts_get ();
}

template <class TYPE> INLINE TYPE *
ACE_TSS<TYPE>::make_TSS_TYPE (void) const
{
  return new TYPE;
}

#if defined (ACE_HAS_THREADS)

template <class LOCK, class TYPE> INLINE TYPE 
ACE_Atomic_Op<LOCK, TYPE>::operator++ (void)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator++");
  ACE_Guard<LOCK> m (this->lock_);
  return ++this->type_;
}

template <class LOCK, class TYPE> INLINE TYPE 
ACE_Atomic_Op<LOCK, TYPE>::operator++ (int)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator++");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_++;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator+= (const TYPE i)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator+=");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ += i;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator-- (void)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator--");
  ACE_Guard<LOCK> m (this->lock_);
  return --this->type_;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator-- (int)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator--");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_--;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator-= (const TYPE i)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator-=");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ -= i;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator== (const TYPE i)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator==");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ == i;
}

template <class LOCK, class TYPE> INLINE TYPE
ACE_Atomic_Op<LOCK, TYPE>::operator>= (const TYPE i)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator>=");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ >= i;
}

template <class LOCK, class TYPE> INLINE TYPE 
ACE_Atomic_Op<LOCK, TYPE>::operator> (const TYPE rhs)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator>");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ > rhs;
}

template <class LOCK, class TYPE> INLINE TYPE 
ACE_Atomic_Op<LOCK, TYPE>::operator<= (const TYPE rhs)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator<=");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ <= rhs;
}

template <class LOCK, class TYPE> INLINE TYPE 
ACE_Atomic_Op<LOCK, TYPE>::operator< (const TYPE rhs)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator<");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_ < rhs;
}

template <class LOCK, class TYPE> INLINE void
ACE_Atomic_Op<LOCK, TYPE>::operator= (const TYPE i)
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator=");
  ACE_Guard<LOCK> m (this->lock_);
  this->type_ = i;
}

template <class LOCK, class TYPE> INLINE
ACE_Atomic_Op<LOCK, TYPE>::operator TYPE ()
{
  ACE_TRACE ("ACE_Atomic_Op<LOCK, TYPE>::operator TYPE");
  ACE_Guard<LOCK> m (this->lock_);
  return this->type_;    
}

template<class LOCK> INLINE int
ACE_Recursive_Lock<LOCK>::remove (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::remove");
  return this->lock_.remove ();
}

template<class LOCK> INLINE 
ACE_Recursive_Lock<LOCK>::operator LOCK &()
{
  return this->lock_;
}

template<class LOCK> INLINE thread_t 
ACE_Recursive_Lock<LOCK>::get_thread_id (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::get_thread_id");
  ACE_Guard<LOCK> m (this->mutex_);
  return this->thr_id_;
}

template<class LOCK> INLINE void
ACE_Recursive_Lock<LOCK>::set_thread_id (thread_t t)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::set_thread_id");
  this->thr_id_ = t;
}

template<class LOCK> INLINE int
ACE_Recursive_Lock<LOCK>::get_nesting_level (void)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::get_nesting_level");
  ACE_Guard<LOCK> m (this->mutex_);
  return this->nesting_level_;
}

template<class LOCK> INLINE void     
ACE_Recursive_Lock<LOCK>::set_nesting_level (int d)
{
  ACE_TRACE ("ACE_Recursive_Lock<LOCK>::set_nesting_level");
  this->nesting_level_ = d;
}

template<class MUTEX> INLINE int
ACE_Condition<MUTEX>::remove (void)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::remove");

  // cond_destroy() is called in a loop if the condition variable is
  // BUSY.  This avoids a condition where a condition is signaled and
  // because of some timing problem, the thread that is to be signaled
  // has called the cond_wait routine after the signal call.  Since
  // the condition signal is not queued in any way, deadlock occurs.

  int result = 0;

  while ((result = ACE_OS::cond_destroy (&this->cond_)) == -1
	 && errno == EBUSY)
    {
      ACE_OS::cond_broadcast (&this->cond_);
      ACE_OS::thr_yield ();
    }

  return result;
}

template<class MUTEX> INLINE MUTEX &
ACE_Condition<MUTEX>::mutex (void)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::mutex");
  return this->mutex_;
}

template <class MUTEX> INLINE int 
ACE_Condition<MUTEX>::signal (void)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::signal");
  return ACE_OS::cond_signal (&this->cond_);
}

template <class MUTEX> INLINE int
ACE_Condition<MUTEX>::broadcast (void)
{
  ACE_TRACE ("ACE_Condition<MUTEX>::broadcast");
  return ACE_OS::cond_broadcast (&this->cond_);
}
#endif /* ACE_HAS_THREADS */

