#if !defined (ACE_TASK_C)
#define ACE_TASK_C

#include "ace/Task.h"
#include "ace/Module.h"
#include "ace/Service_Config.h"

#if !defined (__INLINE__)
#include "ace/Task.i"
#endif /* __INLINE__ */

// Grab hold of the Task * so that we can close() it in the
// destructor.

template<ACE_SYNCH_1>
ACE_Task_Exit<ACE_SYNCH_2>::ACE_Task_Exit (ACE_Task<ACE_SYNCH_2> *t)
  : t_ (t), 
    status_ ((void *) -1),
    tc_ (t->thr_mgr_)
{
  ACE_TRACE ("ACE_Task_Exit<ACE_SYNCH_2>::ACE_Task_Exit");
}

// Set the thread exit status value.

template<ACE_SYNCH_1> void *
ACE_Task_Exit<ACE_SYNCH_2>::status (void *s)
{
  ACE_TRACE ("ACE_Task_Exit<ACE_SYNCH_2>::status");
  return this->status_ = s;
}

template<ACE_SYNCH_1> void *
ACE_Task_Exit<ACE_SYNCH_2>::status (void)
{
  ACE_TRACE ("ACE_Task_Exit<ACE_SYNCH_2>::status");
  return this->status_;
}

// When this object is destroyed the Task is automatically closed
// down!

template<ACE_SYNCH_1>
ACE_Task_Exit<ACE_SYNCH_2>::~ACE_Task_Exit (void)
{
  ACE_TRACE ("ACE_Task_Exit<ACE_SYNCH_2>::~ACE_Task_Exit");
  if (this->t_ != 0)
    
      this->t_->close (u_long (this->status_));
}

ACE_ALLOC_HOOK_DEFINE(ACE_Task)

template <ACE_SYNCH_1> INLINE void
ACE_Task<ACE_SYNCH_2>::dump (void) const
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::dump");
  ACE_DEBUG ((LM_INFO, "flags = %x\n", this->flags_));
  this->msg_queue_->dump ();
}

// If the user doesn't supply a ACE_Message_Queue pointer then we'll
// allocate one dynamically.  Otherwise, we'll use the one they give.

template<ACE_SYNCH_1>
ACE_Task<ACE_SYNCH_2>::ACE_Task (ACE_Thread_Manager *thr_man, 
				 ACE_Message_Queue<ACE_SYNCH_2> *mq)
  : delete_msg_queue_ (0),
    thr_mgr_ (thr_man),
    mod_ (0), 
    flags_ (0),
    grp_id_ (0),
    is_active_ (0)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::ACE_Task");
  if (mq == 0)
    {
      mq = new ACE_Message_Queue<ACE_SYNCH_2>;
      if (mq == 0)
	{
	  errno = ENOMEM;
	  return;
	}
      this->delete_msg_queue_ = 1;
    }
  this->msg_queue_ = mq;
}

template<ACE_SYNCH_1>
ACE_Task<ACE_SYNCH_2>::~ACE_Task (void)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::~ACE_Task");
  if (this->delete_msg_queue_)
    delete this->msg_queue_;

  // These assignments aren't strickly necessary but they help guard
  // against odd problems...
  this->delete_msg_queue_ = 0;
  this->msg_queue_ = 0;
  this->thr_mgr_ = 0;
  this->mod_ = 0;
}

// Note that this routine often does not return since the thread that
// is executing it will do an ACE_Thread::exit() first!

template<ACE_SYNCH_1> void *
ACE_Task<ACE_SYNCH_2>::svc_run (ACE_Task<ACE_SYNCH_2> *t)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::svc_run");
  ACE_THREAD_EXIT_HOOK_INIT (ACE_SYNCH_2, t);

  void *status = (void *) t->svc ();

  ACE_THREAD_EXIT_HOOK_FINI (status);
  /* NOTREACHED */
}

template<ACE_SYNCH_1> ACE_Task<ACE_SYNCH_2> *
ACE_Task<ACE_SYNCH_2>::sibling (void)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::sibling");
  if (this->mod_ == 0)
    return 0;
  else
    return this->mod_->sibling (this);
}

template<ACE_SYNCH_1> const char *
ACE_Task<ACE_SYNCH_2>::name (void) const
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::name");
  if (this->mod_ == 0)
    return 0;
  else
    return this->mod_->name ();
}

template<ACE_SYNCH_1> ACE_Module<ACE_SYNCH_2> *
ACE_Task<ACE_SYNCH_2>::module (void) const
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::module");
  return this->mod_;
}

// Get the current group id.
template <ACE_SYNCH_1> INLINE int
ACE_Task<ACE_SYNCH_2>::grp_id (void)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::grp_id");
  ACE_MT (ACE_Mutex_Guard mon (this->lock_));
  return this->grp_id_;
}

// Set the current group id.
template <ACE_SYNCH_1> INLINE void
ACE_Task<ACE_SYNCH_2>::grp_id (int id)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::grp_id");
  ACE_MT (ACE_Mutex_Guard mon (this->lock_));
  this->grp_id_ = id;
}

// Suspend a task.
template<ACE_SYNCH_1> int 
ACE_Task<ACE_SYNCH_2>::suspend (void)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::suspend");
  ACE_MT (ACE_Mutex_Guard mon (this->lock_));
  if (this->is_active_)
    return this->thr_mgr_->suspend_grp (this->grp_id_);
  else
    return 0;
}

// Resume a suspended task.
template<ACE_SYNCH_1> int 
ACE_Task<ACE_SYNCH_2>::resume (void)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::resume");
  ACE_MT (ACE_Mutex_Guard mon (this->lock_));
  if (this->is_active_)
    return this->thr_mgr_->resume_grp (this->grp_id_);
  else
    return 0;
}

template<ACE_SYNCH_1> int
ACE_Task<ACE_SYNCH_2>::activate (long flags, 
				 int n_threads, 
				 int force_active)
{
  ACE_TRACE ("ACE_Task<ACE_SYNCH_2>::activate");
#if defined (ACE_MT_SAFE)
  ACE_Mutex_Guard mon (this->lock_);
  
  if (this->is_active_ && force_active == 0)
    return 1; // Already active.
  else
    this->is_active_ = 1;

  // Use the ACE_Thread_Manager singleton if we're running as an
  // active object.
  if (this->thr_mgr_ == 0)   
    this->thr_mgr_ = ACE_Service_Config::thr_mgr ();

  this->grp_id_ = this->thr_mgr_->spawn_n (n_threads, 
					   ACE_THR_FUNC (&ACE_Task<ACE_SYNCH_2>::svc_run),
					   (void *) this, flags);
  if (this->grp_id_ == -1)
    return -1;
  else
    return 0;
#else
  {
    n_threads = n_threads;
    force_active = force_active;
    flags = flags; // Keep the compiler from complaining.
    errno = EINVAL;
    return -1;
  }
#endif /* ACE_MT_SAFE */
}

#endif /* ACE_TASK_C */
