
#include "ace/Token_Collection.h"
#include "ace/Log_Msg.h"

#if !defined (__INLINE__)
#include "ace/Token_Collection.i"
#endif /* __INLINE__ */

ACE_Token_Collection::ACE_Token_Collection (int debug)
: ACE_Token_Proxy ((ACE_Token_Proxy::PROXY_TYPE) 0,
		   (ACE_Tokens::TOKEN_TYPE) 0),
  debug_ (debug)
{
  ACE_TRACE ("ACE_Token_Collection::ACE_Token_Collection");
}

int
ACE_Token_Collection::insert (ACE_Token_Proxy &new_token)
{
  ACE_TRACE ("ACE_Token_Collection::insert");

  TOKEN_NAME name (new_token.name(), new_token.type ());

  // check if the new_proxy is already in the list.
  if (collection_.find (name) == 1)
    // One already exists!  Fail
    return -1;

  // clone the new token
  ACE_Token_Proxy *temp = new_token.clone();

  if (collection_.bind (name, temp) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "bind failed\n"), -1);
  return 0;
}

int
ACE_Token_Collection::extract (const TOKEN_NAME &token_name, ACE_Token_Proxy *&proxy)
{
  ACE_TRACE ("ACE_Token_Collection::extract");
  return collection_.unbind (token_name, proxy);
}

ACE_Token_Proxy *
ACE_Token_Collection::is_member (const TOKEN_NAME &token_name)
{
  ACE_TRACE ("ACE_Token_Collection::is_member");
  ACE_Token_Proxy *temp;
  // get the token from the collection
  return collection_.find (token_name, temp) == -1 ? 0 : temp;
}

int
ACE_Token_Collection::is_member (const ACE_Token_Proxy &token)
{
  ACE_TRACE ("ACE_Token_Collection::is_member");
  return collection_.find (TOKEN_NAME (token.name(), token.type())) == 0;
}

int 
ACE_Token_Collection::acquire (int notify,
			       void (*sleep_hook)(void *),
			       ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Collection::acquire");
  return 0;
}

int 
ACE_Token_Collection::acquire (const TOKEN_NAME &token_name,
			       int notify,
			       void (*sleep_hook)(void *),
			       ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Collection::acquire");
  ACE_Token_Proxy *temp;
  // get the token from the collection
  int result = collection_.find (token_name, temp);
  // did we find it?
  if (result == -1)
    return result;
  // perform the operation
  return temp->acquire (notify, sleep_hook, options);
}

int 
ACE_Token_Collection::renew (int requeue_position,
			     ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Collection::renew");
  return 0;
}

int 
ACE_Token_Collection::renew (const TOKEN_NAME &token_name,
			     int requeue_position,
			     ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Collection::renew");
  ACE_Token_Proxy *temp;
  // get the token from the collection
  int result = collection_.find (token_name, temp);
  // did we find it?
  if (result == -1)
    {
      ACE_ERROR_RETURN ((LM_DEBUG, "%p %s\n",
			 "not in collection ",
			 token_name), -1);
      return result;
    }
  // perform the operation
  return temp->renew (requeue_position, options);
}

int 
ACE_Token_Collection::release (ACE_Synch_Options &options)

{
  ACE_TRACE ("ACE_Token_Collection::release");
  COLLECTION_ITERATOR iterator (collection_);

  for (COLLECTION_ENTRY *temp = 0;
       iterator.next (temp) != 0;
       iterator.advance ())
    {
      if (debug_)
	ACE_DEBUG ((LM_DEBUG, "collection releasing %s\n",
		    temp->int_id_->name ()));
      temp->int_id_->release ();
    }

  return 0;
}

int 
ACE_Token_Collection::release (const TOKEN_NAME &token_name,
			       ACE_Synch_Options &options)
{
  ACE_TRACE ("ACE_Token_Collection::release");
  ACE_Token_Proxy *temp;
  // get the token from the collection
  int result = collection_.find (token_name, temp);
  // did we find it?
  if (result != 0)
    return result;
  // perform the operation
  return temp->release (options);
}

ACE_Token_Collection::~ACE_Token_Collection ()
{
  ACE_TRACE ("ACE_Token_Collection::~ACE_Token_Collection");
  COLLECTION_ITERATOR iterator (collection_);

  for (COLLECTION_ENTRY *temp = 0;
       iterator.next (temp) != 0;
       iterator.advance ())
    {
      delete temp->int_id_;
      // The ext_id_'s delete themselves when the array of
      // COLLECTION_ENTRYs goes away.
    }
}


ACE_Token_Proxy *
ACE_Token_Collection::clone ()
{
  ACE_TRACE ("ACE_Token_Collection::clone");
  return (ACE_Token_Proxy *) 0;
}

int 
ACE_Token_Collection::tryacquire (void (*sleep_hook)(void *))
{
  ACE_TRACE ("ACE_Token_Collection::tryacquire");
  return 0;
}


#if defined (ACE_TEMPLATES_REQUIRE_SPECIALIZATION)
 template class ACE_Token_Name <ACE_Token_Proxy::PROXY_TYPE>;
 template class ACE_Map_Manager <TOKEN_NAME, ACE_Token_Proxy *, ACE_Null_Mutex>;
 template ACE_Map_Iterator <TOKEN_NAME, ACE_Token_Proxy *, ACE_Null_Mutex>;
 template ACE_Map_Entry <TOKEN_NAME, ACE_Token_Proxy *>;
#endif /* ACE_TEMPLATES_REQUIRE_SPECIALIZATION */
