/* Defines the member functions for the base class of the ACE_UPIPE
   abstraction. */ 

#include "ace/UPIPE_Stream.h"

#if defined (ACE_HAS_THREADS)

#if !defined (__INLINE__) 
#include "ace/UPIPE_Stream.i"
#endif /* __INLINE__ */


ACE_ALLOC_HOOK_DEFINE(ACE_UPIPE_Stream)

void
ACE_UPIPE_Stream::dump (void) const
{
  ACE_TRACE ("ACE_UPIPE_Stream::dump");
}

int 
ACE_UPIPE_Stream::close (void)
{
  ACE_TRACE ("ACE_UPIPE_Stream::close");
  // Since the UPIPE should have been closed earlier we won't bother
  // checking to see if closing it now fails.
  if (this->ACE_SPIPE::get_handle () != ACE_INVALID_HANDLE)
    this->ACE_SPIPE::close ();
  return this->MT_Stream::close (0);
}

int 
ACE_UPIPE_Stream::get_remote_addr (ACE_UPIPE_Addr &remote_sap) const
{
  ACE_TRACE ("ACE_UPIPE_Stream::get_remote_addr");
  remote_sap = this->remote_addr_;
  return 0;
}

int 
ACE_UPIPE_Stream::send_msg (ACE_Message_Block *mb_p, 
			    ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_UPIPE_Stream::send_msg");
  return this->MT_Stream::put (mb_p, timeout);
}

int ACE_UPIPE_Stream::recv_msg (ACE_Message_Block *& mb_p, 
				ACE_Time_Value *timeout)
{
  return this->MT_Stream::get (mb_p, timeout);
}

// send a buffer

int 
ACE_UPIPE_Stream::send (const char *buffer, 
			size_t siz, 
			ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_UPIPE_Stream::send");
  ACE_Message_Block *mb_p = new ACE_Message_Block (siz);

  if (mb_p == 0)
    return -1;     // ENOMEM

  ACE_OS::memcpy (mb_p->rd_ptr (), buffer, siz);
  return this->MT_Stream::put (mb_p, timeout);
}

// receive a buffer
int 
ACE_UPIPE_Stream::recv (char *buffer, 
			size_t siz, 
			ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_UPIPE_Stream::recv");
  int bufptr = 0;   // index in buffer

  while (bufptr < siz)
  {
    if (mb_last != 0)
    {
      // we have remaining data in our last read Message_Buffer

      if (remaining < siz)
      {
        // the remaining data is not enough

        ACE_OS::memcpy (&buffer[bufptr], mb_last->rd_ptr (), remaining);
        bufptr += remaining;
        remaining = 0;
        delete mb_last;
        mb_last = 0;
      }
      else
      {
        // the remaining data is enough

        ACE_OS::memcpy (&buffer[bufptr], mb_last->rd_ptr (), siz);

        bufptr += siz;
        mb_last->rd_ptr (mb_last->rd_ptr () + siz);  // advance rd_ptr
        remaining -= siz;
        if (remaining == 0)
        {
          // now the Message_Buffer is empty

          delete mb_last;
          mb_last = 0;
        }
      }
    }
    else
    {
      // We have to get a new Message_Buffer from our stream

      int error = this->MT_Stream::get (mb_last, timeout);
      if (error != 0)
        return error;
      remaining = mb_last->size ();
    }
  }
  return 0;
}

int 
ACE_UPIPE_Stream::send_n (const char *buf, 
			  size_t len, 
			  ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_UPIPE_Stream::send_n");
  size_t bytes_written;
  int	 n;

  for (bytes_written = 0; bytes_written < len; bytes_written += n)
    if ((n = this->send (buf + bytes_written, len - bytes_written, timeout)) == -1)
      return -1;

  return bytes_written;
}

int 
ACE_UPIPE_Stream::recv_n (char *buf, 
			  size_t len, 
			  ACE_Time_Value *timeout)
{
  ACE_TRACE ("ACE_UPIPE_Stream::recv_n");
  size_t bytes_read;
  int	 n;

  for (bytes_read = 0; bytes_read < len; bytes_read += n)
    if ((n = this->recv (buf + bytes_read, len - bytes_read, timeout)) == -1)
      return -1;
    else if (n == 0)
      break;

  return bytes_read;      
}

#endif /* ACE_HAS_THREADS */
  
