/* -*- C++ -*- */

#if !defined (_CONFIG_FILES)
#define _CONFIG_FILES

#include "ace/OS.h"

class FP 
// = TITLE
//     This class serves as a namespace for the Return_Type
{
public:
  enum Return_Type { EOFILE = EOF, SUCCESS, EOLINE = '\n', COMMENT, 
		     ERROR = 1234, BUFSIZE = BUFSIZ };
};

template <class ENTRY>
class File_Parser
// = TITLE
//     Class used to parse the configuration file for the routing 
//     table.  
{
public:
  // = Open and Close the file specified
  int open (const char filename[]);
  int close (void);

  virtual FP::Return_Type read_entry (ENTRY &, int &line_number) = 0;
  // Implementations use protected methods to fill in the entry.

protected:
  FP::Return_Type getword (char buf[]);
  // Read the next ASCII word.

  FP::Return_Type getint (int &value);
  // Read the next integer.

  FP::Return_Type readword (char buf[]);
  int delimiter (char ch);
  int endofline (char ch);
  int comments (char ch);
  int skipline (void);

  FILE *infile_;
};

class CC_Config_File_Entry
{
public:
  int     conn_id_;
  char    host_[FP::BUFSIZE];
  u_short remote_port_;
  char    direction_;
  int     max_retry_delay_;
  u_short local_port_;
};

class CC_Config_File_Parser : public File_Parser<CC_Config_File_Entry>
{
public:
  virtual FP::Return_Type
    read_entry (CC_Config_File_Entry &entry, int &line_number);
};

class RT_Config_File_Entry
{
public:
  enum {MAX_DESTINATIONS = 100};
  int conn_id_;
  int logical_id_;
  int payload_type_;
  int destinations_[MAX_DESTINATIONS];
  int total_destinations_;
};  

class RT_Config_File_Parser : public File_Parser<RT_Config_File_Entry>
{
public:
  virtual FP::Return_Type
    read_entry (RT_Config_File_Entry &entry, int &line_number);
};

class Map_Config_File_Entry
{
public:
  int logical_id_;
  int routing_id_;
};

class Map_Config_File_Parser : public File_Parser<Map_Config_File_Entry>
{
public:
  virtual FP::Return_Type read_entry (Map_Config_File_Entry &entry, 
				      int &line_number);
};

#endif /* _CONFIG_FILES */
