/* -*- C++ -*- */
// The Gateway class integrates everything else together.

#if !defined (_GATEWAY)
#define _GATEWAY

//#include "Channel_Acceptor.h"
#include "Channel_Connector.h"

template <class INPUT_CHANNEL, class OUTPUT_CHANNEL>
class Gateway : public ACE_Service_Object
{
public:
  Gateway (ACE_Thread_Manager * = 0);

  virtual int init (int argc, char *argv[]);
  // Perform initialization.

  virtual int fini (void);
  // Perform termination.

protected:
  int handle_input (ACE_HANDLE);

  int handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t * = 0, ucontext_t * = 0
#endif /* ACE_HAS_SIGINFO_T */
    );

  typedef ACE_Map_Manager<CONN_ID, Channel *, MUTEX> CONFIG_TABLE;
  typedef ACE_Map_Iterator<CONN_ID, Channel *, MUTEX> CONFIG_ITERATOR;

  CONFIG_TABLE config_table_;
  // Table that maps Connection IDs to Channel *'s.

  ROUTING_TABLE routing_table_;
  // Table that maps Peer addresses to a set of Channel *'s for output.

  virtual int info (char **, size_t) const;
  // Return info about this service.

  int parse_args (int argc, char *argv[]); 
  // Parse gateway configuration arguments obtained from svc.conf file.

  int parse_cc_config_file (void);
  // Parse the channel connection configuration file.

  int parse_rt_config_file (void);
  // Parse the routing table configuration file.

  int initiate_connections (void);
  // Initiate connections to the peers.

  virtual int handle_timeout (const ACE_Time_Value &, const void *arg);
  // Perform timer-based performance profiling.

  const char *cc_config_file_;
  // Name of the channel connection configuration file.

  const char *rt_config_file_;
  // Name of the routing table configuration file.

  int performance_window_;
  // Number of seconds after connection establishment to report throughput.
  
  int blocking_semantics_;
  // 0 == blocking connects, ACE_NONBLOCK == non-blocking connects.

  int debug_;
  // Are we debugging?

  Channel_Connector *connector_;
  // This is used to establish the connections actively.

  int socket_queue_size_;
  // Size of the socket queue (0 means "use default").

  // = Manage output and input channel threads (if used.)
  // if both input and output mt is used, they will share thr_mgr_,
  // thr_mgr_ will always reference the thread manager being used
  // regardless of whether input, output, or both channels are using mt.
  ACE_Thread_Manager *thr_mgr_;
  ACE_Thread_Manager *input_thr_mgr_;
  ACE_Thread_Manager *output_thr_mgr_;
};

// Factory function that allocates a new Gateway daemon.
extern "C" ACE_Service_Object *_alloc_gatewayd (void);

#endif /* _GATEWAY */
