/* -*- C++ -*- */
// Define the Peer message schema (this may change).

#if !defined (PEER_MESSAGE)
#define PEER_MESSAGE

// This is the unique connection identifier that denotes a particular Channel.
typedef short CONN_ID;

// Peer address is used to identify the source/destination of an ISRP message.

class Peer_Addr
{
public:
  Peer_Addr (CONN_ID cid = -1, u_char lid = 0, u_char pay = 0)
    : conn_id_ (cid), logical_id_ (lid), payload_ (pay) {}

  int operator== (const Peer_Addr &pa) const
  {
    return this->conn_id_ == pa.conn_id_ 
      && this->logical_id_ == pa.logical_id_
      && this->payload_ == pa.payload_;
  }

  CONN_ID conn_id_;
  // Unique connection identifier that denotes a particular Channel.

  u_char logical_id_;
  // Logical ID.

  u_char payload_;
  // Payload type.
};

// Fixed sized header.

class Peer_Header
{
public:
  typedef u_short ROUTING_ID;
  // Type used to route messages from gatewayd.

  enum
  {
    INVALID_ID = -1 // No peer may use this number.
  };

  ROUTING_ID routing_id_;
  // Source ID.

  size_t  len_;
  // Length of the message in bytes.
};

// Variable-sized message (buf_ may be variable-sized between
// 0 and MAX_PAYLOAD_SIZE).

class Peer_Message
{
public:
  enum { MAX_PAYLOAD_SIZE = 1024 };
  // The maximum size of an Peer message (see Peer protocol specs for exact #).

  Peer_Header header_;

  char buf_[MAX_PAYLOAD_SIZE];
  // Message payload
};

#endif /* PEER_MESSAGE */
