/* -*- C++ -*- */

#include "ace/INET_Addr.h"
#include "ace/SOCK_Connector.h"
#include "ace/Log_Msg.h"
#include "ace/Signal.h"

INLINE void
Client_Logger::stderr_output (int)
{
  Client_Logger::logger_output_.set_handle (fileno (stderr));
}

INLINE int
Client_Logger::remove (void)
{
  return 0;
}

/* Receive a PDU from the Message ACE_Queue.  Returns either -1 if
   something went wrong, or the number of bytes in the PDU (don't
   forget that we need to add the TYPE field back into the count,
   since msgrcv doesn't do this automatically!). */

INLINE int
Client_Logger::recv (void)
{
  ACE_Str_Buf recv_msg ((void *) &this->log_record_, 
			0, sizeof this->log_record_);

  return this->message_queue_.recv (recv_msg);
}

/* Encodes the contents of log_record object using network byte-order
   (if we are writing to the remote logger daemon).  Note that in
   either case there is no need to encode the msg_data field (since it
   is an ASCII representation of the logging message). */

INLINE int  
Client_Logger::send (void)
{
  if (Client_Logger::logger_output_.get_handle () == fileno (stderr))
    this->log_record_.print ("<localhost>", 0, stderr);
  else
    {
      long len = this->log_record_.length ();

      this->log_record_.encode ();

      if (Client_Logger::logger_output_.send (4, &len, sizeof len,
					      (char *) &this->log_record_, 
					      this->log_record_.length ()) == -1)
	{
	  Client_Logger::stderr_output ();
	  return -1;
	}
    }
  return 0;
}

/* Try to open the remote logging daemon and create the message queue.
   If we fail, we'll write to stderr. */

INLINE int  
Client_Logger::open (u_short remote_port, 
		     const char *remote_host, 
		     char *logger_key)
{
  /* Note, if we put a trace call here, we will lose! */
  ACE_Sig_Action sig ((ACE_SignalHandler) Client_Logger::stderr_output, SIGPIPE);

  ACE_SOCK_Connector con;

  if (con.connect (Client_Logger::logger_output_, 
		   ACE_INET_Addr (remote_port, remote_host)) == -1)
    Client_Logger::stderr_output ();	  

  if (ACE_OS::unlink (logger_key) == -1 && errno == EACCES)
    return -1;
  else
    return this->message_queue_.open (logger_key);
}

INLINE
Client_Logger::Client_Logger (void)
{
}

INLINE
Client_Logger::Client_Logger (u_short remote_port, 
			      const char *remote_host, 
			      char *logger_key)
{
  if (this->open (remote_port, remote_host, logger_key) == -1)
    ACE_ERROR ((LM_ERROR, "%p: remote host = %s, logger key = %s\n", 
	       "open failed", remote_host, logger_key));
}
