/* The program collects logging information from various subsystems
   and either writes it to the standard error stream or forwards it
   to a network logging daemon. */

#include "ace/Log_Msg.h"
#include "ace/SOCK_Stream.h"
#include "ace/INET_Addr.h"
#include "ace/Signal.h"
#include "Client_Logger.h"

char *logger_key;

static void
cleanup (int)
{
  ACE_OS::unlink (logger_key);
  ACE_OS::exit (1);
}

int
main (int argc, char *argv[])
{
  Client_Logger	 client_logger;
  unsigned short remote_port  = argc < 2 ? ACE_DEFAULT_SERVER_PORT : ACE_OS::atoi (argv[1]);
  const char	 *remote_host = argc < 3 ? ACE_DEFAULT_SERVER_HOST : argv[2];
                 logger_key   = argc < 4 ? ACE_DEFAULT_RENDEZVOUS : argv[3];

  ACE_Sig_Action sig ((ACE_SignalHandler) cleanup, SIGINT);

  if (client_logger.open (remote_port, remote_host, logger_key) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p: remote host = %s, logger key = %s\n", 
		      "open failed", remote_host, logger_key), 1);

  while (client_logger.recv () != -1)
    client_logger.send ();

  /* NOTREACHED */
  return 0;
}
