#include "ace/Log_Msg.h"
#include "ace/Reactor.h"
#include "Client_Handler.h"

#if !defined (__INLINE__)
#include "Client_Handler.i"
#endif /* __INLINE__ */

/* Default constructor */

Client_Handler::Client_Handler (void)
{
}

Client_Handler::~Client_Handler (void)
{
  this->cli_stream_.close ();
}

// Extract the underlying ACE_SOCK_Stream (e.g., for purposes of
// accept()).

Client_Handler::operator ACE_SOCK_Stream &()
{
  return this->cli_stream_;
}

// Perform the logging record receive. 

int
Client_Handler::handle_input (int)
{
  ssize_t n; 
  size_t  len;

  // Perform two recv's to emulate record-oriented semantics.  Note
  // that this code is not entirely portable since it relies on the
  // fact that sizeof (ssize_t) is the same on both the sender and
  // receiver side.  To correctly handle this is painful, and we leave
  // it as an exercise for the reader ;-).

  switch (n = this->cli_stream_.recv ((void *) &len, sizeof len))
    {
    case -1:
      ACE_ERROR_RETURN ((LM_ERROR, "%p at host %s\n",
			"client logger", this->host_name_), -1);      
      /* NOTREACHED */
    case 0:
      ACE_ERROR_RETURN ((LM_ERROR, "closing log daemon at host %s (fd = %d)\n",
			this->host_name_, this->get_handle ()), -1);
      /* NOTREACHED */
    case sizeof (size_t):
      {
	ACE_Log_Record lp;

        len = ntohl (len);
	if ((n = this->cli_stream_.recv_n ((void *) &lp, len)) != len)
	  ACE_ERROR_RETURN ((LM_ERROR, "%p at host %s\n",
			    "client logger", this->host_name_), -1);
	/* NOTREACHED */
	  
	lp.decode ();

	if (lp.length () == n)
	  lp.print (this->host_name_, 0, stderr);
	else
	  ACE_ERROR ((LM_ERROR, "error, lp.length = %d, n = %d\n",
		     lp.length (), n));
	break;
      }
    default:
      ACE_ERROR_RETURN ((LM_ERROR, "%p at host %s\n",
			"client logger", this->host_name_), -1);
      /* NOTREACHED */
    }

  return 0;
}

/* Extract underlying device descriptor */

int
Client_Handler::get_handle (void) const
{
  return this->cli_stream_.get_handle ();
}

int
Client_Handler::open (void *reactor)
{
  ACE_INET_Addr addr;
  
  if (this->cli_stream_.get_remote_addr (addr) == -1)
    return -1;
  else
    {
      ACE_OS::strncpy (this->host_name_, addr.get_host_name (), MAXHOSTNAMELEN + 1);
      if (((ACE_Reactor *) reactor)->register_handler (this, ACE_Event_Handler::READ_MASK) == -1)
	ACE_ERROR_RETURN ((LM_ERROR, "can't register with reactor\n"), -1);
      else
      	ACE_DEBUG ((LM_DEBUG, "connected with %s\n", this->host_name_));
      return 0;
    }
}

// Perform termination activities when deregistered from the ACE_Reactor.

int
Client_Handler::handle_close (int, ACE_Reactor_Mask)
{
  /* Must be allocated dynamically! */
  delete this;		
  return 0;
}

/* Perform termination activities when close fails. */

int
Client_Handler::close (void)
{
  return this->handle_close (-1, ACE_Event_Handler::RWE_MASK);
}
