/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ace
// 
// = FILENAME
//   Server_Logger.h 
//
// = DESCRIPTION
//     Defines the classes that perform server logging daemon
//     functionality.
//
// = AUTHOR
//    Doug Schmidt 
// 
// ============================================================================

#if !defined (ACE_SERVER_LOGGER_H)
#define ACE_SERVER_LOGGER_H

#include "ace/Synch.h"
#include "ace/Svc_Handler.h" 
#include "ace/Acceptor.h"
#include "ace/TLI_Acceptor.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/INET_Addr.h"

template <ACE_PEER_STREAM_1, class COUNTER, ACE_SYNCH_1>
class Logging_Handler : public ACE_Svc_Handler<ACE_PEER_STREAM_2, ACE_SYNCH_2>
{
  // = TITLE
  //   Receives, frames, and processes logging records.
public:
  Logging_Handler (ACE_Thread_Manager * = 0);

  virtual int handle_input (ACE_HANDLE = -1);
  // Process remote logging records. 

protected:
  int handle_logging_record (void);
  // Receive the logging record from a client.
  
#if !defined (ACE_LACKS_STATIC_DATA_MEMBER_TEMPLATES)
  static COUNTER request_count_;
  // Count the number of logging records that arrive.
#endif /* ACE_LACKS_STATIC_DATA_MEMBER_TEMPLATES */

  char host_name_[MAXHOSTNAMELEN + 1];
  // Name of the host we are connected to.

private:
  // Don't need this method here...
  virtual int put (ACE_Message_Block *, ACE_Time_Value *); 
};

#if !defined (ACE_HAS_TLI)
#define LOGGER_PEER_STREAM ACE_SOCK_STREAM
#define LOGGER_PEER_ACCEPTOR ACE_SOCK_ACCEPTOR
#else
#define LOGGER_PEER_STREAM ACE_TLI_STREAM
#define LOGGER_PEER_ACCEPTOR ACE_TLI_ACCEPTOR
#endif

// = Instantiate a Logging_Handler with single-threaded synchronization.

typedef Logging_Handler <LOGGER_PEER_STREAM, u_long, ACE_NULL_SYNCH>
        LOGGING_Handler;

class Logging_Acceptor : public ACE_Acceptor<LOGGING_Handler, LOGGER_PEER_ACCEPTOR>
  // = TITLE
  //    Accepts new connections from client logging daemons
  //    and registers them with the ACE_Reactor.
{
public:
  Logging_Acceptor (void);

  int info (char **strp, size_t length) const;
  // Returns information on the currently active service.

  int init (int argc, char *argv[]);
  // Initialize the acceptor when it's linked dynamically.

  int handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
    );
};

#if defined (__INLINE__)
#define INLINE inline
#include "Server_Logger.i"
#else
#define INLINE
#endif /* __INLINE__ */

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "Server_Logger.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */
#endif /* ACE_SERVER_LOGGER_H */
