/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ace
// 
// = FILENAME
//    Thr_Server_Logger.h 
//
// = DESCRIPTION
//   Defines the classes that perform server logging daemon functionality.
//
// = AUTHOR
//    Doug Schmidt 
// 
// ============================================================================

#if !defined (ACE_THR_SERVER_LOGGER_H)
#define ACE_THR_SERVER_LOGGER_H

#include "ace/Synch.h"
#include "Server_Logger.h"

#if !defined (ACE_HAS_THREADS) 
typedef u_long COUNTER;
#define SYNCH ACE_NULL_SYNCH 
#else
typedef ACE_Atomic_Op <ACE_Mutex, u_long> COUNTER;
#define SYNCH ACE_MT_SYNCH 
#endif /* ACE_HAS_THREADS */

typedef Logging_Handler<LOGGER_PEER_STREAM, COUNTER, SYNCH> LOGGING_HANDLER;

class Thr_Logging_Handler : public LOGGING_HANDLER
  // = TITLE
  //     Threaded version of the server logger.  
  //
  // = DESCRIPTION
  //     Each client is handled in its own separate thread.
{
public:
  Thr_Logging_Handler (ACE_Thread_Manager * = 0);

  virtual int open (void * = 0);
  // Override activation definition in the ACE_Svc_Handler class (will
  // spawn a new thread if we've got threads).

  virtual int close (u_long = 0);
  // Override close () definition from the ACE_Svc_Handler class.

  virtual int svc (void);
  // Process remote logging records. 
};

class Thr_Logging_Acceptor : public ACE_Acceptor<Thr_Logging_Handler, LOGGER_PEER_ACCEPTOR>
  // = TITLE
  //     A version of the Logging_Acceptor that creates multi-threaded handlers.
{
public:
  Thr_Logging_Acceptor (void);

  virtual int info (char **, size_t) const;
  // Returns information on the currently active service.

  int init (int argc, char *argv[]);
  // Initialize the acceptor when it's linked dynamically.

  int handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
    );
};

#endif /* ACE_THR_SERVER_LOGGER_H */
