/* -*- C++ -*- */
/* Defines the classes that handle the server logging daemon. */

#include "ace/Log_Record.h"
#include "ace/Log_Msg.h"
#include "ace/Get_Opt.h"

// No-op...

INLINE 
Thr_Logging_Handler::Thr_Logging_Handler (ACE_Thread_Manager *)
{
}

// Close up shop, delete this, and bail out.

INLINE int
Thr_Logging_Handler::close (u_long)
{
  delete this;
  return 0;
}

// Override definition in the ACE_Svc_Handler class (spawn a new
// thread if we're configured with ACE_HAS_THREADS!).

INLINE int
Thr_Logging_Handler::open (void *)
{
  // Shut off non-blocking IO!
  if (this->peer ().disable (ACE_NONBLOCK) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "disable"), -1);

  // Spawn a new thread of control to handle logging records with the client.
  if (this->activate (THR_BOUND | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "spawn"), -1);    
  return 0;
}

// Process remote logging records. 

INLINE int
Thr_Logging_Handler::svc (void)
{
  int result = 0;

  // Loop until the client terminates the connection or an error occurs.

  while ((result = this->handle_input ()) > 0)
    continue;

  return result;
}

INLINE int
Thr_Logging_Acceptor::init (int argc, char *argv[])
{
  ACE_Get_Opt get_opt (argc, argv, "dp:", 0);
  ACE_INET_Addr addr;

  for (int c; (c = get_opt ()) != -1; )
     switch (c)
       {
       case 'p': 
	 addr.set (ACE_OS::atoi (get_opt.optarg));
	 break;
       case 'd':
	 break;
       default:
	 break;
       }
  
  if (this->open (addr, ACE_Service_Config::reactor ()) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "open"), -1);
  return 0;
}

/* Returns information on the currently active service */

INLINE int
Thr_Logging_Acceptor::info (char **strp, size_t length) const
{
  char buf[BUFSIZ];
  char addr[BUFSIZ];
  ACE_INET_Addr local_addr;

  if (this->acceptor ().get_local_addr (local_addr) == -1)
    return -1;
  else 
    local_addr.addr_to_string (addr, sizeof addr);

  ACE_OS::sprintf (buf, "%s\t %s/%s %s", 
	     "Thr_Logger", addr, "tcp", 
	     "# multi-threaded server logger\n");

  if (*strp == 0 && (*strp = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*strp, buf, length);
  return ACE_OS::strlen (buf);
}

int
Thr_Logging_Acceptor::handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
)
{
  ACE_DEBUG ((LM_DEBUG, "%S\n", signum));

  // Shut down the main event loop.
  ACE_Service_Config::end_event_loop ();
  return 0;
}

INLINE 
Thr_Logging_Acceptor::Thr_Logging_Acceptor (void)
{
  // Register ourselves to receive SIGINT so that
  // we shut down gracefully when user signals us.

  if (ACE_Service_Config::reactor ()->register_handler (SIGINT, this) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "register_handler"));
}

