/* This program sends logging records directly to the server, rather
   than going through the client logging daemon. */

#include "ace/INET_Addr.h"
#include "ace/SOCK_Stream.h"
#include "ace/Log_Msg.h"
#include "ace/Log_Record.h"

static u_short LOGGER_PORT = ACE_DEFAULT_SERVER_PORT;
static const char *const LOGGER_HOST = ACE_DEFAULT_SERVER_HOST;
static const char *const DATA = "hello world\n";

INLINE int
round_up (size_t len)
{
  const int ALIGN = 8;

  return (len + ALIGN - 1) & ~(ALIGN - 1);
}

int 
main (int argc, char *argv[])
{
  ACE_SOCK_Stream logger;
  ACE_Log_Record log_Record (LM_DEBUG, ACE_OS::time ((time_t *) 0), ACE_OS::getpid ());
  u_short logger_port  = argc > 1 ? atoi (argv[1]) : LOGGER_PORT;
  const char *logger_host = argc > 2 ? argv[2] : LOGGER_HOST;
  const int length = round_up ((sizeof log_Record - ACE_Log_Record::MAXLOGMSGLEN) + strlen (DATA));

  if (logger.open (ACE_INET_Addr (logger_port, logger_host)) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "open"), -1);
  
  ACE_OS::strcpy (log_Record.msg_data, DATA);

  log_Record.encode ();

  if (logger.send (&log_Record, length) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "send"), -1);
  else if (logger.close () == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "close"), -1);
  return 0;
}
