#include "ace/Name_Options.h"
#include "ClientTest.h"

ClientTest::ClientTest (void)
{
  ns_context_ = new ACE_Naming_Context;
}

ClientTest::~ClientTest (void)
{
  // Deregister this handler with the ACE_Reactor.
  this->reactor_->remove_handler (this, ACE_Event_Handler::DONT_CALL
				  | ACE_Event_Handler::READ_MASK);
}

int 
ClientTest::init (int argc, char *argv[])
{
  this->listopt ();

  //determine name context
  if (ACE_OS::strcmp (ACE_Name_Options::instance ()->nameserver_host (), "localhost") == 0) 
    {
      if (ns_context_->open (ACE_Naming_Context::PROC_LOCAL) == -1)
	ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "ns_context_->open"), -1);
    }
  else
    {
      // Don't really need to do this but it's a hack to fix
      // the problme of Display () not printing the right hostname
      ACE_OS::strcpy (this->hostname_, 
		ACE_Name_Options::instance ()->nameserver_host ());
      this->port_ = ACE_Name_Options::instance ()->nameserver_port ();

      if (this->ns_context_->open (ACE_Naming_Context::NET_LOCAL) == -1)
	ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "ns_context_->open"), -1);
    }
  return 0;
}


int
ClientTest::open (ACE_Reactor* reactor)
{
  this->reactor_ = reactor;
  this->DisplayMenu ();
  return this->reactor_->register_handler (this->get_handle (), this, 
					   ACE_Event_Handler::READ_MASK);
}

int
ClientTest::handle_input (ACE_HANDLE )
{
  char option[BUFSIZ];
  char buf1[BUFSIZ];
  char buf2[BUFSIZ];
  char buf3[BUFSIZ];
  char *temp_buf;
  int port;
  char input[256];

  if (::scanf ("%s", option) <= 0)
    {
      cerr << "try again" << endl;
      return 0;
    }
  
  int result = -1;
  switch (option[0])
    {
    case 'P' :
    case 'p' :
      result = set_proc_local ();
      break;
    case 'N' :
    case 'n' :
      result = set_node_local ();
      break;
    case 'H' :
    case 'h' :
      if (::scanf ("%s %d", buf1, &port) <= 0)
	break;
      result = set_host (buf1, port);
      break;
    case 'B' :
    case 'b' :
      // get the input from stdin
      ACE_OS::gets (input);

      // get the key
      if (temp_buf = ACE_OS::strtok (input, " "))
	{
	  ACE_OS::strcpy (buf1, temp_buf);

	  // get the value
	  if (temp_buf = ACE_OS::strtok (0, " "))
	    {
	      ACE_OS::strcpy (buf2, temp_buf);

	      // get the type (if entered)
	      if (temp_buf = ACE_OS::strtok (0, " "))
		{
		  ACE_OS::strcpy (buf3, temp_buf);
		  result = bind (buf1, buf2, buf3);
		}
	      else
	        result = bind (buf1, buf2);
	    }
	  else
	    cerr << "Bind Failed! Value not entered." << endl;
	}
      else
        cerr << "Bind Failed! Key and value not entered." << endl;
      break;
    case 'U' :
    case 'u' :
      if (::scanf ("%s", buf1) <= 0)
	break;
      result = unbind (buf1);
      break;
    case 'R' :
    case 'r' :
      // get the input from stdin
      ACE_OS::gets (input);

      // get the key
      if (temp_buf = ACE_OS::strtok (input, " "))
	{
	  ACE_OS::strcpy (buf1, temp_buf);

	  // get the value
	  if (temp_buf = ACE_OS::strtok (0, " "))
	    {
	      ACE_OS::strcpy (buf2, temp_buf);

	      // get the type (if entered)
	      if (temp_buf = ACE_OS::strtok (0, " "))
		{
		  ACE_OS::strcpy (buf3, temp_buf);
		  result = rebind (buf1, buf2, buf3);
		}
	      else
	        result = rebind (buf1, buf2);
	    }
	  else
	    cerr << "Rebind Failed! Value not entered." << endl;
	}
      else
        cerr << "Rebind Failed! Key and value not entered." << endl;
      break;
    case 'F' :
    case 'f' :
      if (::scanf ("%s", buf1) <= 0)
	break;
      result = find (buf1);
      break;
    case 'Q' :
    case 'q' :
      result = quit ();
      break;
    default :
      cout << "Unrecognized command." << endl;
    }
//  if (result == 0)
//    cout << "Last operation was successful!" << endl;
//  else
//    cout << "Last operation returned: " << result << endl;

  DisplayMenu ();
  return 0;
}

void
ClientTest::DisplayMenu (void)
{
  cout << endl;
  cout << "         Name Service Main Menu" << endl;
  cout << "         ----------------------" << endl;

  // Check if using local name space or remote name space
  if (ACE_OS::strcmp (ACE_Name_Options::instance ()->nameserver_host (), "localhost") == 0)
    {
      if (this->ns_scope_ == ACE_Naming_Context::PROC_LOCAL)
	cout << "  *** Using Process Local Database ***" << endl << endl;
      else
	cout << "  *** Using Node Local Database ***" << endl << endl;
    }
  else
    {
      cout << "  Hostname: " << this->hostname_;
      cout << "  Port Number: " << this->port_ << endl << endl;
    }
  cout << "<P> Use Process Local Database" << endl;
  cout << "<N> Use Node Local Database" << endl;
  cout << "<H> Set Remote Name server <host> and <port>" << endl << endl;
  cout << "<B> Bind <key> <value> [<type>]" << endl;
  cout << "<U> Unbind <key>" << endl;
  cout << "<R> Rebind <key> <value> [<type>]" << endl;
  cout << "<F> Find <key>" << endl;
  cout << "<Q> or ^C (exit) " << endl;
}


int
ClientTest::set_proc_local (void)
{
  // Set Name Options
  ACE_Name_Options::instance ()->nameserver_host ("localhost");
  ACE_Name_Options::instance ()->nameserver_port (0);

  // Set Naming Context scope
  this->ns_scope_ = ACE_Naming_Context::PROC_LOCAL;
  
  // Remove old naming context
  delete this->ns_context_;
  
  // Create new Naming Context
  this->ns_context_ = new ACE_Naming_Context;
  if (this->ns_context_->open (ACE_Naming_Context::PROC_LOCAL) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "ns_context_->open"), -1);
  return 0;
}

int
ClientTest::set_node_local (void)
{
  // Set Name Options
  ACE_Name_Options::instance ()->nameserver_host ("localhost");
  ACE_Name_Options::instance ()->nameserver_port (0);

  // Set Naming Context scope
  this->ns_scope_ = ACE_Naming_Context::NODE_LOCAL;  

  // Remove old naming context
  delete ns_context_;

  // Create new Naming Context
  ns_context_ = new ACE_Naming_Context;
  if (ns_context_->open (ACE_Naming_Context::NODE_LOCAL) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "ns_context_->open"), -1);
  return 0;
}

int 
ClientTest::set_host (char* hostname, int port)
{
  // Set Name Options
  ACE_Name_Options::instance ()->nameserver_host (hostname);
  ACE_Name_Options::instance ()->nameserver_port (port);

  // don't really need to do this but it's a hack to fix
  // the problme of Display () not printing the right hostname
  ACE_OS::strcpy (this->hostname_, hostname);
  this->port_ = port;
  this->ns_scope_ = ACE_Naming_Context::NET_LOCAL;  

  // remove old naming context
  delete ns_context_;

  // Create new Naming Context
  ns_context_ = new ACE_Naming_Context;

  // assume net_local context
  if (ns_context_->open (ACE_Naming_Context::NET_LOCAL) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "ns_context_->open"), -1);
  
  return 0;
}

int 
ClientTest::bind (char* key, char* value, char* type)
{
  if (ns_context_->bind (key, value, type) == -1)
    {
      cerr << "Bind Failed! Key \"" << key << "\" exists." << endl;
      return -1;
    }
  return 0;
}

int 
ClientTest::unbind (char* key)
{
  if (ns_context_->unbind (key) == -1)
    {
      cerr << "Unbind Failed! Key \"" << key << "\" not found." << endl;
      return -1;
    }
  return 0;
}

int 
ClientTest::rebind (char* key, char* value, char* type)
{
  return ns_context_->rebind (key, value, type );
}

int 
ClientTest::find (char* key)
{
  char* value;
  char* type;
  if (ns_context_->resolve (key, value, type) == -1)
    {
      cerr << "Find Failed! Key \"" << key << "\" not found." << endl;
      return -1;
    }
  else
    {
      cerr << "Binding for " << key << ": " << value << endl;
      // cerr << "Binding for " << key << ": " << value << "           Type: " << type << endl;
      delete[] value;
      delete[] type;
      return 0;
    }
}

int 
ClientTest::quit (void)
{
  return ACE_OS::kill ( ACE_OS::getpid (), SIGINT );
}


void 
ClientTest::listopt (void)
{
  cout << "process_name is " << ACE_Name_Options::instance ()->process_name () << endl;
  cout << "namespace_dir is " << ACE_Name_Options::instance ()->namespace_dir () << endl;
}

ACE_HANDLE
ClientTest::get_handle (void) const
{
  return 0;  // stdin
}
