#include "ace/Event_Handler.h"
#include "ace/Reactor.h"
#include "ace/Naming_Context.h"

class ClientTest : public ACE_Event_Handler
{
public:
  ClientTest (void);
  ~ClientTest (void);

  int open (ACE_Reactor* reactor);
  // Cache reactor and then register self with reactor

  virtual int handle_input (ACE_HANDLE fd);
  // Handle user entered commands

  virtual ACE_HANDLE get_handle() const;
  // Return handle

  int init (int argc, char ** argv);
  // Initialize name options and naming context

  void listopt (void);
  // Print name options

private:
  ACE_Naming_Context *ns_context_;
  // Current naming context

  ACE_Naming_Context::Context_Scope_Type ns_scope_;
  // Defines the scope of the naming context

  ACE_Reactor *reactor_;

  char hostname_[MAXHOSTNAMELEN + 1];
  // Cache the hostname and port number for remote case

  int port_;

  void DisplayMenu (void);
  // Display user menu

  int set_proc_local (void);
  // Set options to use PROC_LOCAL naming context

  int set_node_local (void);
  // Set options to use NODE_LOCAL naming context

  int set_host (char* hostname, int port);
  // Set options to use NET_LOCAL naming context 
  // specifying host name and port number

  int bind (char* key, char* value, char* type = "");
  // Bind a key to a value

  int unbind (char* key);
  // Unbind a name binding

  int rebind (char* key, char* value, char* type = "");
  // Rebind a name binding
  
  int find (char* key);
  // Find the value associated with a key

  int quit (void);
  // Gracefully exit
};

