#include "ace/Log_Msg.h"
#include "Token_Handler.h"
#include "Token_Acceptor.h"

// Default "do nothing" constructor.

Token_Acceptor::Token_Acceptor (void)
{
}

// Open up the Token_Acceptor.

int 
Token_Acceptor::open (const ACE_INET_Addr local_addr, ACE_Reactor *reactor)
{
  this->reactor_  = reactor;

  return this->acceptor_.open (local_addr, 1);
}

// Open up the Token_Acceptor.

Token_Acceptor::Token_Acceptor (const ACE_INET_Addr local_addr, ACE_Reactor *reactor)
{
  if (this->open (local_addr, reactor) == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "Token_Acceptor::Token_Acceptor"));
}

// = Demultiplexing hooks.
/* VIRTUAL */ int 
Token_Acceptor::get_handle (void) const
{
  return this->acceptor_.get_handle ();
}

// ACE_Acceptor factory.

/* VIRTUAL */ int 
Token_Acceptor::handle_input (ACE_HANDLE)
{
  // Create a new Token_Handler.
  Token_Handler *token_handler = new Token_Handler;

  if (token_handler == 0)
    {
      errno = ENOMEM;
      return -1;
    }

  // Accept connection into token_handler.
  else if (this->acceptor_.accept (*token_handler) == -1)
    {
      // Close down to avoid descriptor/resource leaks.
      token_handler->close ();
      ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "accept"), -1);
    }

  // Delegate control to token_handler.
  else if (token_handler->open (this) == -1)
    token_handler->close ();
  return 0;
}

/* VIRTUAL */ int 
Token_Acceptor::handle_close (ACE_HANDLE, ACE_Reactor_Mask)
{
  // Check to make sure we don't delete twice.
  if (this->reactor_ != 0)
    {
      ACE_DEBUG ((LM_DEBUG, "shutting down the Token_Acceptor\n"));

      this->reactor_ = 0;
      this->acceptor_.close ();
    }
  return 0;
}

Token_Acceptor::~Token_Acceptor (void)
{
  this->handle_close ();
}
