/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ACE
// 
// = FILENAME
//    Token_Acceptor.h
//
// = DESCRIPTION 
//    Factory that accepts connections from clients and 
//    produces <TokenHandlers>.
//
// = AUTHOR
//    Douglas C. Schmidt (schmidt@cs.wustl.edu)
// 
// ============================================================================

#if !defined (_TOKEN_ACCEPTOR_H)
#define _TOKEN_ACCEPTOR_H

#include "ace/Reactor.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/INET_Addr.h"

class Token_Acceptor : public ACE_Event_Handler
  // = TITLE
  //   Factory that accepts connections from clients 
  //   and produces <TokenHandlers>.
  //
  // = DESCRIPTION
  //   Implements the ACE_Acceptor pattern, which decouples 
  //   active connection establishment from the service
  //   performed once the connection is established.
{
friend class Token_Handler;
public:
  Token_Acceptor (void);
  // Default constructor.

  ~Token_Acceptor (void);
  // Close down the server.

  // = Create a passive-mode endpoint to listen for connection requests from clients.
  Token_Acceptor (const ACE_INET_Addr local_addr, ACE_Reactor *r);
  int open (const ACE_INET_Addr local_addr, ACE_Reactor *r);

  // = Demultiplexing hooks.
  virtual int get_handle (void) const;
  // Return the underlying ACE_HANDLE.

  virtual int handle_input (ACE_HANDLE);
  // Process connection events by creating a new <Token_Handler>

  virtual int handle_close (ACE_HANDLE = -1, ACE_Reactor_Mask = ACE_Event_Handler::RWE_MASK);
  // Operations performed when the <Token_Acceptor> is terminated.

private:
  ACE_SOCK_Acceptor acceptor_;
  // ACE_Acceptor factory used to establish connections passively.

  ACE_Reactor *reactor_;
  // Pointer to <ACE_Reactor>.
};

#endif /* _TOKEN_ACCEPTOR_H */
