#include "ace/Log_Msg.h"
#include "ace/INET_Addr.h"
#include "ace/Get_Opt.h"
#include "Token_Server.h"

Token_Server::Token_Server (int argc, char *argv[])
  : serverPort_ (ACE_DEFAULT_SERVER_PORT), // Default server port.
    done_ (0) // Continue until <SIGINT> occurs.
{
  // Parse the command line arguments and set options.
  this->parse_args (argc, argv);

  // Create the server address.
  this->addr_.set (this->serverPort_);
}

// Run the token server.

int
Token_Server::run (void)
{
  // Set the acceptor endpoint into listen mode.
  if (this->acceptor_.open (this->addr_, &this->reactor_) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%n: %p on port %d\n", 
		      "acceptor::open failed", 
		      this->addr_.get_port_number ()), -1);

  // Register the acceptor with the ACE_Reactor.
  else if (this->reactor_.register_handler 
	   (&this->acceptor_, ACE_Event_Handler::READ_MASK) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%n: %p\n", 
		      "register_handler (acceptor)"), -1);

  // Register ourselves to receive SIGINT so we can shutdown gracefully.
  else if (this->reactor_.register_handler (SIGINT, this) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%n: %p\n", 
		      "register_handler (SIGINT)"), -1);

  // Ignore SIPPIPE so that each Token_Handler can handle this on its own.
  ACE_OS::signal (SIGPIPE, SIG_IGN);

  ACE_DEBUG ((LM_DEBUG, "starting up server test at port %d on handle %d\n", 
	     this->serverPort_, this->acceptor_.get_handle ()));

  // Loop until we receive SIGINT.

  while (this->done_ == 0)
    this->reactor_.handle_events ();

  ACE_DEBUG ((LM_DEBUG, "shutting down server test\n"));
}

// Signal the server to shutdown gracefully.

int 
Token_Server::handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
)
{
  ACE_DEBUG ((LM_DEBUG, "%S\n", signum));
  this->done_ = 1;
  return 0;
}

int
Token_Server::parse_args (int argc, char *argv[])
{
  ACE_LOG_MSG->open (argv[0]);

  ACE_Get_Opt get_opt (argc, argv, "p:", 1);

  for (int c; (c = get_opt ()) != -1; )
    {
      switch (c)
	{
	case 'p':
	  this->serverPort_ = ACE_OS::atoi (get_opt.optarg);
	  break;
	default:
	  ACE_ERROR_RETURN ((LM_ERROR, 
			    "%n:\n[-p server-port]\n%a", 1),
			   -1);
	  break;
	}
    }
  return 0;
}

