/* -*- C++ -*- */

// ============================================================================
//
// = LIBRARY
//    ACE
// 
// = FILENAME
//    Token_Server.h
//
// = DESCRIPTION 
//    Main class that integrates all the components of the ACE_Token Server.
//
// = AUTHOR
//    Douglas C. Schmidt (schmidt@cs.wustl.edu)
// 
// ============================================================================

#if !defined (_TOKEN_SERVER_H)
#define _TOKEN_SERVER_H

#include "Token_Acceptor.h"

class Token_Server : public ACE_Event_Handler
  // = TITLE
  //   Integrate the various components that implement the ACE_Token Server.
  //
  // = DESCRIPTION
  //   The <Token_Server> listens to a port number.  When a connection arrives,
  //   the <Token_Acceptor> creates a new <Token_Handler> (one per connection).
  //   The <Token_Handler> processes all requests from a client to acquire, 
  //   renew, release, and remove a token.  The <Token_Server> shuts down 
  //   gracefully on <SIGINT>.
{
public:
  Token_Server (int argc, char *argv[]);
  // Initialize the test server.

  int run (void);
  // Run the test server.

private:
  virtual int handle_signal (int signum
#if defined (ACE_HAS_SIGINFO_T)
, siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */  
);
  // Handle SIGINT.

  int parse_args (int argc, char *argv[]);
  // Parse the command line and set the options.

  u_short serverPort_;
  // Port number for the server.

  sig_atomic_t done_;
  // Used to shut down the server gracefully when a <SIGINT> occurs.

  ACE_INET_Addr addr_;
  // Address that the <Token_Server> uses to listen for connections.
  
  Token_Acceptor acceptor_;
  // Factory that accepts connections from clients and creates <TokenHandlers>.

  ACE_Reactor reactor_;
  // Event demultiplexer;
};

#endif /* _TOKEN_SERVER_H */
