#include "ace/Service_Config.h"

class Sig_Handler : public ACE_Event_Handler
{
public:
  Sig_Handler (void);
#if defined (ACE_HAS_SIGINFO_T)
  virtual int handle_signal (int signum, siginfo_t * = 0, ucontext_t * = 0);
#else
  virtual int handle_signal (int signum);
#endif /* ACE_HAS_SIGINFO_T */
  sig_atomic_t done (void);
  virtual int handle_input (ACE_HANDLE fd);
  virtual int handle_close (ACE_HANDLE, ACE_Reactor_Mask);

private:
  sig_atomic_t finished;
} sig_handler;

Sig_Handler::Sig_Handler (void): finished (0)
{
}

int 
Sig_Handler::handle_signal (int
#if defined (ACE_HAS_SIGINFO_T)
			    , siginfo_t *, ucontext_t *
#endif /* ACE_HAS_SIGINFO_T */
)
{ 
  this->finished = 1;
  return 0; 
}

sig_atomic_t
Sig_Handler::done (void) 
{ 
  return this->finished; 
}

int 
Sig_Handler::handle_input (ACE_HANDLE fd)
{
  char buf[BUFSIZ];
  int n;
  if ((n = ACE_OS::read (fd, buf, sizeof buf)) == -1)
    return -1;
  else if (ACE_OS::write (1, buf, n) != n)
    return -1;
  else
    return 0;
}

int 
Sig_Handler::handle_close (ACE_HANDLE, ACE_Reactor_Mask)
{
  ACE_DEBUG ((LM_DEBUG, "closing Sig_Handler\n"));
  return 0;
}

int
main (int argc, char *argv[])
{
  ACE_Service_Config loggerd;

  if (loggerd.open (argc, argv) == -1 && errno != ENOENT)
    ACE_ERROR ((LM_ERROR, "%p\n%a", "open", 1));

  ACE_Service_Config::reactor ()->register_handler (SIGINT, &sig_handler);
  ACE_Service_Config::reactor ()->register_handler (0, &sig_handler,
						    ACE_Event_Handler::READ_MASK);
  
  /* Run forever, performing logging service */

  while (sig_handler.done () == 0)
    loggerd.run_event_loop ();

  return 0;
}
