// Test program for the TLI_SAP supplier generator API
// (you must terminate this program by typing ^C...) */

#include "ace/Log_Msg.h"
#include "ace/Thread_Manager.h"
#include "ace/TLI_Connector.h"
#include "ace/Signal.h"

#if defined (ACE_HAS_THREADS)

/* Global thread manager. */
static ACE_Thread_Manager thr_manager;

/* Port number to use. */
static unsigned short port_number = ACE_DEFAULT_SERVER_PORT + 1;

/* Name of remote host. */
static char *host_name = ACE_DEFAULT_SERVER_HOST;

/* ACE_TLI endpoint */
static ACE_TLI_Stream sc;

static void
handle_sigint (int)
{
  if (sc.close () == -1)
    ACE_OS::t_error ("close"), ACE_OS::exit (1);
  ACE_OS::exit (0);
}

void
parse_args (int argc, char *argv[])
{
  extern char *optarg;
  extern int  optind;
  int c;

  while ((c = ACE_OS::getopt (argc, argv, "h:p:")) != -1)
    switch (c)
      {
      case 'h':
	host_name = optarg;
	break;
      case 'p':
	port_number = ACE_OS::atoi (optarg);
	break;
      default:
	ACE_ERROR ((LM_ERROR, "usage: %n [-p portnum] [-h host_name]\n%a", 1));
	break;
      }
}

static void *
supplier_writer (ACE_TLI_Stream *stream)
{
  ACE_Thread_Control tc (&thr_manager);
  char	   buf[BUFSIZ];
  int	   n;

  while ((n = ACE_OS::read (0, buf, sizeof buf)) > 0)
    {
      if (stream->send_n (buf, n) != n)
	return (void *) -1;
    }

  return 0;
}

static void *
supplier_reader (ACE_TLI_Stream *stream)
{
  ACE_Thread_Control tc (&thr_manager);
  char	   buf[BUFSIZ];
  int	   n;

  while ((n = stream->recv (buf, sizeof buf)) > 0)
    if (ACE_OS::write (1, buf, n) != n)
      return (void *) -1;
  
  return 0;
}

int 
main (int argc, char *argv[]) 
{ 
  ACE_LOG_MSG->open (argv[0]);
  parse_args (argc, argv);

  ACE_TLI_Connector con;

  ACE_Sig_Action sig ((ACE_SignalHandler) handle_sigint, SIGINT);

  if (con.connect (sc, ACE_INET_Addr (port_number, host_name)) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", host_name), -1);

  if (thr_manager.spawn (ACE_THR_FUNC (supplier_writer), (void *) &sc, 
			 THR_BOUND | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "writer"), -1);

  if (thr_manager.spawn (ACE_THR_FUNC (supplier_reader), (void *) &sc, 
			 THR_BOUND | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "reader"), -1);

  thr_manager.wait ();
  return 0;
}
#else
int 
main (void)
{
  ACE_OS::fprintf (stderr, "test not defined for this platform\n");
  return 0;
}
#endif /* ACE_HAS_THREADS */
