// implementation of class TTY_IO

#include "tty_io.h"

// Interface for reading/writing serial device parameters

int 
TTY_IO::control (Control_Mode cmd, 
		 Serial_Params *arg) const
{
#if defined (ACE_HAS_TERM_IOCTLS)
  struct termios devpar;
  u_long c_iflag;
  u_long c_oflag;
  u_long c_cflag;
  u_long c_lflag;
  u_long c_line;
  u_char ivmin_cc4;
  u_char ivtime_cc5;
 
  c_iflag=0;
  c_oflag=0;
  c_cflag=0;
  c_lflag=0;
  c_line=0;

  /* get default device parameters */
 
  if (this->ACE_IO_SAP::control (TCGETS, (void *) &devpar) == -1)
    return -1;

  switch (cmd)
    {
    case SETPARAMS:
      switch (arg->baudrate)
	{
	case   600:
	  c_cflag |= B600;
	  break;
	case  1200:
	  c_cflag |= B1200;
	  break;
	case  2400:
	  c_cflag |= B2400;
	  break;
	case  4800:
	  c_cflag |= B4800;
	  break;
	case  9600:
	  c_cflag |= B9600;
	  break;
	case 19200:
	  c_cflag |= B19200;
	  break;
	
	case 38400:
	  c_cflag |= B38400;
	  break;
	default:  
	  return -1;
	}
      switch (arg->databits)
	{
	case   5:
	  c_cflag |= CS5;
	  break;
	case   6: 
	  c_cflag |= CS6;
	  break;
	case   7: 
	  c_cflag |= CS7;
	  break;
	case   8: 
	  c_cflag |= CS8;
	  break;
	default:  
	  return -1;
	}
      switch (arg->stopbits)
	{
	case   1:
	  break;
	case   2:
	  c_cflag |= CSTOPB;
	  break;
	default:  
	  return -1;
	}
      if (arg->parityenb)
	{
	  c_cflag |= PARENB;
	  if (strcmp((char *) arg->paritymode,"ODD")==0 ||
	      strcmp((char *) arg->paritymode,"odd")==0)
	    c_cflag |= PARODD;
	}                 
      if (arg->ctsenb) /* enable CTS/RTS protocoll */
	c_cflag |= CRTSCTS;
      if (arg->rcvenb) /* enable receiver */
	c_cflag |= CREAD;
 
      c_oflag=0;
      c_iflag=IGNPAR|INPCK|ISTRIP;
      c_lflag=0;

      ivmin_cc4 =(u_char)0;
      ivtime_cc5=(u_char)(arg->readtimeoutmsec/100); 
      devpar.c_iflag = c_iflag;
      devpar.c_oflag = c_oflag;
      devpar.c_cflag = c_cflag;
      devpar.c_lflag = c_lflag;
      devpar.c_cc[4] = ivmin_cc4;
      devpar.c_cc[5] = ivtime_cc5;
      
      return(this->ACE_IO_SAP::control(TCSETS,(void *)&devpar));

    case GETPARAMS:
      return -1;	// not yet implemented

    default:
      return -1;	// wrong cmd
    }
#else
  return -1;
#endif /* ACE_HAS_TERM_IOCTLS */
}

TTY_IO::operator ACE_DEV_IO &()
{
  return (ACE_DEV_IO &) *this;
}  

