// class definitions for tty specific features
// This class represents an example interface for a specific
// device (a serial line) It extends the capability of the
// underlying DEV_IO class by adding a control method
// that takes a special structure (Serial_Params) as argument
// to allow a comfortable user interface (away from that
// annoying termios structure, which is very specific to UNIX).

#include "ace/Log_Msg.h"
#include "ace/OS.h"
#include "ace/DEV_Addr.h"
#include "ace/DEV_Connector.h"
#include "ace/DEV_IO.h"

class TTY_IO : public ACE_DEV_IO
{
public:
  enum Control_Mode
  {
    SETPARAMS,	// set control parameters
    GETPARAMS   // get control parameters
  };

  struct Serial_Params
  {
    int baudrate;
    int parityenb;
    char *paritymode;
    int databits;
    int stopbits;
    int readtimeoutmsec;
    int ctsenb;
    int rcvenb;
  };

  int control (Control_Mode cmd, Serial_Params * arg) const;
  // Interface for reading/writing serial device parameters

  operator ACE_DEV_IO &();
  // this is necessary to pass TTY_IO as parameter to DEV_Connector

};
