/* This is a simple test of the ACE_SOCK_Acceptor class. */
#include "ace/Log_Msg.h"
#include "ace/SOCK_Acceptor.h"                             
#include "ace/SOCK_Stream.h"
#include "ace/INET_Addr.h"

/* ACE_SOCK Server */

int 
main (int argc, char *argv[])
{                                                                
  u_short port = argc > 1 ? ACE_OS::atoi (argv[1]) : ACE_DEFAULT_SERVER_PORT;
  ACE_Time_Value timeout (argc > 2 ? ACE_OS::atoi (argv[2]) : ACE_DEFAULT_TIMEOUT);

  ACE_SOCK_Acceptor peer_acceptor;

  /* Create a server address. */
  ACE_INET_Addr server_addr (port);

  /* Create a server, reuse the address */
  if (peer_acceptor.open (server_addr, 1) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "open"), 1);

  /* Keep these guys out here to prevent excessive constructor
     calls... */
  ACE_SOCK_Stream new_stream;                                   
  ACE_INET_Addr    cli_addr;

  ACE_DEBUG ((LM_DEBUG, "starting server at port %d\n", 
	     server_addr.get_port_number ()));

  /* Performs the iterative server activities */

  for (;;) 
    {
      char buf[BUFSIZ];                                          
                                                                     
      /* Create a new ACE_SOCK_Stream endpoint (note
         automatic restart if errno == EINTR) */

      if (peer_acceptor.accept (new_stream, &cli_addr, &timeout) == -1)
	{
	  ACE_ERROR ((LM_ERROR, "%p\n", "accept"));
	  continue;
	}          
                                                                 
      ACE_DEBUG ((LM_DEBUG, "client %s connected from %d\n", 
		 cli_addr.get_host_name (), cli_addr.get_port_number ()));

      /* Read data from client (terminate on error) */

      for (int r_bytes; 
	   (r_bytes = new_stream.recv (buf, sizeof buf)) > 0; )
        if (ACE::send_n (1, buf, r_bytes) != r_bytes)
	  ACE_ERROR ((LM_ERROR, "%p\n", "ACE::send_n"));

      if (new_stream.send_n ("", 1) != 1)
	ACE_ERROR ((LM_ERROR, "%p\n", "send_n"));

      /* Close new endpoint (listening endpoint stays open) */
      if (new_stream.close () == -1) 
	ACE_ERROR ((LM_ERROR, "%p\n", "close"));
    }

  /* NOTREACHED */
  return 0;
}
