// example for using Local ACE_Thread Communication
// Author : Gerhard Lenzer

#include "ace/Log_Msg.h"
#include "ace/Stream.h"
#include "ace/UPIPE_Acceptor.h"
#include "ace/UPIPE_Connector.h"

#if defined (ACE_HAS_THREADS)

// Global thread manager.
ACE_Thread_Manager thr_mgr;

//ACE_UPIPE_Manager ltc_mgr;

// Global pattern
static ACE_UPIPE_Addr addr ("pattern");

// consumer thread; 

static void *
consumer (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.
  ACE_UPIPE_Stream c_stream;
  ACE_UPIPE_Addr c_addr ("pattern");
  ACE_OS::sleep (5);  // give server time to register pattern
  cout << "consumer starting connect" << endl;
  ACE_UPIPE_Connector con;

  if (con.connect (c_stream, addr) == -1)
    cout << "consumer ACE_UPIPE_Connector failed" << endl;
  else
    cout << "consumer handle is " << c_stream.get_handle () << endl;

  ACE_Message_Block *mb = new ACE_Message_Block (BUFSIZ);
  ACE_OS::strcpy (mb->rd_ptr (),"hello");
  mb->wr_ptr (6);
  if (c_stream.put (mb, 0) == -1)
    cout << "error consumer put" << endl;
  

  if (c_stream.get (mb, 0) == -1)
    cout << "error reader get" << endl;
  cout << "read back : " << mb->rd_ptr () << endl; 


  // now try the send recv interface
  char mytext[] = "This string is sent by consumer as buffer";
  if (c_stream.send (mytext, sizeof mytext) == -1)
    cout << "buffer send from consumer failed" << endl;

  char conbuf[30];  // buffer to receive response
  char c = ' ';

  cout << " consumer received buffer with :" << endl;

  while (c != '!')
    {
      if (c_stream.recv (&c,1) == -1)
	cout << "buffer recv from consumer failed" << endl;
      else
	cout << c;
    }

  cout << endl;

  return 0;
}

static void *
server (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.
  ACE_UPIPE_Stream s_stream;
  ACE_UPIPE_Addr serv_addr ("pattern");
  ACE_UPIPE_Acceptor acc (addr);

  cout << "server starting accept" << endl;
  if (acc.accept (s_stream) == -1)
    cout << "ACE_UPIPE_Acceptor.accept failed" << endl;

  // Allocate a new message.
  ACE_Message_Block *mb = new ACE_Message_Block (BUFSIZ);

  if (s_stream.get (mb, 0) == -1)
    cout << "error server get" << endl;
  cout << "server read  : " << mb->rd_ptr () << endl; 

  ACE_OS::strcpy (mb->rd_ptr (),"thanks");
  mb->wr_ptr (7);
  if (s_stream.put (mb, 0) == -1)
     cout << "error server put" << endl;

  char s_buf[42];
  if (s_stream.recv ( s_buf, sizeof s_buf) == -1)
    cout << "server receive buffer failed" << endl;
  else
  {
    cout << "server received buffer with : " << s_buf << endl;
  }
  ACE_OS::strcpy (s_buf, "this is the server response!");
  if (s_stream.send ( s_buf, 30) == -1)
    cout << "server send buffer failed" << endl;
  return 0;
}

int 
main (int, char *[])
{
  // spawn the two threads
  if (thr_mgr.spawn (ACE_THR_FUNC (consumer), (void *) 0,
			 THR_NEW_LWP | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "spawn"), 1);
  else if (thr_mgr.spawn (ACE_THR_FUNC (server), (void *) 0,
			      THR_NEW_LWP | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "spawn"), 1);

  // Wait for producer and consumer threads to exit.
  thr_mgr.wait ();
 
  return 0;
}
#else
int 
main (int, char *[])
{
  ACE_ERROR_RETURN ((LM_ERROR, "threads not supported on this platform\n"), -1);
}
#endif /* ACE_HAS_THREADS */


