// example for using Local ACE_Thread Communication
// Author : Gerhard Lenzer

#include <fstream.h>
#include "ace/Log_Msg.h"
#include "ace/Synch.h"
#include "ace/Stream.h"
#include "ace/UPIPE_Addr.h"
#include "ace/UPIPE_Connector.h"
#include "ace/UPIPE_Acceptor.h"

#if defined (ACE_HAS_THREADS)

//typedef ACE_UPIPE_Stream ACE_UPIPE_Stream;

// Global thread manager.
ACE_Thread_Manager thr_mgr;

// Global pattern
static ACE_UPIPE_Addr addr("pattern");

// data for testsuite
int size, iterations, acc_sleep, con_sleep;

static void *
consumer (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.
  ACE_UPIPE_Stream c_stream;

  ACE_UPIPE_Addr c_addr ("pattern");

  ACE_OS::sleep (con_sleep);  // give server time to register pattern

  ACE_Message_Block *mb_p;

  char *mybuf = new char[size];

  for (int i =0; i<size; i++)
    mybuf[i] = 'a';

  ACE_Message_Block *mblast = new ACE_Message_Block (10);

  cout << "connector starting connect" << endl;

  ACE_UPIPE_Connector con;

  if (con.connect (c_stream, addr) == -1)
    cout << "consumer ACE_UPIPE_Connector failed" << endl;
  cout << "connection ok" << endl;

  // just for test of asynchronisity
  c_stream.enable (SIGIO);

  for (i=0; i < iterations; i++)
  {
    mb_p = new ACE_Message_Block (size, ACE_Message_Block::MB_DATA, 
				  (ACE_Message_Block *) 0, mybuf);
    if (c_stream.send_msg (mb_p, 0) == -1)
    {
      cout << "error put" << endl;
      return 0;
    }
  }
  mblast->size (0);
  if (c_stream.send_msg (mblast, 0) == -1)
    {
      cout << "error put" << endl;
      return 0;
    }
  
  thr_mgr.wait ();
  delete mybuf;
  return 0;
}

static void *
server (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.

  ACE_UPIPE_Stream s_stream;

  // set the high water mark to size
  // to achieve optimum performance

  int wm = size * iterations;

  if (s_stream.control (ACE_IO_Cntl_Msg::SET_HWM, &wm) == -1)
    cout << "set HWM failed" << endl;

  ACE_UPIPE_Addr serv_addr ("pattern");

  ACE_UPIPE_Acceptor acc (serv_addr);  // accept will wait up to 4 seconds

  ACE_OS::sleep (acc_sleep);
  cout << "server starting accept" << endl;
  if (acc.accept (s_stream) == -1)
    cout << "ACE_UPIPE_Acceptor.accept failed" << endl;

  ACE_Message_Block *mb = 0;

  // time measurement
  time_t currsec;
  ACE_OS::time (&currsec);
  int start = (int) currsec;
  int received_messages = 0;

  while (s_stream.recv_msg (mb, 0) == 0)
    {
      if (mb->size () == 0)
	break;
      else
	{
	  received_messages++;
	  delete mb;
	}

    }

  ACE_OS::time (&currsec);
  int secs = (int) currsec - start;
  cout << "transferred " << received_messages << " blocks of size " << size << endl;
  cout << "the program ran " << secs << " seconds" << endl;
 
  return 0;
}

int 
main (int argc, char *argv[])
{
  size = argc > 1 ? ACE_OS::atoi (argv[1]) : 32;
  iterations = argc > 2 ? ACE_OS::atoi (argv[2]) : 16;
  acc_sleep = argc > 3 ? ACE_OS::atoi (argv[3]) : 0;
  con_sleep = argc > 4 ? ACE_OS::atoi (argv[4]) : 1;

  // spawn the two threads
  if (thr_mgr.spawn (ACE_THR_FUNC (server), (void *) 0,
		     THR_NEW_LWP | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "spawn"), 1);

  consumer ( (void *) 0);

  // Wait for producer and consumer threads to exit.  thr_mgr.wait ();
 
  return 0;
}

#else
int 
main (int, char *[])
{
  ACE_ERROR ( (LM_ERROR, "threads not supported on this platform\n"));
  return 0;
}
#endif /* ACE_HAS_THREADS */


