// example for using Local ACE_Thread Communication
// Author : Gerhard Lenzer
// this example uses char buffers as input/output
// interface to the ACE_UPIPE_Stream

#include <fstream.h>
#include "ace/Log_Msg.h"
#include "ace/Synch.h"
#include "ace/Stream.h"
#include "ace/UPIPE_Addr.h"
#include "ace/UPIPE_Connector.h"
#include "ace/UPIPE_Acceptor.h"
#include <sys/time.h>

#if defined (ACE_HAS_THREADS)

//typedef ACE_UPIPE_Stream ACE_UPIPE_Stream;

// Global thread manager.
ACE_Thread_Manager thr_mgr;

// Global pattern
static ACE_UPIPE_Addr addr("pattern");

// data for testsuite
int size, iterations, acc_sleep, con_sleep;

static void *
consumer (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.
  ACE_UPIPE_Stream c_stream;
  ACE_UPIPE_Addr c_addr ("pattern");

  ACE_OS::sleep (con_sleep);  // give server time to register pattern

  cout << "consumer starting connect" << endl;

  ACE_UPIPE_Connector con;
  
  if (con.connect (c_stream, addr) == -1)
    cout << "consumer ACE_UPIPE_Connector failed" << endl;

  char *mybuf = new char[size];

  for (int i =0; i<size; i++)
    mybuf[i] = 'a';

  for (i=0; i < iterations; i++)
    if (c_stream.send (mybuf, size) == -1)
      {
	cout << "error put" << endl;
	return 0;
      }
  
  delete mybuf;
  thr_mgr.wait ();
  return 0;
}

static void *
server (void * dummy)
{
  ACE_Thread_Control thread_control (&thr_mgr); // Insert thread into thr_mgr.
  ACE_UPIPE_Stream s_stream;
  ACE_UPIPE_Addr serv_addr ("pattern");
  ACE_UPIPE_Acceptor acc (addr);  // accept will wait up to 4 seconds

  ACE_OS::sleep (acc_sleep);
  cout << "server starting accept" << endl;

  if (acc.accept (s_stream) == -1)
    cout << "ACE_UPIPE_Acceptor.accept failed" << endl;

  char * mybuf = new char[size];
  time_t currsec;

  ACE_OS::time (&currsec);

  int start = (int) currsec;

  for (int i =0; i<iterations; i++)
    if (s_stream.recv (mybuf, size) == -1)
      cout << "recv failed" << endl;

  ACE_OS::time (&currsec);
  int secs = (int) currsec - start;
  cout << "transferred " << iterations << " blocks of size " << size << endl;
  cout << "the program ran " << secs << " seconds" << endl;
  delete mybuf;
  return 0;
}

int 
main (int argc, char *argv[])
{
  size = argc > 1 ? ACE_OS::atoi (argv[1]) : 32;
  iterations = argc > 2 ? ACE_OS::atoi (argv[2]) : 16;
  acc_sleep = argc > 3 ? ACE_OS::atoi (argv[3]) : 0;
  con_sleep = argc > 4 ? ACE_OS::atoi (argv[4]) : 1;

  // spawn the thread
  if (thr_mgr.spawn (ACE_THR_FUNC (server), (void *) 0,
			      THR_NEW_LWP | THR_DETACHED) == -1)
    ACE_ERROR_RETURN ( (LM_ERROR, "%p\n", "spawn"), 1);

  consumer ((void *) 0);
 
  return 0;
}
#else
int 
main (int, char *[])
{
  ACE_ERROR ( (LM_ERROR, "threads not supported on this platform\n"));
  return 0;
}
#endif /* ACE_HAS_THREADS */


