/* -*- C++ -*- */

#include "ace/Get_Opt.h"

INLINE
Handle_L_CODgram::Handle_L_CODgram (void)
{
}

INLINE int
Handle_L_CODgram::open (const ACE_UNIX_Addr &suad, int async)
{
  if (this->ACE_LSOCK_CODgram::open (ACE_Addr::sap_any, suad) == -1)
    return -1;
  else if (async && this->ACE_LSOCK_CODgram::enable (SIGIO) == -1)
    return -1;
  else
    return 0;
}

INLINE int 
Handle_L_CODgram::info (char **strp, size_t length) const
{
  char      buf[BUFSIZ];
  ACE_UNIX_Addr sa;

  if (ACE_LSOCK_CODgram::get_local_addr (sa) == -1)
    return -1;
  
  ACE_OS::sprintf (buf, "%s %s", sa.get_path_name (), "# tests local connected datagram\n");

  if (*strp == 0 && (*strp = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*strp, buf, length);
  return ACE_OS::strlen (buf);
}

INLINE int
Handle_L_CODgram::init (int argc, char *argv[])
{
  ACE_UNIX_Addr  sucd;
  ACE_Get_Opt    get_opt (argc, argv, "r:", 0);
  const char *r = Handle_L_CODgram::DEFAULT_RENDEZVOUS;

  for (int c; (c = get_opt ()) != -1; )
     switch (c)
       {
       case 'r': 
	 r = get_opt.optarg;
	 break;
       default:
	 break;
       }

  ACE_OS::strncpy (this->rendezvous, r, MAXPATHLEN);
  ACE_OS::unlink (this->rendezvous);
  sucd.set (this->rendezvous);
  if (this->open (sucd) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "open"), -1);
  else if (ACE_Service_Config::reactor ()->register_handler (this,
						      ACE_Event_Handler::READ_MASK) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "registering service with ACE_Reactor\n"), -1);
  return 0;
}

INLINE int 
Handle_L_CODgram::fini(void) 
{
  return ACE_Service_Config::reactor ()->remove_handler (this, ACE_Event_Handler::READ_MASK);
}

INLINE int 
Handle_L_CODgram::get_handle (void) const
{ 
  return ACE_LSOCK_CODgram::get_handle ();
}

INLINE int 
Handle_L_CODgram::handle_input (int)
{
  int  fd = -1;
  char buf[BUFSIZ];

  if (this->recv_handle (fd) == -1)
    return -1;
  else
    ACE_DEBUG ((LM_INFO, "received fd (%d)\n", fd));

  ACE_OS::puts ("----------------------------------------");

  for (int n; n = ACE_OS::read (fd, buf, sizeof buf); )
    ACE_OS::write (1, buf, n);

  ACE_OS::puts ("----------------------------------------");

  if (ACE_OS::close (fd) == -1)
    return -1;

  return 0;
}

INLINE int
Handle_L_CODgram::handle_close (int, ACE_Reactor_Mask)
{
  this->ACE_LSOCK_CODgram::close ();
  return ACE_OS::unlink (this->rendezvous);
}
