/* This test program illustrates how the ACE barrier synchronization
   mechanisms work. */

#include "ace/Log_Msg.h"
#include "ace/Synch.h"
#include "ace/Thread.h"

#if defined (ACE_HAS_THREADS)

// Total number of iterations.
static int n_iterations = 10;

// Iterate <n_iterations> time printing off a message and "waiting"
// for all other threads to complete this iteration.

static void *
tester (ACE_Barrier *barrier)
{
  for (int iterations = 1; iterations <= n_iterations; iterations++)
    {
      ACE_DEBUG ((LM_DEBUG, "(%t) in iteration %d\n", iterations));
      // Block until all other threads have waited, then continue.
      barrier->wait ();
    }
  return 0;
}

// Spawn off n_threads.

static const int DEFAULT_ITERATIONS = 5;

int main (int argc, char *argv[])
{
  int n_threads = argc > 1 ? ACE_OS::atoi (argv[1]) : ACE_DEFAULT_THREADS;
  n_iterations = argc > 2 ? ACE_OS::atoi (argv[2]) : DEFAULT_ITERATIONS;

  ACE_Barrier barrier (n_threads);
  
  if (ACE_Thread::spawn_n (n_threads, ACE_THR_FUNC (tester),
			   (void *) &barrier, THR_NEW_LWP) == n_threads)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "spawn_n"), 1);

#if defined (ACE_HAS_STHREADS)
  for (thread_t thr_id; ACE_Thread::join (0, &thr_id, 0) == 0; )
    ACE_DEBUG ((LM_DEBUG, "reaping %d\n", thr_id));

  ACE_DEBUG ((LM_DEBUG, "exiting main thread (%t)\n"));
#else
  ACE_Thread::exit (0);
#endif /* ACE_HAS_STHREADS */  
  return 0;
}
#else
int 
main (int, char *[])
{
  ACE_ERROR ((LM_ERROR, "threads not supported on this platform\n"));
  return 0;
}
#endif /* ACE_HAS_THREADS */
