#include "ace/OS.h"
#include "ace/Log_Msg.h"
#include "ace/Local_Tokens.h"
#include "ace/Thread.h"
#include "ace/Get_Opt.h"

#if defined (ACE_HAS_THREADS)

static ACE_Local_Mutex *global_mutex;

struct two_mutexes
{
  ACE_Token_Proxy* first_;
  ACE_Token_Proxy* first_release_;
  ACE_Token_Proxy* second_;
};

static int iterations = 100;
static int dual_release = 0;
static int ignore_deadlock = 0;

static void *
run_test (void *vp)
{
  two_mutexes* tm = (two_mutexes*) vp;

  for (int x = 0; x < iterations; x++)
    {
      // test recursive acquisition of a global proxy
      if (tm->first_->acquire () == -1) 
	{
	  ACE_DEBUG ((LM_DEBUG, "(%t) deadlock detected\n"));
	  continue;
	}

      if (tm->second_->acquire () == -1) 
	{
	  ACE_DEBUG ((LM_DEBUG, "(%t) deadlock detected\n"));
	  goto G1;
	}

      tm->second_->release ();
    G1:
      if (dual_release)
	tm->first_release_->release ();
      else
	tm->first_->release ();

    }

  cerr << "thread " << ACE_OS::thr_self () << " exiting." << endl;
  return 0;
}

static void 
wait_for_children (void)
{
  thread_t thread_id;
  void *status = 0;

  /* Wait for ANY one thread to exit */
  while (ACE_Thread::join ((thread_t) 0, &thread_id, &status) == 0)
    ACE_DEBUG ((LM_DEBUG, "(%t) reaping thread %d with status = %d\n", thread_id, status));
}

static int
parse_args (int argc, char *argv[])
{
  ACE_LOG_MSG->open (argv[0]);

  ACE_Get_Opt get_opt (argc, argv, "iun:d", 1);

  for (int c; (c = get_opt ()) != -1; )
    {
      switch (c)
	{
	case 'i':
	  ignore_deadlock = 1;
	  break;
	case 'd':
	  dual_release = 1;
	  break;
	case 'n':
	  iterations = ACE_OS::atoi (get_opt.optarg);
	  break;
	case 'u':
	default:
	  ACE_ERROR_RETURN ((LM_ERROR, 
			    "%n:\n"
			    "[-n <iterations>]\n"
			    "[-d dual releases]\n"
			    "[-i ignore deadlock]\n%a", 1), -1);
	  break;
	}
    }

  return 0;
}

int
main (int argc, char* argv[])
{
  if (parse_args (argc, argv) == -1)
    return -1;

  global_mutex = new ACE_Local_Mutex ("global proxy", ignore_deadlock, 1);

  two_mutexes one, two;
  one.first_ = new ACE_Local_Mutex ("local proxy", ignore_deadlock, 1);
  one.first_release_ = new ACE_Local_Mutex ("local proxy", ignore_deadlock, 1);
  one.second_ = global_mutex;
  two.first_ = global_mutex;
  two.first_release_ = global_mutex;
  two.second_ = one.first_release_;

  if (ACE_Thread::spawn (ACE_THR_FUNC (run_test),
			 (void *) &one, THR_BOUND) != 0)
    ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "first spawn"), -1);

  if (ACE_Thread::spawn (ACE_THR_FUNC (run_test),
			 (void *) &two, THR_BOUND) != 0)
    ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "second spawn"), -1);

  wait_for_children ();

  return 42;
}

#else
int 
main (int, char *[])
{
  ACE_ERROR_RETURN ((LM_ERROR, 
		     "threads not supported on this platform\n"), -1);
}
#endif /* ACE_HAS_THREADS */
