
#include "ace/Thread.h"
#include "ace/Local_Tokens.h"
#include "ace/Log_Msg.h"
#include "ace/OS.h"

#if defined (ACE_HAS_THREADS)

struct two_mutexes
{
  ACE_Token_Proxy* first_;
  ACE_Token_Proxy* second_;
  int sleep_time_;   // ***********
};

void *
run_test (void *vp)
{
  two_mutexes* tm = (two_mutexes*) vp;

  for (int x = 0; x < 100; x++)   // **********
    {
      // test recursive acquisition of a global proxy
      if (tm->first_->acquire () == -1)
	ACE_DEBUG ((LM_DEBUG, "(%t) acquire failed %s\n", tm->first_->client_id ()));
      else
	ACE_DEBUG ((LM_DEBUG, "(%t) acquired %s\n", tm->first_->client_id ()));

//      ACE_OS::sleep(tm->sleep_time_);   // **********

      if (tm->second_->acquire () == -1)
	ACE_DEBUG ((LM_DEBUG, "(%t) acquire failed %s\n", tm->second_->client_id ()));
      else
	ACE_DEBUG ((LM_DEBUG, "(%t) Acquired %s\n", tm->second_->client_id ()));

      tm->second_->release ();
      tm->first_->release ();
    }

  ACE_DEBUG ((LM_DEBUG, "(%t) exiting\n"));
  return 0;
}

static void 
wait_for_children (void)
{
  thread_t thread_id;
  void *status = 0;

  /* Wait for ANY one thread to exit */
  while (ACE_Thread::join ((thread_t) 0, &thread_id, &status) == 0)
    ACE_DEBUG ((LM_DEBUG, "(%t) reaping thread %d with status = %d\n", thread_id, status));
}

int
main (int argc, char* argv[])
{
  thread_t thread_id;

  int ignore_deadlock = 0;
  if (argc > 1)
    ignore_deadlock = 1;

  two_mutexes one, two, three;
  ACE_Local_Mutex R1  ("R1", ignore_deadlock, 1);
  ACE_Local_Mutex R2  ("R2", ignore_deadlock, 1);
  ACE_Local_Mutex R3  ("R3", ignore_deadlock, 1);
  one.first_ = &R1;
  one.second_ = &R2;
  one.sleep_time_ = 10;
  two.first_ = &R2;
  two.second_ = &R3;
  two.sleep_time_ = 6;
  three.first_ = &R3;
  three.second_ = &R1;
  three.sleep_time_ = 2;

  if (ACE_Thread::spawn (ACE_THR_FUNC (run_test),
			 (void *) &one, THR_BOUND) != 0)
    ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "first spawn"), -1);

  if (ACE_Thread::spawn (ACE_THR_FUNC (run_test),
			 (void *) &two, THR_BOUND) != 0)
    ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "second spawn"), -1);

  if (ACE_Thread::spawn (ACE_THR_FUNC (run_test),
			 (void *) &three, THR_BOUND) != 0)
    ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "third spawn"), -1);

  ::wait_for_children ();

  return 42;
}

#else
int 
main (int, char *[])
{
  ACE_ERROR_RETURN ((LM_ERROR, 
		     "threads not supported on this platform\n"), -1);
}
#endif /* ACE_HAS_THREADS */
