#include "ace/OS.h"
#include "ace/Log_Msg.h"
#include "ace/Local_Tokens.h"
#include "ace/Thread.h"
#include "ace/Get_Opt.h"

#if defined (ACE_HAS_THREADS)

static int ignore_deadlock = 0;
static int debug = 0;
static int spawn_count = 2;
static ACE_Local_Mutex global_mutex;

void *
run_test1 (void *)
{
  // test recursive acquisition of a global proxy
  ACE_Guard<ACE_Local_Mutex> m1 (global_mutex);
  ACE_Guard<ACE_Local_Mutex> m2 (global_mutex);
  ACE_DEBUG ((LM_DEBUG, "First time in global %s\n", global_mutex.client_id ()));
  global_mutex.renew ();
  ACE_DEBUG ((LM_DEBUG, "(%t) Second time in global\n"));
  return 0;
}

void *
run_test2 (void*)
{
  ACE_Local_Mutex local_mutex ("local proxy", ignore_deadlock, debug);
  // acquisition of a local proxy
  ACE_Guard<ACE_Local_Mutex> m1 (local_mutex);

  ACE_DEBUG ((LM_DEBUG, "First time in local %s\n", local_mutex.client_id ()));

  if (local_mutex.renew () == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "renew failed"), 0);

  ACE_DEBUG ((LM_DEBUG, "(%t) Second time in local\n"));

  return 0;
}

static void 
wait_for_children (void)
{
  thread_t thread_id;
  void *status = 0;

  /* Wait for ANY one thread to exit */
  while (ACE_Thread::join ((thread_t) 0, &thread_id, &status) == 0)
    ACE_DEBUG ((LM_DEBUG, "(%t) reaping thread %d with status = %d\n", thread_id, status));
}

static int
parse_args (int argc, char *argv[])
{
  ACE_LOG_MSG->open (argv[0]);

  ACE_Get_Opt get_opt (argc, argv, "iudt:", 1);

  for (int c; (c = get_opt ()) != -1; )
    {
      switch (c)
	{
	case 'i':  // specify the host machine on which the server is running
	  ignore_deadlock = 1;
	  break;
	case 'd':
	  debug = 1;
	  break;
	case 't':  // specify the port on which the server is running
	  spawn_count = ACE_OS::atoi (get_opt.optarg);
	  break;
	case 'u':
	default:
	  ACE_ERROR_RETURN ((LM_ERROR, 
			    "%n:\n"
			    "[-d debug]\n"
			    "[-t <number of threads>]\n"
			    "[-i ignore deadlock]\n%a", 1), -1);
	  break;
	}
    }

  return 0;
}

int
main (int argc, char* argv[])
{
  parse_args (argc, argv);
  global_mutex.open ("global proxy", ignore_deadlock, debug);

  for (int x = 0; x < spawn_count; x++)
    if (ACE_Thread::spawn (ACE_THR_FUNC (run_test1),
			   (void *) x, THR_BOUND) != 0)
      ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "spawn"), -1);

  wait_for_children ();

  for (int y = 0; y < spawn_count; y++)
    if (ACE_Thread::spawn (ACE_THR_FUNC (run_test2),
			   (void *) y, THR_BOUND) != 0)
      ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "spawn"), -1);

  wait_for_children ();

  return 42;
}
#else
int main (void)
{
  ACE_ERROR_RETURN ((LM_ERROR, "you must have threads to run this test program\n"), -1);
}
#endif /* ACE_HAS_THREADS */
