#include "ace/OS.h"
#include "ace/Log_Msg.h"
#include "ace/Local_Tokens.h"
#include "ace/Thread.h"

#if defined (ACE_HAS_THREADS)

static ACE_Local_Mutex mutex ("global proxy");

static void *
run_test1 (void *)
{
  // test recursive acquisition of a global proxy
  mutex.acquire ();
  mutex.acquire ();
  ACE_DEBUG ((LM_DEBUG, "First time in global %s\n", mutex.client_id ()));
  mutex.renew ();
  ACE_DEBUG ((LM_DEBUG, "(%t) Second time in global\n"));
  mutex.release ();
  mutex.release ();
  return 0;
}

static void *
run_test2 (void*)
{
  ACE_Local_Mutex mutex2 ("local proxy");

  // acquisition of a local proxy
  if (mutex2.acquire () == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "acquire failed"), 0);

  ACE_DEBUG ((LM_DEBUG, "(%t) First time in local %s\n", mutex2.client_id ()));

  if (mutex2.renew () == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "renew failed"), 0);

  ACE_DEBUG ((LM_DEBUG, "(%t) Second time in local\n"));

  if (mutex2.release () == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "release failed"), 0);
  return 0;
}

static void 
wait_for_children (void)
{
  thread_t thread_id;
  void *status = 0;

  /* Wait for ANY one thread to exit */
  while (ACE_Thread::join ((thread_t) 0, &thread_id, &status) == 0)
    ACE_DEBUG ((LM_DEBUG, "(%t) reaping thread %d with status = %d\n", thread_id, status));
}

int
main (int argc, char* argv[])
{
  int spawn_count = argc == 2 ? ACE_OS::atoi (argv[1]) : 2;

  for (int x = 0; x < spawn_count; x++)
    if (ACE_Thread::spawn (ACE_THR_FUNC (run_test1),
			   (void *) x, THR_BOUND) != 0)
      ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "spawn"), -1);

  wait_for_children ();

  for (int y = 0; y < spawn_count; y++)
    if (ACE_Thread::spawn (ACE_THR_FUNC (run_test2),
			   (void *) y, THR_BOUND) != 0)
      ACE_ERROR_RETURN ((LM_DEBUG, "%p\n", "spawn"), -1);

  wait_for_children ();

  return 42;
}
#else
int main (void)
{
  ACE_ERROR_RETURN ((LM_ERROR, "you must have threads to run this test program\n"), -1);
}
#endif /* ACE_HAS_THREADS */
